//----------------------------------------------------------------------------
//  Project Test_At3dx_Viewer
//  MicroMouse Productions
//  Copyright  2010-2021. All Rights Reserved.
//  Author: Wayne Hogue
//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <stdlib.h>
#include <stdio.h>
#include <dos.h>

#include "at3ddll.h"
#include "Test_At3dx_LocationsApp.h"
#include "at3dx_my_functions.h"

#define COLOR_PANEL_LINE              0
#define COLOR_PANEL_VERTEX1           1
#define COLOR_PANEL_VERTEX2           2
#define COLOR_PANEL_OBSTRUCTION       3
#define COLOR_PANEL_OBSERVER_LINE     4
#define COLOR_PANEL_OBSERVER_VERTEX   5

#define COLOR_PANEL_LINE_ML              6
#define COLOR_PANEL_VERTEX1_ML           7
#define COLOR_PANEL_VERTEX2_ML           8
#define COLOR_PANEL_OBSTRUCTION_ML       9
#define COLOR_PANEL_OBSERVER_LINE_ML    10
#define COLOR_PANEL_OBSERVER_VERTEX_ML  11

#define COLOR_PANEL_SPHERE_MARKER1      12
#define COLOR_PANEL_SPHERE_MARKER2      13
#define COLOR_PANEL_SPHERE_MARKER3      14
#define COLOR_PANEL_SPHERE_MARKER4      15
#define COLOR_PANEL_SPHERE_MARKER5      16
#define COLOR_PANEL_SPHERE_MARKER6      17
#define COLOR_PANEL_SPHERE_MARKER7      18

#define COLOR_PANEL_MANY_VIEWS_BORDER   19

// --

#define DISPLAY_RUBBER_CLEAR_ALL       -1
//#define DISPLAY_RUBBER_MOUSE_HOVER      0
//#define DISPLAY_RUBBER_MOUSE_DOWN       1
//#define DISPLAY_RUBBER_MOUSE_MOVE       2
#define DISPLAY_RUBBER_MOUSE_UP         3
#define DISPLAY_RUBBER_MOUSE_SWAP       4
#define DISPLAY_RUBBER_MOUSE_CLIP       5
#define DISPLAY_RUBBER_OBSERVER_ADD     6
#define DISPLAY_RUBBER_OBSERVER_CANCEL  7

// used when adding static mesh or clone mesh
//
#define ROTATE_ON_X_AXIS_NO            0
#define ROTATE_ON_X_AXIS_MINUS_90      1
#define ROTATE_ON_X_AXIS_PLUS_90       2
#define ROTATE_ON_X_AXIS_180           3

//---------------------------------------------------------------------------

// To be able to send XYZ locations for where the mouse is clicked on a mesh
// the DLL has to have a message number to  use in the Windows OS PostMessage(...)
// this number must be >= WM_USER.  See Windows documentation for more
// information on the PostMessage(...) function
// Select a message number that does not conflict with any other message numbers
// that your application may be using
//
#define WM_MESH_POSITION    WM_USER + 7

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTest_At3dx_LocationsForm *Test_At3dx_LocationsForm;
//---------------------------------------------------------------------------

__fastcall TTest_At3dx_LocationsForm::TTest_At3dx_LocationsForm(TComponent* Owner)
  : TForm(Owner)
{
  sender_W8057_Flag = 0;
  shift_W8057_Flag  = 0;
  done_W8057_Flag   = 0;

  spare1 = 0;

  dllOpen    = 0;
  fileLoaded = 0;

  my_memset(inName, 0, sizeof(char) * MAX_PATH);

  inputFileType = 2;

  // Display textures
  //
  useTexturesFlag = 1;

  // default setting for camera cursor
  //
  alwaysShowCameraCursorFlag = 0;

  // if set, touch screen is available
  //
  touchScreenInputFlag = 0;

  // --

  rgbLineArray[0] = 1.0f;
  rgbLineArray[1] = 0.0f;
  rgbLineArray[2] = 0.0f;

  rgbPoint1Array[0] = 1.0f;
  rgbPoint1Array[1] = 0.0f;
  rgbPoint1Array[2] = 1.0f;

  rgbPoint2Array[0] = 0.0f;
  rgbPoint2Array[1] = 1.0f;
  rgbPoint2Array[2] = 1.0f;

  rgbObstructionArray[0] = 1.0f;
  rgbObstructionArray[1] = 1.0f;
  rgbObstructionArray[2] = 0.0f;

  rgbLineObserverArray[0] = 0.0f;
  rgbLineObserverArray[1] = 0.0f;
  rgbLineObserverArray[2] = 1.0f;

  rgbPointObserverArray[0] = 0.0f;
  rgbPointObserverArray[1] = 1.0f;
  rgbPointObserverArray[2] = 0.0f;

  lineWidth     = 2.0f;
  observerWidth = 2.0f;

  pointSize = 4.0f;
  pointObstructionSize = 4.0f;
  observerSize = 4.0f;

  depthTestFlag = 1;
  pointsFlag    = 1;

  obstructionDisplayFlag     = 1;
  observerLineFlag           = 1;
  observerPointFlag          = 1;
  haveObserverFlag           = 0;
  haveObstructionMessageFlag = 0;

  // --

  screenX0 = 0;
  screenX1 = 0;
  screenX2 = 0;
  screenXObstruction = 0;
  screenXObserver    = 0;

  screenY0 = 0;
  screenY1 = 0;
  screenY2 = 0;
  screenYObstruction = 0;
  screenYObserver    = 0;

  hitFlag0 = MESH_LOCATION_NO_DATA;
  hitFlag1 = MESH_LOCATION_NO_DATA;
  hitFlag2 = MESH_LOCATION_NO_DATA;
  hitFlagObstruction = MESH_LOCATION_NO_DATA;
  hitFlagObserver    = MESH_LOCATION_NO_DATA;

  for (Int32 i=0; i<3; i++)
  {
    xyzArray0[i] = 0.0;
    xyzArray1[i] = 0.0;
    xyzArray2[i] = 0.0;
    xyzArrayObstruction[i] = 0.0;
    xyzArrayObserver[i]    = 0.0;

    xyzDeltaArrayObserver[i] = 0.0;
  }

  rubberbandDistance  = 0.0;
  obstructionDistance = 0.0;

  obstructionSkipDistance = 0.0;

  rgbArray0[0] = 0.0f;
  rgbArray0[1] = 0.0f;
  rgbArray0[2] = 0.0f;

  rgbArray1[0] = 0.0f;
  rgbArray1[1] = 0.0f;
  rgbArray1[2] = 0.0f;

  rgbArray2[0] = 0.0f;
  rgbArray2[1] = 0.0f;
  rgbArray2[2] = 0.0f;

  rgbArrayObstruction[0] = 0.0f;
  rgbArrayObstruction[1] = 0.0f;
  rgbArrayObstruction[2] = 0.0f;

  rgbArrayObserver[0] = 0.0f;
  rgbArrayObserver[1] = 0.0f;
  rgbArrayObserver[2] = 0.0f;

  // --

  spare2 = 0;

  cameraXViewpoint = 0.0;
  cameraYViewpoint = 0.0;
  cameraZViewpoint = 0.0;

  cameraFieldOfView = 30.0;
  cameraStepFOV     = 1.0;

  cameraRotateSpeed = CAMERA_ROTATE_SPEED_2;

  cameraXAngle      = 0.0;
  cameraYAngle      = 0.0;
  cameraZAngle      = 0.0;
  cameraStepAngle   = 5.0;

  cameraXPan    = 0;
  cameraZPan    = 0;
  cameraStepPan = 1;

  reversePanDirectionFlag  = 0;

  reverseZoomDirectionFlag = 0;

  cameraZoomSpeed = CAMERA_ZOOM_SPEED_6;

  roamingZoomLock = ROAMING_LOCK_ZOOM_BOTH;

  zoomAutomaticFlag = 1;

  cameraTotal         = 0;
  activeCameraID      = 0;
  cameraID            = 0;
  getFromCameraID     = 0;
  topLeftCameraID     = 0;
  topRightCameraID    = 0;
  bottomLeftCameraID  = 0;
  bottomRightCameraID = 0;

  roaming_asDisplayed_Flag = 0;

  cameraInitialDistance    = 0.0;
  cameraDistance           = 0.0;

  xRotation_Lock_Flag = 0;
  yRotation_Lock_Flag = 0;
  zRotation_Lock_Flag = 0;

  xRotation_Constrain_Flag = 0;
  yRotation_Constrain_Flag = 0;
  zRotation_Constrain_Flag = 0;

  xRotation_Constrain_From = 0.0;
  xRotation_Constrain_To   = 0.0;

  yRotation_Constrain_From = 0.0;
  yRotation_Constrain_To   = 0.0;

  zRotation_Constrain_From = 0.0;
  zRotation_Constrain_To   = 0.0;

  for (Int32 i=0; i<3; i++)
  {
    roaming_Eye[i]    = 0.0;
    roaming_Target[i] = 0.0;
  }

  roaming_Up_Rotation = 0.0;
  roaming_FieldOfView = 65.0;

  moveAheadBack_Step = 1.0;
  moveRightLeft_Step = 0.1;
  moveUpDown_Step    = 1.0;
  moveTurn_Step      = 5.0;

  gpu_Max_Texture_Size      = 0;
  user_Max_Texture_Size     = 0;
  powerOf2_Max_Texture_Size = 0;

  spare3 = 0;

  // --

  rgbLineArray_ML[0] = 0.616f;
  rgbLineArray_ML[1] = 0.094f;
  rgbLineArray_ML[2] = 0.067f;

  rgbPoint1Array_ML[0] = 0.714f;
  rgbPoint1Array_ML[1] = 0.047f;
  rgbPoint1Array_ML[2] = 0.714f;

  rgbPoint2Array_ML[0] = 0.0f;
  rgbPoint2Array_ML[1] = 0.616f;
  rgbPoint2Array_ML[2] = 0.616f;

  rgbObstructionArray_ML[0] = 0.733f;
  rgbObstructionArray_ML[1] = 0.733f;
  rgbObstructionArray_ML[2] = 0.0f;

  rgbLineObserverArray_ML[0] = 0.0f;
  rgbLineObserverArray_ML[1] = 0.502f;
  rgbLineObserverArray_ML[2] = 1.0f;

  rgbPointObserverArray_ML[0] = 0.0f;
  rgbPointObserverArray_ML[1] = 0.502f;
  rgbPointObserverArray_ML[2] = 0.0f;

  lineWidth_ML     = 2.0f;
  observerWidth_ML = 2.0f;

  pointSize_ML = 4.0f;
  pointObstructionSize_ML = 4.0f;
  observerSize_ML = 4.0f;

  depthTestFlag_ML = 1;
  pointsFlag_ML    = 1;

  obstructionDisplayFlag_ML = 1;
  observerLineFlag_ML       = 1;
  observerPointFlag_ML      = 1;

#ifdef USING_WIN32
  w32_pad1 = 0;
#endif

  fontColor = 0;
  measureColorFlag = 0;

  measureData_ListPtr = MY_NULL_PTR;

  my_memset(&billboardData, 0, sizeof(MEASURE_BILLBOARD));

  billboardImagePtr    = MY_NULL_PTR;
  billboardImageWidth  = 0;
  billboardImageHeight = 0;
  billboardAlphaFlag   = 0;

  // --

  my_memset(projectFilename, 0, sizeof(char) * MAX_PATH);

  my_memset(measurementsFilename, 0, sizeof(char) * MAX_PATH);

  my_memset(camerasFilename, 0, sizeof(char) * MAX_PATH);

  my_memset(billboardsFilename, 0, sizeof(char) * MAX_PATH);

  my_memset(clonesFilename, 0, sizeof(char) * MAX_PATH);

  my_memset(sphereMarkersFilename, 0, sizeof(char) * MAX_PATH);

  my_memset(decalsFilename, 0, sizeof(char) * MAX_PATH);

  // --

  rgbSphereMarkerArray1[0] = 1.0f;
  rgbSphereMarkerArray1[1] = 1.0f;
  rgbSphereMarkerArray1[2] = 0.0f;

  rgbSphereMarkerArray2[0] = 1.0f;
  rgbSphereMarkerArray2[1] = 1.0f;
  rgbSphereMarkerArray2[2] = 0.0f;

  rgbSphereMarkerArray3[0] = 1.0f;
  rgbSphereMarkerArray3[1] = 1.0f;
  rgbSphereMarkerArray3[2] = 0.0f;

  rgbSphereMarkerArray4[0] = 1.0f;
  rgbSphereMarkerArray4[1] = 1.0f;
  rgbSphereMarkerArray4[2] = 0.0f;

  rgbSphereMarkerArray5[0] = 1.0f;
  rgbSphereMarkerArray5[1] = 1.0f;
  rgbSphereMarkerArray5[2] = 0.0f;

  rgbSphereMarkerArray6[0] = 1.0f;
  rgbSphereMarkerArray6[1] = 1.0f;
  rgbSphereMarkerArray6[2] = 0.0f;

  rgbSphereMarkerArray7[0] = 1.0f;
  rgbSphereMarkerArray7[1] = 1.0f;
  rgbSphereMarkerArray7[2] = 1.0f;

  // --

  spare4 = 0;

  for (Int32 i=0; i<CAMERA_INDEX_TOTAL; i++)
  {
    cameraComboboxPtr[i] = MY_NULL_PTR;
  }

  active_TabPageIndex = 0;

  // --

  spare5 = 0;

  decalTypeNamesPtr[VIEW_DECAL_TYPE_TOP_LEFT]     = "Top_Left";
  decalTypeNamesPtr[VIEW_DECAL_TYPE_TOP_RIGHT]    = "Top_Right";
  decalTypeNamesPtr[VIEW_DECAL_TYPE_BOTTOM_LEFT]  = "Bottom_Left";
  decalTypeNamesPtr[VIEW_DECAL_TYPE_BOTTOM_RIGHT] = "Bottom_Right";
  decalTypeNamesPtr[VIEW_DECAL_TYPE_CENTER]       = "Center";
  decalTypeNamesPtr[VIEW_DECAL_TYPE_ANYWHERE]     = "Anywhere";

  my_memset(decalNew_ImageFilename, 0, sizeof(char) * MAX_PATH);
  my_memset(decalSelected_ImageFilename, 0, sizeof(char) * MAX_PATH);
  my_memset(decalSelected_Name, 0, sizeof(char) * LEN_LAYER_NAME);

  // --

  lightMode = LIGHT_MODE_EXAMINE;

  for (Int32 i=0; i<LIGHT_MODE_COUNT; i++)
  {
    LIGHT_MODE* lightModePtr = &lightModes[i];

    if (i == LIGHT_MODE_EXAMINE)
    {
      lightModePtr->backgroundType = BACKGROUND_DEFAULT;
    }
    else if (i == LIGHT_MODE_SUN_TRACKING)
    {
      lightModePtr->backgroundType = BACKGROUND_BRIGHT;
    }
    else if (i == LIGHT_MODE_NIGHT_GOGGLES)
    {
      lightModePtr->backgroundType = BACKGROUND_DARK;
    }

    lightModePtr->global_ambientRGBA[0] = 1.0f;
    lightModePtr->global_ambientRGBA[1] = 1.0f;
    lightModePtr->global_ambientRGBA[2] = 1.0f;
    lightModePtr->global_ambientRGBA[3] = 1.0f;

    lightModePtr->light0_enabledFlag = 1;

    if (i == LIGHT_MODE_NIGHT_GOGGLES)
    {
      lightModePtr->light0_ambientRGBA[0]  = 0.0f;
      lightModePtr->light0_ambientRGBA[1]  = 1.0f;
      lightModePtr->light0_ambientRGBA[2]  = 0.0f;
      lightModePtr->light0_ambientRGBA[3]  = 1.0f;

      lightModePtr->light0_diffuseRGBA[0] = 0.0f;
      lightModePtr->light0_diffuseRGBA[1] = 1.0f;
      lightModePtr->light0_diffuseRGBA[2] = 0.0f;
      lightModePtr->light0_diffuseRGBA[3] = 1.0f;

      lightModePtr->light0_specularRGBA[0] = 0.0f;
      lightModePtr->light0_specularRGBA[1] = 1.0f;
      lightModePtr->light0_specularRGBA[2] = 0.0f;
      lightModePtr->light0_specularRGBA[3] = 1.0f;
    }
    else
    {
      if (i == LIGHT_MODE_SUN_TRACKING) lightModePtr->light0_staticFlag  = 1;

      lightModePtr->light0_ambientRGBA[0] = 1.0f;
      lightModePtr->light0_ambientRGBA[1] = 1.0f;
      lightModePtr->light0_ambientRGBA[2] = 1.0f;
      lightModePtr->light0_ambientRGBA[3] = 1.0f;

      lightModePtr->light0_diffuseRGBA[0] = 1.0f;
      lightModePtr->light0_diffuseRGBA[1] = 1.0f;
      lightModePtr->light0_diffuseRGBA[2] = 1.0f;
      lightModePtr->light0_diffuseRGBA[3] = 1.0f;

      lightModePtr->light0_specularRGBA[0] = 1.0f;
      lightModePtr->light0_specularRGBA[1] = 1.0f;
      lightModePtr->light0_specularRGBA[2] = 1.0f;
      lightModePtr->light0_specularRGBA[3] = 1.0f;
    }

    lightModePtr->light0_position[0] = 0.0f;
    lightModePtr->light0_position[1] = 0.0f;
    lightModePtr->light0_position[2] = 1.0f;

    lightModePtr->light0_ambientPercent  =   0.0f;
    lightModePtr->light0_diffusePercent  = 100.0f;
    lightModePtr->light0_specularPercent =   0.0f;
  }

  // --

  Form_Setup();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Form_Setup(void)
{
#if defined(_WIN64)
  #ifdef _DEBUG
    const char* cptr = " (x64-Debug)";
  #else
    const char* cptr = " (x64)";
  #endif
#else
  #ifdef _DEBUG
    const char* cptr = " (x32-Debug)";
  #else
    const char* cptr = " (x32)";
  #endif
#endif

  Caption = Caption + cptr;

  Application->OnMessage = AppMessage;
  Application->OnIdle    = MyIdleHandler;

  // --

  try
  {
    measureData_ListPtr = new TList;
  }
  catch (...)
  {
    measureData_ListPtr = MY_NULL_PTR;
  }

  // clear all location data on display
  //
  Display_Rubberband_Data(DISPLAY_RUBBER_CLEAR_ALL);

  // set intial file type to read
  //
  if (!inputFileType)
  {
    TypeObj->Checked = 1;
  }
  else if (inputFileType == 1)
  {
    TypeWrl->Checked = 1;
  }
  else
  {
    TypeAtm->Checked = 1;
  }

  // --

  Start_DLL();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::FormDestroy(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Clear_measureData_List();
  if (measureData_ListPtr) delete measureData_ListPtr;

  if (billboardData.imagePtr) delete[] billboardData.imagePtr;

  if (billboardImagePtr) delete[] billboardImagePtr;

  // free all memory used by the DLL
  //
  if (dllOpen) Stop_DLL();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AppMessage(MSG &Msg, bool &Handled)
{
  if ( (Msg.message == WM_MESH_POSITION) &&
       (dllOpen) )
  {
    if (Msg.lParam == 1)
    {
      if (Msg.wParam == 3)
      {
        // event: left button up
        //
        Display_Rubberband_Data(DISPLAY_RUBBER_MOUSE_UP);
      }
      else if ( (Msg.wParam > 999) &&
                (Msg.wParam < 2000) )
      {
        // measurement line has been selected
        //
        MEASURE_DATA data;

        Int32 result = Get_MeasureList_Item( static_cast<Int32>(Msg.wParam) - 1000, &data );

        Display_Measure_Data(result, &data);
      }
      else if ( (Msg.wParam > 1999) &&
                (Msg.wParam < 3000) )
      {
        // measurement billboard has been selected
        //
        char buffer[LEN_LAYER_NAME];

        if (At3d_Measure_Billboard_Get_ID(static_cast<Int32>(Msg.wParam) - 2000, buffer) > 0)
        {
          Billboard_Selected_ID->Text = AnsiString(buffer);
        }
      }
      else if (Msg.wParam > 2999)
      {
        // sphere marker has been selected
        //
        char buffer[LEN_LAYER_NAME];

        if (At3d_Sphere_Marker_Get_Name(static_cast<Int32>(Msg.wParam) - 3000, buffer) > 0)
        {
          Selected_SphereMarker_Name->Text = AnsiString(buffer);

          UnicodeString tempString = buffer;
          for (Int32 i=0; i<SphereMarkers_List->Count; i++)
          {
            if (tempString.CompareIC(SphereMarkers_List->Items->Strings[i]) == 0)
            {
              SphereMarkers_List->ItemIndex = i;
              Display_Sphere_Marker_Data(buffer);
              Change_SphereMarker_Name->Text = buffer;
              Change_SphereMarker_Get_DataClick(MY_NULL_PTR);
              break;
            }
          }
        }
      }
    }

    Handled = true;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MyIdleHandler(TObject *Sender, bool &Done)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;
  done_W8057_Flag   = (Done) ? 1 : 0;

  // --

  ZoomAutomatic->Down = zoomAutomaticFlag > 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read3dFileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Screen->Cursor = TCursor(crHourGlass);
  Read_3D_File(0);
  Screen->Cursor = TCursor(crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MultiReadClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Screen->Cursor = TCursor(crHourGlass);
  Read_3D_File(1);
  Screen->Cursor = TCursor(crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Start_DLL(void)
{
  if (dllOpen) return;

  // --

  dllOpen = At3d_Start();

  if (!dllOpen) return;

  // --

  // To detect mouse scroll wheel messages the DLL has to know the
  // handle to the main window of the application which
  // receives all the messages from the Windows Operating System
  //
  //At3d_SetAppHandle(Handle);

  // To be able to send XYZ locations for where the mouse is clicked on a mesh
  // the DLL has to know the handle to the main window of the application which
  // receives all the messages from the Windows Operating System
  // plus a message number to indicate the type of message being sent
  //
  At3d_Init_Mesh_Location(Handle, WM_MESH_POSITION);

  // Set parameters for rubberband line
  //
  Setup_Rubberband_Parameters();

  // Set parameters for obstuction point on rubberband line
  //
  Setup_Obstruction_Point_Parameters();

  // set parameters for observer on rubberband line
  //
  Setup_Observer_Parameters();

  // set deltas for obseerver on rubberband line
  //
  Observer_DeltaX->Text = xyzDeltaArrayObserver[0];
  Observer_DeltaY->Text = xyzDeltaArrayObserver[1];
  Observer_DeltaZ->Text = xyzDeltaArrayObserver[2];

  // Set size of DLL window
  //
  At3d_Size(600, 500);

  // Set position of DLL window
  //
  At3d_Position(16, 26);

  // Make DLL window a child of a window in the application
  //
  At3d_SetParent(ViewPanel->Handle);

  // If the DLL window does not have a parent
  // At3d_SetParent(0);
  // a button will appear on the Windows Taskbar
  // use this function to set the name that will appear
  //
  At3d_SetCaption("DLL (at3d)");

  // Use the OpenGL view window
  //
  At3d_WriteFlag(LIBFLAG_USE_OPENGL, 1);

  // Light both sides of the polygons
  //
  At3d_WriteFlag(LIBFLAG_OGL_2SIDED_LIGHTING, 1);

  // Cull back faces of polygons
  //
  At3d_WriteFlag(LIBFLAG_OGL_CULL_BACKFACES,1);

  UseTextures->Checked = (useTexturesFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_USE_TEXTURES, useTexturesFlag);

  // Display default texture if there is no texture file assigned
  //
  At3d_WriteFlag(LIBFLAG_OGL_USE_DEFAULT_TEXTURE, 1);

  // Let texture colors be affected by the lighting
  //
  At3d_WriteFlag(LIBFLAG_OGL_REPLACE, 0);

  // Do not blend polygon color with the texture colors
  //
  At3d_WriteFlag(LIBFLAG_OGL_WHITE_BASE, 1);

  AlwaysShowCameraCursor->Checked = (alwaysShowCameraCursorFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_ALWAYS_SHOW_CAMERA_CURSOR, alwaysShowCameraCursorFlag);

  TouchScreenInput->Checked = (touchScreenInputFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_TOUCH_SCREEN_INPUT, touchScreenInputFlag);

  // Measuring distances (LIBFLAG_DISPLAY_MESH_LOCATION)
  // between locations on mesh is OFF by default
  //
  Locations_Off->Checked = 1;

  // Measuring mode of selecting vertices is DRAG by default
  //
  Locations_Drag->Checked = 1;

  // --

  CameraXViewpoint->Text = cameraXViewpoint;
  CameraYViewpoint->Text = cameraYViewpoint;
  CameraZViewpoint->Text = cameraZViewpoint;

  CameraFieldOfView->Text = cameraFieldOfView;
  CameraStepFOV->Text     = cameraStepFOV;

  if (cameraRotateSpeed == CAMERA_ROTATE_SPEED_1)
  {
    CameraRotateSpeed->Text = "R1";
  }
  else
  {
    CameraRotateSpeed->Text = "R2";
  }

  CameraXAngle->Text = cameraXAngle;
  CameraYAngle->Text = cameraYAngle;
  CameraZAngle->Text = cameraZAngle;

  CameraStepAngle->Text = cameraStepAngle;

  CameraXPan->Text = cameraXPan;
  CameraZPan->Text = cameraZPan;

  CameraStepPan->Text = cameraStepPan;

  CameraInitialDistance->Text = cameraInitialDistance;
  CameraDistance->Text        = cameraDistance;

  if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_1)
  {
    CameraZoomSpeed->Text = "Z1";
  }
  else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_2)
  {
    CameraZoomSpeed->Text = "Z2";
  }
  else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_3)
  {
    CameraZoomSpeed->Text = "Z3";
  }
  else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_4)
  {
    CameraZoomSpeed->Text = "Z4";
  }
  else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_5)
  {
    CameraZoomSpeed->Text = "Z5";
  }
  else
  {
    CameraZoomSpeed->Text = "Z6";
  }

  At3d_WriteFlag(LIBFLAG_CAMERA_ZOOM_SPEED_AUTOMATIC, zoomAutomaticFlag);

  if (roamingZoomLock == ROAMING_LOCK_ZOOM_NONE)
  {
    Zoom_Lock_None->Checked = 1;
  }
  else if (roamingZoomLock == ROAMING_LOCK_ZOOM_IN)
  {
    Zoom_Lock_In->Checked = 1;
  }
  else if (roamingZoomLock == ROAMING_LOCK_ZOOM_OUT)
  {
    Zoom_Lock_Out->Checked = 1;
  }
  else //if (roamingZoomLock == ROAMING_LOCK_ZOOM_BOTH)
  {
    Zoom_Lock_Both->Checked = 1;
  }

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);

  Set_Camera_Lists();

  Display_Roaming_Parameters();

  Move_AheadBack_Step->Text = moveAheadBack_Step;
  Move_RightLeft_Step->Text = moveRightLeft_Step;
  Move_UpDown_Step->Text    = moveUpDown_Step;
  Move_Turn_Step->Text      = moveTurn_Step;

  // Set parameters for rubberband line, obstruction point and observer
  // for a series of measurements stored in a list
  //
  Setup_Rubberband_Parameters_ML();
  Setup_Obstruction_Point_Parameters_ML();
  Setup_Observer_Parameters_ML();

  // --

  // By default, set viewpoint is set to use middle mouse button
  //
  SetViewpoint_Middle->Checked = 1;

  // By default, left button toggle is set to rotate
  //
  LeftToggle_Rotate->Checked = 1;

  // --

  // get axis rotation parameters
  //
  Get_Axis_XClick(MY_NULL_PTR);
  Get_Axis_YClick(MY_NULL_PTR);
  Get_Axis_ZClick(MY_NULL_PTR);

  // --

  ImageWidth->Text  = 600;
  ImageHeight->Text = 500;

  ImageLineWidth->Text = 1.0;
  ImagePointSize->Text = 1.0;

  // --

  GPUMaxTextureSize->Text       = gpu_Max_Texture_Size;
  UserMaxTextureSize->Text      = user_Max_Texture_Size;
  PowerOf2_MaxTextureSize->Text = powerOf2_Max_Texture_Size;

  // --

  StatusBar_Top->Checked = 1;
  StatusBar_Height->Text = 22;
  At3d_StatusBar_Show( 1, 22 );

  FontColorPanel->Color = static_cast<TColor>(fontColor);
  FontSize->Text = 10;

  measureColorFlag = 0;
  MeasureColor_Default->Checked = 1;

  // --

  DisplayBillboards->Checked = 0;

  BillboardFontColor1->Checked = 1;
  BillboardAlpha->Checked = 1;

  // --

  BillboardPivotTypeList->Items->BeginUpdate();

  try
  {
    BillboardPivotTypeList->Clear();

    BillboardPivotTypeList->Items->Add("Center");
    BillboardPivotTypeList->Items->Add("Bottom Left");
    BillboardPivotTypeList->Items->Add("Bottom Center");
    BillboardPivotTypeList->Items->Add("Bottom Right");
    BillboardPivotTypeList->Items->Add("Top Left");
    BillboardPivotTypeList->Items->Add("Top Center");
    BillboardPivotTypeList->Items->Add("Top Right");
    BillboardPivotTypeList->Items->Add("Left Center");
    BillboardPivotTypeList->Items->Add("Right Center");

    BillboardPivotTypeList->ItemIndex = 1;

    BillboardPivotTypeList->Items->EndUpdate();
  }
  catch(...)
  {
    BillboardPivotTypeList->Items->EndUpdate();
  }

  // --

  BillboardStatic->Checked = 0;
  BillboardVisible->Checked = 1;
  BillboardOglReplace->Checked = 1;

  BillboardType->Text = 0;

  BillboardLineWidth->Text = 1.0;

  BillboardHeight->Text = 1.0;

  BillboardPositionX->Text = 0.0;
  BillboardPositionY->Text = 0.0;
  BillboardPositionZ->Text = 0.0;

  BillboardPivotX->Text = 0.0;
  BillboardPivotY->Text = 0.0;
  BillboardPivotZ->Text = 0.0;

  BillboardRotateX->Text = 0.0;
  BillboardRotateY->Text = 0.0;
  BillboardRotateZ->Text = 0.0;

  Billboard_Selected_Red->Text   = 1.0;
  Billboard_Selected_Green->Text = 1.0;
  Billboard_Selected_Blue->Text  = 1.0;

  Billboard_Selected_Width->Text = 4.0;

  Billboard_Line_Red->Text   = 0.0;
  Billboard_Line_Green->Text = 0.0;
  Billboard_Line_Blue->Text  = 0.0;

  Billboard_Line_Width->Text = 0.0;

  // --

  Billboard_PivotTypeList->Items->BeginUpdate();

  try
  {
    Billboard_PivotTypeList->Clear();

    Billboard_PivotTypeList->Items->Add("Center");
    Billboard_PivotTypeList->Items->Add("Bottom Left");
    Billboard_PivotTypeList->Items->Add("Bottom Center");
    Billboard_PivotTypeList->Items->Add("Bottom Right");
    Billboard_PivotTypeList->Items->Add("Top Left");
    Billboard_PivotTypeList->Items->Add("Top Center");
    Billboard_PivotTypeList->Items->Add("Top Right");
    Billboard_PivotTypeList->Items->Add("Left Center");
    Billboard_PivotTypeList->Items->Add("Right Center");

    Billboard_PivotTypeList->ItemIndex = -1;

    Billboard_PivotTypeList->Items->EndUpdate();
  }
  catch(...)
  {
    Billboard_PivotTypeList->Items->EndUpdate();
  }

  // --

  Get_ScreenX->Text = 0;
  Get_ScreenY->Text = 0;

  Get_MeshX->Caption = "";
  Get_MeshY->Caption = "";
  Get_MeshZ->Caption = "";

  Get_LayerName->Caption = "";
  Get_CloneItemName->Caption = "";

  Get_Red1->Text   = "";
  Get_Green1->Text = "";
  Get_Blue1->Text  = "";

  Get_Red2->Text   = "";
  Get_Green2->Text = "";
  Get_Blue2->Text  = "";

  // --

  LineMesh_V1_X->Text = 0.0;
  LineMesh_V1_Y->Text = 0.0;
  LineMesh_V1_Z->Text = 0.0;

  LineMesh_V2_X->Text = 0.0;
  LineMesh_V2_Y->Text = 0.0;
  LineMesh_V2_Z->Text = 0.0;

  LineMesh_SkipDistance->Text = 0.0;

  LineMesh_Intersection_X->Text = 0.0;
  LineMesh_Intersection_Y->Text = 0.0;
  LineMesh_Intersection_Z->Text = 0.0;

  // --

  ReadAddPositionX->Text = 0.0;
  ReadAddPositionY->Text = 0.0;
  ReadAddPositionZ->Text = 0.0;

  ReadAddScaleFactor->Text = 1.0;

  ReadAddRotateX_No->Checked = 1;

  AdjustMesh_PivotX->Text = 0.0;
  AdjustMesh_PivotY->Text = 0.0;
  AdjustMesh_PivotZ->Text = 0.0;

  AdjustMesh_RotateX->Text = 0.0;
  AdjustMesh_RotateY->Text = 0.0;
  AdjustMesh_RotateZ->Text = 0.0;

  AdjustMesh_ScaleX->Text = 1.0;
  AdjustMesh_ScaleY->Text = 1.0;
  AdjustMesh_ScaleZ->Text = 1.0;

  AdjustMesh_TranslateX->Text = 0.0;
  AdjustMesh_TranslateY->Text = 0.0;
  AdjustMesh_TranslateZ->Text = 0.0;

  CloneMesh_SizeX->Text = 0.0;
  CloneMesh_SizeY->Text = 0.0;
  CloneMesh_SizeZ->Text = 0.0;

  CloneMesh_Old_ScaleFactor->Text = 1.0;
  CloneMesh_New_ScaleFactor->Text = 1.0;

  CloneMesh_RotateX_Minus90->Checked = 1;

  // --

  Add_CloneMesh_ScaleFactor->Text = 1.0;

  Add_CloneMesh_RotateX_No->Checked = 1;

  Add_CloneItem_LocationX->Text = 0.0;
  Add_CloneItem_LocationY->Text = 0.0;
  Add_CloneItem_LocationZ->Text = 0.0;

  Add_CloneItem_RotationX->Text = 0.0;
  Add_CloneItem_RotationY->Text = 0.0;
  Add_CloneItem_RotationZ->Text = 0.0;

  Add_CloneItem_ScaleFactor->Text = 1.0;

  CloneItem_RotateX->Text = 0.0;
  CloneItem_RotateY->Text = 0.0;
  CloneItem_RotateZ->Text = 0.0;

  CloneItem_TranslateX->Text = 0.0;
  CloneItem_TranslateY->Text = 0.0;
  CloneItem_TranslateZ->Text = 0.0;

  Data_CloneItem_LocationX->Text = 0.0;
  Data_CloneItem_LocationY->Text = 0.0;
  Data_CloneItem_LocationZ->Text = 0.0;

  Data_CloneItem_RotationX->Text = 0.0;
  Data_CloneItem_RotationY->Text = 0.0;
  Data_CloneItem_RotationZ->Text = 0.0;

  Data_CloneItem_ScaleFactor->Text = 1.0;

  Snap_CloneItem_FromX->Text = 0.0;
  Snap_CloneItem_FromY->Text = 0.0;
  Snap_CloneItem_FromZ->Text = 0.0;

  Snap_CloneItem_ToX->Text = 0.0;
  Snap_CloneItem_ToY->Text = 0.0;
  Snap_CloneItem_ToZ->Text = 0.0;

  Size_CloneItem_OriginalX->Text = 0.0;
  Size_CloneItem_OriginalY->Text = 0.0;
  Size_CloneItem_OriginalZ->Text = 0.0;

  Size_CloneItem_CurrentX->Text = 0.0;
  Size_CloneItem_CurrentY->Text = 0.0;
  Size_CloneItem_CurrentZ->Text = 0.0;

  Size_CloneItem_Old_ScaleFactor->Text = 1.0;

  Size_CloneItem_New_ScaleFactor->Text = 1.0;

  // --

  Int32 cloneVisibleFlag = At3d_ReadFlag(LIBFLAG_DISPLAY_CLONES);
  DisplayClones->Checked = (cloneVisibleFlag) ? 1 : 0;

  // --

  Add_SphereMarker_LocationX->Text = 0.0;
  Add_SphereMarker_LocationY->Text = 0.0;
  Add_SphereMarker_LocationZ->Text = 0.0;

  Add_SphereMarker_Diameter->Text = 1.0;

  Add_SphereMarker_Interval->Text = 1000;

  Do_RGB_Panel(COLOR_PANEL_SPHERE_MARKER1);
  Do_RGB_Panel(COLOR_PANEL_SPHERE_MARKER2);
  Do_RGB_Panel(COLOR_PANEL_SPHERE_MARKER3);
  Do_RGB_Panel(COLOR_PANEL_SPHERE_MARKER4);
  Do_RGB_Panel(COLOR_PANEL_SPHERE_MARKER5);
  Do_RGB_Panel(COLOR_PANEL_SPHERE_MARKER6);
  Do_RGB_Panel(COLOR_PANEL_SPHERE_MARKER7);

  DisplaySphereMarkers->Checked = 0;

  Data_SphereMarker_LocationX->Text = 0.0;
  Data_SphereMarker_LocationY->Text = 0.0;
  Data_SphereMarker_LocationZ->Text = 0.0;

  Data_SphereMarker_Diameter->Text = 1.0;

  Data_SphereMarker_Interval->Text = 1000;

  Change_SphereMarker_LocationX->Text = 0.0;
  Change_SphereMarker_LocationY->Text = 0.0;
  Change_SphereMarker_LocationZ->Text = 0.0;

  Change_SphereMarker_Diameter->Text = 1.0;

  Change_SphereMarker_Interval->Text = 1000;

  Timer_SphereMarker_Interval->Text = 1000;

  // --

  Do_RGB_Panel(COLOR_PANEL_MANY_VIEWS_BORDER);

  // --

  CameraRotateDegrees->Text = 0.5;

  At3d_WriteFlag(LIBFLAG_OGL_2SIDED_LIGHTING, 0);

  At3d_WriteFlag(LIBFLAG_OGL_CULL_BACKFACES, 0);

  // --

  Many_Views_Setup();

  // --

  Decal_New_Name->Text = "";
  Decal_New_Size->Text = 15.0f;
  Decal_New_Use_Frame->Checked = 0;
  Decal_New_Rotate_Angle->Text = 0.0f;
  Decal_New_Visible->Checked = 0;
  Decal_New_Screen_X->Text = 0.0f;
  Decal_New_Screen_Y->Text = 0.0f;
  Decal_New_Image_Filename->Text = "";

  Decal_Selected_Name->Text = "";
  Decal_Selected_Size->Text = 15.0f;
  Decal_Selected_Use_Frame->Checked = 0;
  Decal_Selected_Rotate_Angle->Text = 0.0f;
  Decal_Selected_Visible->Checked = 0;
  Decal_Selected_Screen_X->Text = 0.0f;
  Decal_Selected_Screen_Y->Text = 0.0f;
  Decal_Selected_Image_Filename->Text = "";

  for (Int32 i=0; i<VIEW_DECAL_TYPE_TOTAL; i++)
  {
    Decal_New_Type->Items->Add(decalTypeNamesPtr[i]);

    Decal_Selected_Type->Items->Add(decalTypeNamesPtr[i]);
  }

  Decal_New_Type->ItemIndex      = 0;
  Decal_Selected_Type->ItemIndex = 0;

  decalSelected_Name[0] = '\0';

  // --

  Background_Color_Default->Checked = 1;

  Lighting_Mode_Examine->Checked = 1;
  Switch_Lighting_Mode(LIGHT_MODE_EXAMINE);

  // Make DLL window visible
  //
  At3d_Show();

  // set tab sheet
  //
  PageControl1->ActivePage = LocationsSheet;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Setup_Rubberband_Parameters()
{
  if (!dllOpen) return;

  // --

  Do_RGB_Panel(COLOR_PANEL_LINE);
  Do_RGB_Panel(COLOR_PANEL_VERTEX1);
  Do_RGB_Panel(COLOR_PANEL_VERTEX2);

  LineWidth->Text = static_cast<double>(lineWidth);
  PointSize->Text = static_cast<double>(pointSize);

  UseDepthTest->Checked = (depthTestFlag) ? 1 : 0;

  DisplayVertices->Checked = (pointsFlag) ? 1 : 0;

  At3d_Set_Mesh_Location_Parameters(&rgbLineArray[0], &rgbPoint1Array[0],
                                    &rgbPoint2Array[0], lineWidth, pointSize,
                                    depthTestFlag, pointsFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Setup_Obstruction_Point_Parameters(void)
{
  if (!dllOpen) return;

  // --

  Do_RGB_Panel(COLOR_PANEL_OBSTRUCTION);

  ObstructionPointSize->Text = static_cast<double>(pointObstructionSize);

  DisplayObstructionVertex->Checked = (obstructionDisplayFlag) ? 1 : 0;

  ObstructionSkipDistance->Text = obstructionSkipDistance;

  At3d_Set_Mesh_Obstruction_Parameters(rgbObstructionArray, pointObstructionSize,
                                       obstructionDisplayFlag, obstructionSkipDistance);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Setup_Observer_Parameters(void)
{
  if (!dllOpen) return;

  // --

  Do_RGB_Panel(COLOR_PANEL_OBSERVER_LINE);
  Do_RGB_Panel(COLOR_PANEL_OBSERVER_VERTEX);

  ObserverLineWidth->Text = static_cast<double>(observerWidth);
  ObserverPointSize->Text = static_cast<double>(observerSize);

  DisplayObserverLine->Checked   = (observerLineFlag) ? 1 : 0;
  DisplayObserverVertex->Checked = (observerPointFlag) ? 1 : 0;

  At3d_Set_Mesh_Location_Observer_Parameters( rgbLineObserverArray, rgbPointObserverArray,
                                              observerWidth, observerSize,
                                              observerLineFlag, observerPointFlag );
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Setup_Rubberband_Parameters_ML()
{
  if (!dllOpen) return;

  // --

  Do_RGB_Panel(COLOR_PANEL_LINE_ML);
  Do_RGB_Panel(COLOR_PANEL_VERTEX1_ML);
  Do_RGB_Panel(COLOR_PANEL_VERTEX2_ML);

  LineWidth_ML->Text = static_cast<double>(lineWidth_ML);
  PointSize_ML->Text = static_cast<double>(pointSize_ML);

  UseDepthTest_ML->Checked = (depthTestFlag_ML) ? 1 : 0;

  DisplayVertices_ML->Checked = (pointsFlag_ML) ? 1 : 0;

  At3d_MeasureList_Set_Parameters(&rgbLineArray_ML[0], &rgbPoint1Array_ML[0],
                                  &rgbPoint2Array_ML[0], lineWidth_ML, pointSize_ML,
                                  depthTestFlag_ML, pointsFlag_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Setup_Obstruction_Point_Parameters_ML(void)
{
  if (!dllOpen) return;

  // --

  Do_RGB_Panel(COLOR_PANEL_OBSTRUCTION_ML);

  ObstructionPointSize_ML->Text = static_cast<double>(pointObstructionSize_ML);

  DisplayObstructionVertex_ML->Checked = (obstructionDisplayFlag_ML) ? 1 : 0;

  At3d_MeasureList_Set_Obstruction_Parameters(rgbObstructionArray_ML,
                                              pointObstructionSize_ML,
                                              obstructionDisplayFlag_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Setup_Observer_Parameters_ML(void)
{
  if (!dllOpen) return;

  // --

  Do_RGB_Panel(COLOR_PANEL_OBSERVER_LINE_ML);
  Do_RGB_Panel(COLOR_PANEL_OBSERVER_VERTEX_ML);

  ObserverLineWidth_ML->Text = static_cast<double>(observerWidth_ML);
  ObserverPointSize_ML->Text = static_cast<double>(observerSize_ML);

  DisplayObserverLine_ML->Checked   = (observerLineFlag_ML) ? 1 : 0;
  DisplayObserverVertex_ML->Checked = (observerPointFlag_ML) ? 1 : 0;

  At3d_MeasureList_Set_Observer_Parameters( rgbLineObserverArray_ML,
                                            rgbPointObserverArray_ML,
                                            observerWidth_ML, observerSize_ML,
                                            observerLineFlag_ML, observerPointFlag_ML );
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Stop_DLL(void)
{
  if (!dllOpen) return;

  // --

  // free all memory used by the DLL
  //
  At3d_Stop();
  dllOpen = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read_3D_File(Int32 multiFlag)
{
  if (!dllOpen) return;

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if (inName[0] != '\0')
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ext0);
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr;
  const char* inputExtensionPtr;
  Int32 inputType;

  if (!inputFileType)
  {
    inputNamePtr      = "Wavefront";
    inputExtensionPtr = ".obj";
    inputType         = INTYPE_WAVEFRONT;
  }
  else if (inputFileType == 1)
  {
    inputNamePtr      = "VRML";
    inputExtensionPtr = ".wrl";
    inputType         = INTYPE_VRML;
  }
  else
  {
    inputNamePtr      = "AT3D Mesh";
    inputExtensionPtr = ".atm";
    inputType         = INTYPE_AT3D_MESH;
  }

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  if (multiFlag)
  {
    dlg->Options = dlg->Options << ofAllowMultiSelect;
  }

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;
    Int32 errorFlag = 0;
    Int32 count = 0;
    char** filesList = MY_NULL_PTR;

    if (multiFlag)
    {
      count = dlg->Files->Count;

      try
      {
        filesList = new char*[static_cast<size_t>(count)];
      }
      catch(...)
      {
        filesList = MY_NULL_PTR;
      }

      if (filesList)
      {
        my_memset(filesList, 0, sizeof(char**) * static_cast<size_t>(count));

        for (Int32 i=0; i<count; i++)
        {
          try
          {
            filesList[i] = new char[LEN_FILE_PATH];
          }
          catch(...)
          {
            filesList[i] = MY_NULL_PTR;
            errorFlag++;
            break;
          }
        }

        if (!errorFlag)
        {
          for (Int32 i=0; i<count; i++)
          {
            my_copystring_to_ansi_from_vcl(&fileName, dlg->Files->Strings[i]);
            my_strcpy(filesList[i], MAX_PATH, fileName.c_str());
          }
        }
      }
    }
    else
    {
      my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
      my_strcpy(inName, MAX_PATH, fileName.c_str());
    }

    // before reading make sure flags are set properly
    // (note: flags maintain their setting until changed with this function)
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_VERTICES, 1);
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_FACES, 1);
    At3d_WriteFlag(LIBFLAG_SHOW_READ_MESSAGES, 0);

    // Set to display messages in statusbar for
    // removing duplicate vertices, edges and polygons
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATES_MSG, 1);

    // read file
    //
    if (multiFlag)
    {
      if ( (filesList) && (!errorFlag) )
      {
        fileLoaded = At3d_Multi_Read(inputType, count, LEN_FILE_PATH,
                                      &filesList[0], 1.0);
      }
    }
    else
    {
      fileLoaded = At3d_Read(inputType, inName, 1.0);
    }

    if (fileLoaded)
    {
      Get_All_ActiveCamera_Data();

      // Measuring distances (LIBFLAG_DISPLAY_MESH_LOCATION)
      // between locations on mesh is turned OFF when a file is cleared from
      // memory when a new file is read
      //
      // Had to add the PostMessage(...) functions for the radio buttons
      // to work correctly if Measure, Pause or Continue radio button
      // had been selected before the file was read
      //
      PostMessage(Locations_Off->Handle, WM_LBUTTONDOWN, 0, 0);
      PostMessage(Locations_Off->Handle, WM_LBUTTONUP, 0, 0);
      Locations_Off->Checked = 1;
    }

    if (filesList)
    {
      for (Int32 i=0; i<count; i++)
      {
        if (filesList[i])
        {
          delete[] filesList[i];
        }
      }

      delete[] filesList;
    }
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UseTexturesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  useTexturesFlag = (UseTextures->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_USE_TEXTURES, useTexturesFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AlwaysShowCameraCursorClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  alwaysShowCameraCursorFlag = (AlwaysShowCameraCursor->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_ALWAYS_SHOW_CAMERA_CURSOR, alwaysShowCameraCursorFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_OffClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (Locations_Off->Checked)
  {
    At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_LOCATION, DISPLAY_MESH_LOCATION_OFF);

    // clear all location data on display
    //
    Display_Rubberband_Data(DISPLAY_RUBBER_CLEAR_ALL);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_MeasureClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (Locations_Measure->Checked)
  {
    At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_LOCATION, DISPLAY_MESH_LOCATION_MEASURE);

    UpdateScreenXYClick(MY_NULL_PTR);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_PauseClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (Locations_Pause->Checked)
  {
    At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_LOCATION, DISPLAY_MESH_LOCATION_PAUSE);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_ContinueClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (Locations_Continue->Checked)
  {
    At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_LOCATION, DISPLAY_MESH_LOCATION_CONTINUE);

    UpdateScreenXYClick(MY_NULL_PTR);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Locations0(void)
{
  // now empty of code
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Locations0(void)
{
  Int32 result = At3d_Get_Mesh_Location(0, &hitFlag0, &xyzArray0[0],
                                        &screenX0, &screenY0, &rgbArray0[0]);

  if (!result)
  {
    hitFlag0 = MESH_LOCATION_NO_DATA;
  }
  
  Display_Locations0();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Locations1(void)
{
  if (hitFlag1 == MESH_LOCATION_NO_DATA)
  {
    Location1_ScreenX->Caption = "";
    Location1_ScreenY->Caption = "";

    Location1_MeshX->Caption = "";
    Location1_MeshY->Caption = "";
    Location1_MeshZ->Caption = "";

    Location1_Red1->Text   = "";
    Location1_Green1->Text = "";
    Location1_Blue1->Text  = "";

    Location1_Red2->Text   = "";
    Location1_Green2->Text = "";
    Location1_Blue2->Text  = "";
  }
  else
  {
    Location1_ScreenX->Caption = screenX1;
    Location1_ScreenY->Caption = screenY1;

    if (hitFlag1 == MESH_LOCATION_HAS_XYZ)
    {
      Location1_MeshX->Caption = xyzArray1[0];
      Location1_MeshY->Caption = xyzArray1[1];
      Location1_MeshZ->Caption = xyzArray1[2];

      Location1_Red1->Text   = my_change_float_to_256_int(rgbArray1[0]);
      Location1_Green1->Text = my_change_float_to_256_int(rgbArray1[1]);
      Location1_Blue1->Text  = my_change_float_to_256_int(rgbArray1[2]);

      Location1_Red2->Text   = rgbArray1[0];
      Location1_Green2->Text = rgbArray1[1];
      Location1_Blue2->Text  = rgbArray1[2];

      if (haveObserverFlag)
      {
        Get_LocationsObserver();
      }
    }
    else
    {
      Location1_MeshX->Caption = "";
      Location1_MeshY->Caption = "";
      Location1_MeshZ->Caption = "";

      Location1_Red1->Text = "";
      Location1_Green1->Text = "";
      Location1_Blue1->Text = "";

      Location1_Red2->Text = "";
      Location1_Green2->Text = "";
      Location1_Blue2->Text = "";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Locations1(void)
{
  Int32 result = At3d_Get_Mesh_Location(1, &hitFlag1, &xyzArray1[0],
                                        &screenX1, &screenY1, &rgbArray1[0]);

  if (!result)
  {
    hitFlag1 = MESH_LOCATION_NO_DATA;
  }

  Display_Locations1();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Locations2(void)
{
  if (hitFlag2 == MESH_LOCATION_NO_DATA)
  {
    Location2_ScreenX->Caption = "";
    Location2_ScreenY->Caption = "";

    Location2_MeshX->Caption = "";
    Location2_MeshY->Caption = "";
    Location2_MeshZ->Caption = "";

    Location2_Red1->Text   = "";
    Location2_Green1->Text = "";
    Location2_Blue1->Text  = "";

    Location2_Red2->Text   = "";
    Location2_Green2->Text = "";
    Location2_Blue2->Text  = "";

    Locations_Distance->Caption = "";
  }
  else
  {
    Location2_ScreenX->Caption = screenX2;
    Location2_ScreenY->Caption = screenY2;

    if (hitFlag2 == MESH_LOCATION_HAS_XYZ)
    {
      Location2_MeshX->Caption = xyzArray2[0];
      Location2_MeshY->Caption = xyzArray2[1];
      Location2_MeshZ->Caption = xyzArray2[2];

      Location2_Red1->Text   = my_change_float_to_256_int(rgbArray2[0]);
      Location2_Green1->Text = my_change_float_to_256_int(rgbArray2[1]);
      Location2_Blue1->Text  = my_change_float_to_256_int(rgbArray2[2]);

      Location2_Red2->Text   = rgbArray2[0];
      Location2_Green2->Text = rgbArray2[1];
      Location2_Blue2->Text  = rgbArray2[2];

      Locations_Distance->Caption = rubberbandDistance;
    }
    else
    {
      Location2_MeshX->Caption = "";
      Location2_MeshY->Caption = "";
      Location2_MeshZ->Caption = "";

      Location2_Red1->Text   = "";
      Location2_Green1->Text = "";
      Location2_Blue1->Text  = "";

      Location2_Red2->Text   = "";
      Location2_Green2->Text = "";
      Location2_Blue2->Text  = "";

      Locations_Distance->Caption = "";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Locations2(void)
{
  Int32 result = At3d_Get_Mesh_Location(2, &hitFlag2, &xyzArray2[0],
                                        &screenX2, &screenY2, &rgbArray2[0]);
  if (!result)
  {
    hitFlag2 = MESH_LOCATION_NO_DATA;
  }
  else
  {
    Calculate_Distances();
  }

  Display_Locations2();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_LocationsObstruction(void)
{
  if (hitFlagObstruction == MESH_LOCATION_HAS_XYZ)
  {
    haveObstructionMessageFlag = 0;

    Obstruction_ScreenX->Caption = screenXObstruction;
    Obstruction_ScreenY->Caption = screenYObstruction;

    Obstruction_MeshX->Caption = xyzArrayObstruction[0];
    Obstruction_MeshY->Caption = xyzArrayObstruction[1];
    Obstruction_MeshZ->Caption = xyzArrayObstruction[2];

    Obstruction_Red1->Text   = my_change_float_to_256_int(rgbArrayObstruction[0]);
    Obstruction_Green1->Text = my_change_float_to_256_int(rgbArrayObstruction[1]);
    Obstruction_Blue1->Text  = my_change_float_to_256_int(rgbArrayObstruction[2]);

    Obstruction_Red2->Text   = rgbArrayObstruction[0];
    Obstruction_Green2->Text = rgbArrayObstruction[1];
    Obstruction_Blue2->Text  = rgbArrayObstruction[2];

    Obstruction_Distance->Caption = obstructionDistance;
  }
  else
  {
    if (haveObstructionMessageFlag)
    {
      haveObstructionMessageFlag = 0;
    }
    else
    {
      ObstructionMessage->Caption = "";
    }

    Obstruction_ScreenX->Caption = "";
    Obstruction_ScreenY->Caption = "";

    Obstruction_MeshX->Caption = "";
    Obstruction_MeshY->Caption = "";
    Obstruction_MeshZ->Caption = "";

    Obstruction_Red1->Text   = "";
    Obstruction_Green1->Text = "";
    Obstruction_Blue1->Text  = "";

    Obstruction_Red2->Text   = "";
    Obstruction_Green2->Text = "";
    Obstruction_Blue2->Text  = "";

    Obstruction_Distance->Caption = "";
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_LocationsObstruction(void)
{
  Int32 result = At3d_Get_Mesh_Location(3, &hitFlagObstruction, &xyzArrayObstruction[0],
                                        &screenXObstruction, &screenYObstruction,
                                        &rgbArrayObstruction[0]);

  if ( (result) &&
       (hitFlagObstruction == MESH_LOCATION_HAS_XYZ) )
  {
    Calculate_Distances();
  }
  else
  {
    hitFlagObstruction = MESH_LOCATION_NO_DATA;
  }

  Display_LocationsObstruction();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_LocationsObserver(void)
{
  if (hitFlagObserver == MESH_LOCATION_NO_DATA)
  {
    Observer_ScreenX->Caption = "";
    Observer_ScreenY->Caption = "";

    Observer_MeshX->Caption = "";
    Observer_MeshY->Caption = "";
    Observer_MeshZ->Caption = "";

    Observer_Red1->Text   = "";
    Observer_Green1->Text = "";
    Observer_Blue1->Text  = "";

    Observer_Red2->Text   = "";
    Observer_Green2->Text = "";
    Observer_Blue2->Text  = "";
  }
  else
  {
    Observer_ScreenX->Caption = screenXObserver;
    Observer_ScreenY->Caption = screenYObserver;

    if (hitFlagObserver == MESH_LOCATION_HAS_XYZ)
    {
      Observer_MeshX->Caption = xyzArrayObserver[0];
      Observer_MeshY->Caption = xyzArrayObserver[1];
      Observer_MeshZ->Caption = xyzArrayObserver[2];

      Observer_Red1->Text   = my_change_float_to_256_int(rgbArrayObserver[0]);
      Observer_Green1->Text = my_change_float_to_256_int(rgbArrayObserver[1]);
      Observer_Blue1->Text  = my_change_float_to_256_int(rgbArrayObserver[2]);

      Observer_Red2->Text   = rgbArrayObserver[0];
      Observer_Green2->Text = rgbArrayObserver[1];
      Observer_Blue2->Text  = rgbArrayObserver[2];
    }
    else
    {
      Observer_MeshX->Caption = "";
      Observer_MeshY->Caption = "";
      Observer_MeshZ->Caption = "";

      Observer_Red1->Text   = "";
      Observer_Green1->Text = "";
      Observer_Blue1->Text  = "";

      Observer_Red2->Text   = "";
      Observer_Green2->Text = "";
      Observer_Blue2->Text  = "";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_LocationsObserver(void)
{
  Int32 result = At3d_Get_Mesh_Location(4, &hitFlagObserver, &xyzArrayObserver[0],
                                        &screenXObserver, &screenYObserver,
                                        &rgbArrayObserver[0]);

  if (!result)
  {
    hitFlagObserver = MESH_LOCATION_NO_DATA;
  }
  else
  {
    Calculate_Distances();
  }

  Display_LocationsObserver();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::TouchScreenInputClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  touchScreenInputFlag = (TouchScreenInput->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_TOUCH_SCREEN_INPUT, touchScreenInputFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewFrontClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_ViewFront();
  GetCameraAnglesClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewBackClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_ViewBack();
  GetCameraAnglesClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewRightClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_ViewRight();
  GetCameraAnglesClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewLeftClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_ViewLeft();
  GetCameraAnglesClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewTopClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_ViewTop();
  GetCameraAnglesClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewBottomClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_ViewBottom();
  GetCameraAnglesClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraViewpointClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_GetCameraViewPoint(&cameraXViewpoint,
                                         &cameraYViewpoint,
                                         &cameraZViewpoint);

  if (result)
  {
    CameraXViewpoint->Text = cameraXViewpoint;
    CameraYViewpoint->Text = cameraYViewpoint;
    CameraZViewpoint->Text = cameraZViewpoint;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraViewpointClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  try
  {
    cameraXViewpoint = CameraXViewpoint->Text.ToDouble();
  }
  catch(...)
  {
    cameraXViewpoint = 0.0;
  }

  try
  {
    cameraYViewpoint = CameraYViewpoint->Text.ToDouble();
  }
  catch(...)
  {
    cameraYViewpoint = 0.0;
  }

  try
  {
    cameraZViewpoint = CameraZViewpoint->Text.ToDouble();
  }
  catch(...)
  {
    cameraZViewpoint = 0.0;
  }

  At3d_SetCameraViewPoint(cameraXViewpoint,
                          cameraYViewpoint,
                          cameraZViewpoint);

  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraFieldOfViewClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_GetCameraFieldOfView(&cameraFieldOfView);

  if (result)
  {
    CameraFieldOfView->Text = cameraFieldOfView;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraFieldOfViewClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  try
  {
    cameraFieldOfView = CameraFieldOfView->Text.ToDouble();
  }
  catch(...)
  {
    cameraFieldOfView = 30.0;
  }

  Int32 result = At3d_SetCameraFieldOfView(0, cameraFieldOfView);

  if (result) GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraStepFOV(void)
{
  try
  {
    cameraStepFOV = CameraStepFOV->Text.ToDouble();
  }
  catch(...)
  {
    cameraStepFOV = 1.0;
  }

  CameraStepFOV->Text = cameraStepFOV;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraFOV1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  GetCameraStepFOV();
  At3d_SetCameraFieldOfView(1, -cameraStepFOV);
  GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraFOV2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  GetCameraStepFOV();
  At3d_SetCameraFieldOfView(1, cameraStepFOV);
  GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraRotateSpeed(void)
{
  Int32 result = At3d_GetCameraRotateSpeed(&cameraRotateSpeed);

  if (result)
  {
    if (cameraRotateSpeed == CAMERA_ROTATE_SPEED_1)
    {
      CameraRotateSpeed->Text = "R1";
    }
    else
    {
      CameraRotateSpeed->Text = "R2";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraRotateSpeed1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_SetCameraRotateSpeed(CAMERA_ROTATE_SPEED_1);

  if (result) GetCameraRotateSpeed();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraRotateSpeed2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_SetCameraRotateSpeed(CAMERA_ROTATE_SPEED_2);

  if (result) GetCameraRotateSpeed();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraAnglesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);

  CameraXAngle->Text = cameraXAngle;
  CameraYAngle->Text = cameraYAngle;
  CameraZAngle->Text = cameraZAngle;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraAnglesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  try
  {
    cameraXAngle = CameraXAngle->Text.ToDouble();
  }
  catch(...)
  {
    cameraXAngle = 0.0;
  }

  try
  {
    cameraYAngle = CameraYAngle->Text.ToDouble();
  }
  catch(...)
  {
    cameraYAngle = 0.0;
  }

  try
  {
    cameraZAngle = CameraZAngle->Text.ToDouble();
  }
  catch(...)
  {
    cameraZAngle = 0.0;
  }

  At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);

  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraStepAngle(void)
{
  try
  {
    cameraStepAngle = CameraStepAngle->Text.ToDouble();
  }
  catch(...)
  {
    cameraStepAngle = 1.0;
  }

  CameraStepAngle->Text = cameraStepAngle;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraXAngle1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  GetCameraStepAngle();
  At3d_RotateCamera(1, -cameraStepAngle, 0.0, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraXAngle2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  GetCameraStepAngle();
  At3d_RotateCamera(1, cameraStepAngle, 0.0, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraYAngle1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, -cameraStepAngle, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraYAngle2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, cameraStepAngle, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZAngle1Click(TObject *Sender)
{
   sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, 0.0, -cameraStepAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZAngle2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, 0.0, cameraStepAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZeroCameraAngleXClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);
  cameraXAngle = 0.0;
  At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZeroCameraAngleYClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);
  cameraYAngle = 0.0;
  At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZeroCameraAngleZClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);
  cameraZAngle = 0.0;
  At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZeroCameraAngleXYZClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);
  cameraXAngle = 0.0;
  cameraYAngle = 0.0;
  cameraZAngle = 0.0;
  At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraPanClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  try
  {
    cameraXPan = CameraXPan->Text.ToInt();
  }
  catch(...)
  {
    cameraXPan = 0;
  }

  try
  {
    cameraZPan = CameraZPan->Text.ToInt();
  }
  catch(...)
  {
    cameraZPan = 0;
  }

  At3d_PanCamera(0, cameraXPan, cameraZPan);

  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraStepPan(void)
{
  try
  {
    cameraStepPan = CameraStepPan->Text.ToInt();
  }
  catch(...)
  {
    cameraStepPan = 1;
  }

  CameraStepPan->Text = cameraStepPan;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraXPan1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  GetCameraStepPan();
  At3d_PanCamera(1, -cameraStepPan, 0);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraXPan2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  GetCameraStepPan();
  At3d_PanCamera(1, cameraStepPan, 0);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZPan1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  GetCameraStepPan();
  At3d_PanCamera(1, 0, -cameraStepPan);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZPan2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  GetCameraStepPan();
  At3d_PanCamera(1, 0, cameraStepPan);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewCenterXClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_CenterX();
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewCenterZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_CenterZ();
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReversePanDirectionClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  reversePanDirectionFlag = (ReversePanDirection->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_PAN_DIRECTION, reversePanDirectionFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraInitialDistanceClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_GetCameraInitialDistance(&cameraInitialDistance);

  if (result)
  {
    CameraInitialDistance->Text = cameraInitialDistance;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraInitialDistanceClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  try
  {
    cameraInitialDistance = CameraInitialDistance->Text.ToDouble();
  }
  catch(...)
  {
    cameraInitialDistance = 1.0;
  }

  Int32 result = At3d_SetCameraInitialDistance(cameraInitialDistance);

  if (result) GetCameraInitialDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraDistanceClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_GetCameraDistance(&cameraDistance);

  if (result)
  {
    CameraDistance->Text = cameraDistance;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraDistanceClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  try
  {
    cameraDistance = CameraDistance->Text.ToDouble();
  }
  catch(...)
  {
    cameraDistance = 1.0;
  }

  Int32 result = At3d_SetCameraDistance(cameraDistance);

  if (result) GetCameraDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraZoomSpeedClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_GetCameraZoomSpeed(&cameraZoomSpeed);

  if (result)
  {
    if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_1)
    {
      CameraZoomSpeed->Text = "Z1";
    }
    else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_2)
    {
      CameraZoomSpeed->Text = "Z2";
    }
    else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_3)
    {
      CameraZoomSpeed->Text = "Z3";
    }
    else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_4)
    {
      CameraZoomSpeed->Text = "Z4";
    }
    else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_5)
    {
      CameraZoomSpeed->Text = "Z5";
    }
    else
    {
      CameraZoomSpeed->Text = "Z6";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_1);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_2);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed3Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_3);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed4Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_4);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed5Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_5);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed6Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_6);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZoomAutomaticClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  zoomAutomaticFlag = (zoomAutomaticFlag) ? 0 : 1;

  At3d_WriteFlag(LIBFLAG_CAMERA_ZOOM_SPEED_AUTOMATIC, zoomAutomaticFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZoomInClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_ZoomIn();
  GetCameraDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZoomOutClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_ZoomOut();
  GetCameraDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZoomResetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_ZoomReset();

  Get_All_ActiveCamera_Data();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReverseZoomDirectionClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  reverseZoomDirectionFlag = (ReverseZoomDirection->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_ZOOM_DIRECTION, reverseZoomDirectionFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_All_ActiveCamera_Data(void)
{
  GetCameraViewpointClick(MY_NULL_PTR);

  GetCameraFieldOfViewClick(MY_NULL_PTR);

  GetCameraAnglesClick(MY_NULL_PTR);

  GetCameraInitialDistanceClick(MY_NULL_PTR);

  GetCameraDistanceClick(MY_NULL_PTR);

  GetCameraZoomSpeedClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SwapEndsClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (At3d_Swap_Ends_Mesh_Location() > 0)
  {
    UpdateObstructionClick(MY_NULL_PTR);
    Display_Rubberband_Data(DISPLAY_RUBBER_MOUSE_SWAP);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_RGB_Panel(Int32 index)
{
  TPanel* panelPtr;
  Int32 red;
  Int32 green;
  Int32 blue;

  if (index == COLOR_PANEL_LINE)
  {
    panelPtr = Line_Panel;

    red   = my_change_float_to_256_int(rgbLineArray[0]);
    green = my_change_float_to_256_int(rgbLineArray[1]);
    blue  = my_change_float_to_256_int(rgbLineArray[2]);
  }
  else if (index == COLOR_PANEL_VERTEX1)
  {
    panelPtr = Vertex1_Panel;

    red   = my_change_float_to_256_int(rgbPoint1Array[0]);
    green = my_change_float_to_256_int(rgbPoint1Array[1]);
    blue  = my_change_float_to_256_int(rgbPoint1Array[2]);
  }
  else if (index == COLOR_PANEL_VERTEX2)
  {
    panelPtr = Vertex2_Panel;

    red   = my_change_float_to_256_int(rgbPoint2Array[0]);
    green = my_change_float_to_256_int(rgbPoint2Array[1]);
    blue  = my_change_float_to_256_int(rgbPoint2Array[2]);
  }
  else if (index == COLOR_PANEL_OBSTRUCTION)
  {
    panelPtr = Obstruction_Panel;

    red   = my_change_float_to_256_int(rgbObstructionArray[0]);
    green = my_change_float_to_256_int(rgbObstructionArray[1]);
    blue  = my_change_float_to_256_int(rgbObstructionArray[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE)
  {
    panelPtr = ObserverLine_Panel;

    red   = my_change_float_to_256_int(rgbLineObserverArray[0]);
    green = my_change_float_to_256_int(rgbLineObserverArray[1]);
    blue  = my_change_float_to_256_int(rgbLineObserverArray[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX)
  {
    panelPtr = ObserverVertex_Panel;

    red   = my_change_float_to_256_int(rgbPointObserverArray[0]);
    green = my_change_float_to_256_int(rgbPointObserverArray[1]);
    blue  = my_change_float_to_256_int(rgbPointObserverArray[2]);
  }
  else if (index == COLOR_PANEL_LINE_ML)
  {
    panelPtr = Line_Panel_ML;

    red   = my_change_float_to_256_int(rgbLineArray_ML[0]);
    green = my_change_float_to_256_int(rgbLineArray_ML[1]);
    blue  = my_change_float_to_256_int(rgbLineArray_ML[2]);
  }
  else if (index == COLOR_PANEL_VERTEX1_ML)
  {
    panelPtr = Vertex1_Panel_ML;

    red   = my_change_float_to_256_int(rgbPoint1Array_ML[0]);
    green = my_change_float_to_256_int(rgbPoint1Array_ML[1]);
    blue  = my_change_float_to_256_int(rgbPoint1Array_ML[2]);
  }
  else if (index == COLOR_PANEL_VERTEX2_ML)
  {
    panelPtr = Vertex2_Panel_ML;

    red   = my_change_float_to_256_int(rgbPoint2Array_ML[0]);
    green = my_change_float_to_256_int(rgbPoint2Array_ML[1]);
    blue  = my_change_float_to_256_int(rgbPoint2Array_ML[2]);
  }
  else if (index == COLOR_PANEL_OBSTRUCTION_ML)
  {
    panelPtr = Obstruction_Panel_ML;

    red   = my_change_float_to_256_int(rgbObstructionArray_ML[0]);
    green = my_change_float_to_256_int(rgbObstructionArray_ML[1]);
    blue  = my_change_float_to_256_int(rgbObstructionArray_ML[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE_ML)
  {
    panelPtr = ObserverLine_Panel_ML;

    red   = my_change_float_to_256_int(rgbLineObserverArray_ML[0]);
    green = my_change_float_to_256_int(rgbLineObserverArray_ML[1]);
    blue  = my_change_float_to_256_int(rgbLineObserverArray_ML[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX_ML)
  {
    panelPtr = ObserverVertex_Panel_ML;

    red   = my_change_float_to_256_int(rgbPointObserverArray_ML[0]);
    green = my_change_float_to_256_int(rgbPointObserverArray_ML[1]);
    blue  = my_change_float_to_256_int(rgbPointObserverArray_ML[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER1)
  {
    panelPtr = Add_SphereMarker_Color1;

    red   = my_change_float_to_256_int(rgbSphereMarkerArray1[0]);
    green = my_change_float_to_256_int(rgbSphereMarkerArray1[1]);
    blue  = my_change_float_to_256_int(rgbSphereMarkerArray1[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER2)
  {
    panelPtr = Add_SphereMarker_Color2;

    red   = my_change_float_to_256_int(rgbSphereMarkerArray2[0]);
    green = my_change_float_to_256_int(rgbSphereMarkerArray2[1]);
    blue  = my_change_float_to_256_int(rgbSphereMarkerArray2[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER3)
  {
    panelPtr = Data_SphereMarker_Color1;

    red   = my_change_float_to_256_int(rgbSphereMarkerArray3[0]);
    green = my_change_float_to_256_int(rgbSphereMarkerArray3[1]);
    blue  = my_change_float_to_256_int(rgbSphereMarkerArray3[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER4)
  {
    panelPtr = Data_SphereMarker_Color2;

    red   = my_change_float_to_256_int(rgbSphereMarkerArray4[0]);
    green = my_change_float_to_256_int(rgbSphereMarkerArray4[1]);
    blue  = my_change_float_to_256_int(rgbSphereMarkerArray4[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER5)
  {
    panelPtr = Change_SphereMarker_Color1;

    red   = my_change_float_to_256_int(rgbSphereMarkerArray5[0]);
    green = my_change_float_to_256_int(rgbSphereMarkerArray5[1]);
    blue  = my_change_float_to_256_int(rgbSphereMarkerArray5[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER6)
  {
    panelPtr = Change_SphereMarker_Color2;

    red   = my_change_float_to_256_int(rgbSphereMarkerArray6[0]);
    green = my_change_float_to_256_int(rgbSphereMarkerArray6[1]);
    blue  = my_change_float_to_256_int(rgbSphereMarkerArray6[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER7)
  {
    panelPtr = Selected_SphereMarker_Color;

    red   = my_change_float_to_256_int(rgbSphereMarkerArray7[0]);
    green = my_change_float_to_256_int(rgbSphereMarkerArray7[1]);
    blue  = my_change_float_to_256_int(rgbSphereMarkerArray7[2]);
  }
  else
  {
    return;
  }

  panelPtr->Color = static_cast<TColor>(red + (green << 8) + (blue << 16));
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_ColorDialog(Int32* inColorPtr, Int32* outColorPtr)
{
  if( (!inColorPtr) || (!outColorPtr) ) return;

  // --

  TColorDialog* ColorDlg;
  try
  {
    ColorDlg = new TColorDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    ColorDlg = MY_NULL_PTR;
  }

  if (ColorDlg)
  {
    ColorDlg->Options = ColorDlg->Options << cdFullOpen;
    ColorDlg->Color = static_cast<TColor>((inColorPtr[0] + (inColorPtr[1] << 8) + (inColorPtr[2] << 16)));

    ColorDlg->Execute();

    outColorPtr[0] = static_cast<UInt8>(ColorDlg->Color & 0xFF);
    outColorPtr[1] = static_cast<UInt8>((ColorDlg->Color & 0xFF00) >> 8);
    outColorPtr[2] = static_cast<UInt8>((ColorDlg->Color & 0xFF0000) >> 16);

    delete ColorDlg;
  }
  else
  {
    for (Int32 i=0; i<3; i++)
    {
      outColorPtr[i] = inColorPtr[i];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Pick_Color(Int32 index)
{
  Int32 temp1[3];
  Int32 temp2[3];

  if (index == COLOR_PANEL_LINE)
  {
    temp1[0] = my_change_float_to_256_int(rgbLineArray[0]);
    temp1[1] = my_change_float_to_256_int(rgbLineArray[1]);
    temp1[2] = my_change_float_to_256_int(rgbLineArray[2]);
  }
  else if (index == COLOR_PANEL_VERTEX1)
  {
    temp1[0] = my_change_float_to_256_int(rgbPoint1Array[0]);
    temp1[1] = my_change_float_to_256_int(rgbPoint1Array[1]);
    temp1[2] = my_change_float_to_256_int(rgbPoint1Array[2]);
  }
  else if (index == COLOR_PANEL_VERTEX2)
  {
    temp1[0] = my_change_float_to_256_int(rgbPoint2Array[0]);
    temp1[1] = my_change_float_to_256_int(rgbPoint2Array[1]);
    temp1[2] = my_change_float_to_256_int(rgbPoint2Array[2]);
  }
  else if (index == COLOR_PANEL_OBSTRUCTION)
  {
    temp1[0] = my_change_float_to_256_int(rgbObstructionArray[0]);
    temp1[1] = my_change_float_to_256_int(rgbObstructionArray[1]);
    temp1[2] = my_change_float_to_256_int(rgbObstructionArray[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE)
  {
    temp1[0] = my_change_float_to_256_int(rgbLineObserverArray[0]);
    temp1[1] = my_change_float_to_256_int(rgbLineObserverArray[1]);
    temp1[2] = my_change_float_to_256_int(rgbLineObserverArray[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX)
  {
    temp1[0] = my_change_float_to_256_int(rgbPointObserverArray[0]);
    temp1[1] = my_change_float_to_256_int(rgbPointObserverArray[1]);
    temp1[2] = my_change_float_to_256_int(rgbPointObserverArray[2]);
  }
  else if (index == COLOR_PANEL_LINE_ML)
  {
    temp1[0] = my_change_float_to_256_int(rgbLineArray_ML[0]);
    temp1[1] = my_change_float_to_256_int(rgbLineArray_ML[1]);
    temp1[2] = my_change_float_to_256_int(rgbLineArray_ML[2]);
  }
  else if (index == COLOR_PANEL_VERTEX1_ML)
  {
    temp1[0] = my_change_float_to_256_int(rgbPoint1Array_ML[0]);
    temp1[1] = my_change_float_to_256_int(rgbPoint1Array_ML[1]);
    temp1[2] = my_change_float_to_256_int(rgbPoint1Array_ML[2]);
  }
  else if (index == COLOR_PANEL_VERTEX2_ML)
  {
    temp1[0] = my_change_float_to_256_int(rgbPoint2Array_ML[0]);
    temp1[1] = my_change_float_to_256_int(rgbPoint2Array_ML[1]);
    temp1[2] = my_change_float_to_256_int(rgbPoint2Array_ML[2]);
  }
  else if (index == COLOR_PANEL_OBSTRUCTION_ML)
  {
    temp1[0] = my_change_float_to_256_int(rgbObstructionArray_ML[0]);
    temp1[1] = my_change_float_to_256_int(rgbObstructionArray_ML[1]);
    temp1[2] = my_change_float_to_256_int(rgbObstructionArray_ML[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE_ML)
  {
    temp1[0] = my_change_float_to_256_int(rgbLineObserverArray_ML[0]);
    temp1[1] = my_change_float_to_256_int(rgbLineObserverArray_ML[1]);
    temp1[2] = my_change_float_to_256_int(rgbLineObserverArray_ML[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX_ML)
  {
    temp1[0] = my_change_float_to_256_int(rgbPointObserverArray_ML[0]);
    temp1[1] = my_change_float_to_256_int(rgbPointObserverArray_ML[1]);
    temp1[2] = my_change_float_to_256_int(rgbPointObserverArray_ML[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER1)
  {
    temp1[0] = my_change_float_to_256_int(rgbSphereMarkerArray1[0]);
    temp1[1] = my_change_float_to_256_int(rgbSphereMarkerArray1[1]);
    temp1[2] = my_change_float_to_256_int(rgbSphereMarkerArray1[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER2)
  {
    temp1[0] = my_change_float_to_256_int(rgbSphereMarkerArray2[0]);
    temp1[1] = my_change_float_to_256_int(rgbSphereMarkerArray2[1]);
    temp1[2] = my_change_float_to_256_int(rgbSphereMarkerArray2[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER3)
  {
    temp1[0] = my_change_float_to_256_int(rgbSphereMarkerArray3[0]);
    temp1[1] = my_change_float_to_256_int(rgbSphereMarkerArray3[1]);
    temp1[2] = my_change_float_to_256_int(rgbSphereMarkerArray3[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER4)
  {
    temp1[0] = my_change_float_to_256_int(rgbSphereMarkerArray4[0]);
    temp1[1] = my_change_float_to_256_int(rgbSphereMarkerArray4[1]);
    temp1[2] = my_change_float_to_256_int(rgbSphereMarkerArray4[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER5)
  {
    temp1[0] = my_change_float_to_256_int(rgbSphereMarkerArray5[0]);
    temp1[1] = my_change_float_to_256_int(rgbSphereMarkerArray5[1]);
    temp1[2] = my_change_float_to_256_int(rgbSphereMarkerArray5[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER6)
  {
    temp1[0] = my_change_float_to_256_int(rgbSphereMarkerArray6[0]);
    temp1[1] = my_change_float_to_256_int(rgbSphereMarkerArray6[1]);
    temp1[2] = my_change_float_to_256_int(rgbSphereMarkerArray6[2]);
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER7)
  {
    temp1[0] = my_change_float_to_256_int(rgbSphereMarkerArray7[0]);
    temp1[1] = my_change_float_to_256_int(rgbSphereMarkerArray7[1]);
    temp1[2] = my_change_float_to_256_int(rgbSphereMarkerArray7[2]);
  }
  else
  {
    return;
  }

  // --

  Do_ColorDialog(temp1, temp2);

  // --

  float red   = my_change_256_int_to_float(temp2[0]);
  float green = my_change_256_int_to_float(temp2[1]);
  float blue  = my_change_256_int_to_float(temp2[2]);

  if (index == COLOR_PANEL_LINE)
  {
    rgbLineArray[0] = red;
    rgbLineArray[1] = green;
    rgbLineArray[2] = blue;
  }
  else if (index == COLOR_PANEL_VERTEX1)
  {
    rgbPoint1Array[0] = red;
    rgbPoint1Array[1] = green;
    rgbPoint1Array[2] = blue;
  }
  else if (index == COLOR_PANEL_VERTEX2)
  {
    rgbPoint2Array[0] = red;
    rgbPoint2Array[1] = green;
    rgbPoint2Array[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSTRUCTION)
  {
    rgbObstructionArray[0] = red;
    rgbObstructionArray[1] = green;
    rgbObstructionArray[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE)
  {
    rgbLineObserverArray[0] = red;
    rgbLineObserverArray[1] = green;
    rgbLineObserverArray[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX)
  {
    rgbPointObserverArray[0] = red;
    rgbPointObserverArray[1] = green;
    rgbPointObserverArray[2] = blue;
  }
  else if (index == COLOR_PANEL_LINE_ML)
  {
    rgbLineArray_ML[0] = red;
    rgbLineArray_ML[1] = green;
    rgbLineArray_ML[2] = blue;
  }
  else if (index == COLOR_PANEL_VERTEX1_ML)
  {
    rgbPoint1Array_ML[0] = red;
    rgbPoint1Array_ML[1] = green;
    rgbPoint1Array_ML[2] = blue;
  }
  else if (index == COLOR_PANEL_VERTEX2_ML)
  {
    rgbPoint2Array_ML[0] = red;
    rgbPoint2Array_ML[1] = green;
    rgbPoint2Array_ML[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSTRUCTION_ML)
  {
    rgbObstructionArray_ML[0] = red;
    rgbObstructionArray_ML[1] = green;
    rgbObstructionArray_ML[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE_ML)
  {
    rgbLineObserverArray_ML[0] = red;
    rgbLineObserverArray_ML[1] = green;
    rgbLineObserverArray_ML[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX_ML)
  {
    rgbPointObserverArray_ML[0] = red;
    rgbPointObserverArray_ML[1] = green;
    rgbPointObserverArray_ML[2] = blue;
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER1)
  {
    rgbSphereMarkerArray1[0] = red;
    rgbSphereMarkerArray1[1] = green;
    rgbSphereMarkerArray1[2] = blue;
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER2)
  {
    rgbSphereMarkerArray2[0] = red;
    rgbSphereMarkerArray2[1] = green;
    rgbSphereMarkerArray2[2] = blue;
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER3)
  {
    rgbSphereMarkerArray3[0] = red;
    rgbSphereMarkerArray3[1] = green;
    rgbSphereMarkerArray3[2] = blue;
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER4)
  {
    rgbSphereMarkerArray4[0] = red;
    rgbSphereMarkerArray4[1] = green;
    rgbSphereMarkerArray4[2] = blue;
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER5)
  {
    rgbSphereMarkerArray5[0] = red;
    rgbSphereMarkerArray5[1] = green;
    rgbSphereMarkerArray5[2] = blue;
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER6)
  {
    rgbSphereMarkerArray6[0] = red;
    rgbSphereMarkerArray6[1] = green;
    rgbSphereMarkerArray6[2] = blue;
  }
  else if (index == COLOR_PANEL_SPHERE_MARKER7)
  {
    rgbSphereMarkerArray7[0] = red;
    rgbSphereMarkerArray7[1] = green;
    rgbSphereMarkerArray7[2] = blue;
  }

  // --

  Do_RGB_Panel(index);

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Line_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_LINE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Vertex1_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_VERTEX1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Vertex2_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_VERTEX2);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UseDepthTestClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  depthTestFlag = (UseDepthTest->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayVerticesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  pointsFlag = (DisplayVertices->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetRubberbandParametersClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  try
  {
    lineWidth = my_double_to_float(LineWidth->Text.ToDouble());
  }
  catch(...)
  {
    lineWidth = 1.0f;
  }

  try
  {
    pointSize = my_double_to_float(PointSize->Text.ToDouble());
  }
  catch(...)
  {
    pointSize = 1.0f;
  }

  // --

  At3d_Set_Mesh_Location_Parameters(&rgbLineArray[0], &rgbPoint1Array[0],
                                    &rgbPoint2Array[0], lineWidth, pointSize,
                                    depthTestFlag, pointsFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Obstruction_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSTRUCTION);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObstructionVertexClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  obstructionDisplayFlag = (DisplayObstructionVertex->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetObstructionParametersClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  try
  {
    pointObstructionSize = my_double_to_float(ObstructionPointSize->Text.ToDouble());
  }
  catch(...)
  {
    pointObstructionSize = 1.0f;
  }

  try
  {
    obstructionSkipDistance = ObstructionSkipDistance->Text.ToDouble();
  }
  catch(...)
  {
    obstructionSkipDistance = 0.0;
  }

  // --

  At3d_Set_Mesh_Obstruction_Parameters(rgbObstructionArray, pointObstructionSize,
                                       obstructionDisplayFlag, obstructionSkipDistance);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CheckForObstructionClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result = At3d_Get_Mesh_Obstruction_Point(&xyzArrayObstruction[0],
                                                 &screenXObstruction,
                                                 &screenYObstruction,
                                                 &rgbArrayObstruction[0]);

  if (result < 0)
  {
    hitFlagObstruction = MESH_LOCATION_NO_DATA;
    haveObstructionMessageFlag = 1;
    ObstructionMessage->Caption = "Error";
  }
  else if (!result)
  {
    hitFlagObstruction = MESH_LOCATION_NO_DATA;
    haveObstructionMessageFlag = 1;
    ObstructionMessage->Caption = "No obstruction point";
  }
  else
  {
    hitFlagObstruction = MESH_LOCATION_HAS_XYZ;
    haveObstructionMessageFlag = 1;
    ObstructionMessage->Caption = "Have obstruction point";

    Calculate_Distances();
  }

  Display_LocationsObstruction();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ClipAtObstructionPointClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (At3d_Clip_At_Mesh_Obstruction_Point() > 0)
  {
    Display_Rubberband_Data(DISPLAY_RUBBER_MOUSE_CLIP);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UpdateScreenXYClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 result;

  if (At3d_Update_Mesh_Location_ScreenXY() > 0)
  {
    result = At3d_Get_Mesh_Location(1, &hitFlag1, &xyzArray1[0],
                                    &screenX1, &screenY1, &rgbArray1[0]);

    if ( (result) &&
         (hitFlag1 == MESH_LOCATION_HAS_XYZ) )
    {
      Location1_ScreenX->Caption = screenX1;
      Location1_ScreenY->Caption = screenY1;
    }

    // --

    result = At3d_Get_Mesh_Location(2, &hitFlag2, &xyzArray2[0],
                                    &screenX2, &screenY2, &rgbArray2[0]);

    if ( (result) && (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
    {
      Location2_ScreenX->Caption = screenX2;
      Location2_ScreenY->Caption = screenY2;
    }

    // --

    result = At3d_Get_Mesh_Location(3, &hitFlagObstruction, &xyzArrayObstruction[0],
                                    &screenXObstruction, &screenYObstruction,
                                    &rgbArrayObstruction[0]);

    if ( (result) && (hitFlagObstruction == MESH_LOCATION_HAS_XYZ) )
    {
      Obstruction_ScreenX->Caption = screenXObstruction;
      Obstruction_ScreenY->Caption = screenYObstruction;
    }

    // --

    result = At3d_Get_Mesh_Location(4, &hitFlagObserver, &xyzArrayObserver[0],
                                    &screenXObserver, &screenYObserver,
                                    &rgbArrayObserver[0]);

    if ( (result) && (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      Observer_ScreenX->Caption = screenXObserver;
      Observer_ScreenY->Caption = screenYObserver;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UpdateObstructionClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_LocationsObstruction();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::TypeObjClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  inputFileType = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::TypeWrlClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  inputFileType = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::TypeAtmClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  inputFileType = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetViewpoint_MiddleClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_SET_VIEWPOINT_MOUSE, SET_VIEWPOINT_MOUSE_MIDDLE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetViewpoint_LeftClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_SET_VIEWPOINT_MOUSE, SET_VIEWPOINT_MOUSE_LEFT);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::LeftToggle_RotateClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_LEFT_MOUSE_ROTATE_PAN, LEFT_MOUSE_TOGGLE_ROTATE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::LeftToggle_PanClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_LEFT_MOUSE_ROTATE_PAN, LEFT_MOUSE_TOGGLE_PAN);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Lock_XClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  xRotation_Lock_Flag = (Axis_Lock_X->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Lock_YClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  yRotation_Lock_Flag = (Axis_Lock_Y->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Lock_ZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  zRotation_Lock_Flag = (Axis_Lock_Z->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Constrain_XClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  xRotation_Constrain_Flag = (Axis_Constrain_X->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Constrain_YClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  yRotation_Constrain_Flag = (Axis_Constrain_Y->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Constrain_ZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  zRotation_Constrain_Flag = (Axis_Constrain_Z->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Axis_XClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = CameraID_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CameraID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    At3d_Get_Rotation_Lock(nameBuffer, 0, &xRotation_Lock_Flag, &xRotation_Constrain_Flag,
                           &xRotation_Constrain_From, &xRotation_Constrain_To);

    // --

    Axis_Lock_X->Checked = (xRotation_Lock_Flag) ? 1 : 0;

    Axis_Constrain_X->Checked = (xRotation_Constrain_Flag) ? 1 : 0;

    Constrain_From_X->Text = xRotation_Constrain_From;

    Constrain_To_X->Text = xRotation_Constrain_To;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Axis_XClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = CameraID_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CameraID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    try
    {
      xRotation_Constrain_From = Constrain_From_X->Text.ToDouble();
    }
    catch(...)
    {
      xRotation_Constrain_From = -180.0;
    }

    try
    {
      xRotation_Constrain_To = Constrain_To_X->Text.ToDouble();
    }
    catch(...)
    {
      xRotation_Constrain_To = 180.0;
    }

    // --

    At3d_Set_Rotation_Lock(nameBuffer, 0, xRotation_Lock_Flag, xRotation_Constrain_Flag,
                           xRotation_Constrain_From, xRotation_Constrain_To);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Axis_YClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = CameraID_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CameraID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    At3d_Get_Rotation_Lock(nameBuffer, 1, &yRotation_Lock_Flag, &yRotation_Constrain_Flag,
                           &yRotation_Constrain_From, &yRotation_Constrain_To);

    // --

    Axis_Lock_Y->Checked = (yRotation_Lock_Flag) ? 1 : 0;

    Axis_Constrain_Y->Checked = (yRotation_Constrain_Flag) ? 1 : 0;

    Constrain_From_Y->Text = yRotation_Constrain_From;

    Constrain_To_Y->Text = yRotation_Constrain_To;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Axis_YClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = CameraID_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CameraID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    try
    {
      yRotation_Constrain_From = Constrain_From_Y->Text.ToDouble();
    }
    catch(...)
    {
      yRotation_Constrain_From = -180.0;
    }

    try
    {
      yRotation_Constrain_To = Constrain_To_Y->Text.ToDouble();
    }
    catch(...)
    {
      yRotation_Constrain_To = 180.0;
    }

    // --

    At3d_Set_Rotation_Lock(nameBuffer, 1, yRotation_Lock_Flag, yRotation_Constrain_Flag,
                           yRotation_Constrain_From, yRotation_Constrain_To);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Axis_ZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = CameraID_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CameraID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    At3d_Get_Rotation_Lock(nameBuffer, 2, &zRotation_Lock_Flag, &zRotation_Constrain_Flag,
                           &zRotation_Constrain_From, &zRotation_Constrain_To);

    // --

    Axis_Lock_Z->Checked = (zRotation_Lock_Flag) ? 1 : 0;

    Axis_Constrain_Z->Checked = (zRotation_Constrain_Flag) ? 1 : 0;

    Constrain_From_Z->Text = zRotation_Constrain_From;

    Constrain_To_Z->Text = zRotation_Constrain_To;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Axis_XYZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_Axis_XClick(MY_NULL_PTR);
  Get_Axis_YClick(MY_NULL_PTR);
  Get_Axis_ZClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Axis_ZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = CameraID_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CameraID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    try
    {
      zRotation_Constrain_From = Constrain_From_Z->Text.ToDouble();
    }
    catch(...)
    {
      zRotation_Constrain_From = -180.0;
    }

    try
    {
      zRotation_Constrain_To = Constrain_To_Z->Text.ToDouble();
    }
    catch(...)
    {
      zRotation_Constrain_To = 180.0;
    }

    // --

    At3d_Set_Rotation_Lock(nameBuffer, 2, zRotation_Lock_Flag, zRotation_Constrain_Flag,
                           zRotation_Constrain_From, zRotation_Constrain_To);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Axis_XYZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Set_Axis_XClick(MY_NULL_PTR);
  Set_Axis_YClick(MY_NULL_PTR);
  Set_Axis_ZClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewFromV1ToV2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      roaming_Eye[0] = xyzArrayObserver[0];
      roaming_Eye[1] = xyzArrayObserver[1];
      roaming_Eye[2] = xyzArrayObserver[2];
    }
    else
    {
      roaming_Eye[0] = xyzArray1[0];
      roaming_Eye[1] = xyzArray1[1];
      roaming_Eye[2] = xyzArray1[2];
    }

    roaming_Target[0] = xyzArray2[0];
    roaming_Target[1] = xyzArray2[1];
    roaming_Target[2] = xyzArray2[2];

    roaming_Up_Rotation = 0.0;
    roaming_FieldOfView = 65.0;

    Display_Roaming_Parameters();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewFromV2ToV1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    roaming_Eye[0] = xyzArray2[0];
    roaming_Eye[1] = xyzArray2[1];
    roaming_Eye[2] = xyzArray2[2];

    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      roaming_Target[0] = xyzArrayObserver[0];
      roaming_Target[1] = xyzArrayObserver[1];
      roaming_Target[2] = xyzArrayObserver[2];
    }
    else
    {
      roaming_Target[0] = xyzArray1[0];
      roaming_Target[1] = xyzArray1[1];
      roaming_Target[2] = xyzArray1[2];
    }

    roaming_Up_Rotation = 0.0;
    roaming_FieldOfView = 65.0;

    Display_Roaming_Parameters();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ObserverLine_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSERVER_LINE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ObserverVertex_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSERVER_VERTEX);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObserverLineClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  observerLineFlag = (DisplayObserverLine->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObserverVertexClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  observerPointFlag = (DisplayObserverVertex->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetObserverParametersClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  try
  {
    observerWidth = my_double_to_float(ObserverLineWidth->Text.ToDouble());
  }
  catch(...)
  {
    observerWidth = 1.0f;
  }

  try
  {
    observerSize = my_double_to_float(ObserverPointSize->Text.ToDouble());
  }
  catch(...)
  {
    observerSize = 1.0f;
  }

  // --

  At3d_Set_Mesh_Location_Observer_Parameters( rgbLineObserverArray, rgbPointObserverArray,
                                              observerWidth, observerSize,
                                              observerLineFlag, observerPointFlag );
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UseObserverClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (UseObserver->Checked)
  {
    try
    {
      xyzDeltaArrayObserver[0] = Observer_DeltaX->Text.ToDouble();
    }
    catch(...)
    {
      xyzDeltaArrayObserver[0] = 0.0;
    }

    // --

    try
    {
      xyzDeltaArrayObserver[1] = Observer_DeltaY->Text.ToDouble();
    }
    catch(...)
    {
      xyzDeltaArrayObserver[1] = 0.0;
    }

    // --

    try
    {
      xyzDeltaArrayObserver[2] = Observer_DeltaZ->Text.ToDouble();
    }
    catch(...)
    {
      xyzDeltaArrayObserver[2] = 0.0;
    }

    // --

    if (At3d_Add_Mesh_Location_Observer(&xyzDeltaArrayObserver[0]) > 0)
    {
      haveObserverFlag = 1;
    }
    else
    {
      // error if return is -1
      // x,y and z are zero if return is zero
      //
      haveObserverFlag = 0;
    }

    Display_Rubberband_Data(DISPLAY_RUBBER_OBSERVER_ADD);
  }
  else
  {
    At3d_Cancel_Mesh_Location_Observer();

    haveObserverFlag = 0;

    Display_Rubberband_Data(DISPLAY_RUBBER_OBSERVER_CANCEL);
  }

  At3d_RenderOpenGLScene();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Rubberband_Data(Int32 item)
{
  if (item == DISPLAY_RUBBER_CLEAR_ALL)
  {
    hitFlag0 = MESH_LOCATION_NO_DATA;
    hitFlag1 = MESH_LOCATION_NO_DATA;
    hitFlag2 = MESH_LOCATION_NO_DATA;
    hitFlagObstruction = MESH_LOCATION_NO_DATA;
    hitFlagObserver = MESH_LOCATION_NO_DATA;

    Display_Locations0();
    Display_Locations1();
    Display_Locations2();
    Display_LocationsObstruction();
    Display_LocationsObserver();
  }
  else if (item == DISPLAY_RUBBER_MOUSE_UP)
  {
    // event: left button up
    //
    hitFlagObstruction = MESH_LOCATION_NO_DATA;
    Display_LocationsObstruction();

    Get_Locations1();
    Get_Locations2();
  }
  else if (item == DISPLAY_RUBBER_MOUSE_SWAP)
  {
    Get_Locations1();
    Get_Locations2();
  }
  else if (item == DISPLAY_RUBBER_MOUSE_CLIP)
  {
    hitFlagObstruction = MESH_LOCATION_NO_DATA;
    Display_LocationsObstruction();

    Get_Locations2();
  }
  else if (item == DISPLAY_RUBBER_OBSERVER_ADD)
  {
    if (haveObserverFlag)
    {
      hitFlagObstruction = MESH_LOCATION_NO_DATA;
      Display_LocationsObstruction();

      Get_LocationsObserver();
      Get_Locations2();
    }
    else
    {
      hitFlagObserver = MESH_LOCATION_NO_DATA;
      Display_LocationsObserver();
    }
  }
  else if (item == DISPLAY_RUBBER_OBSERVER_CANCEL)
  {
    hitFlagObserver = MESH_LOCATION_NO_DATA;
    Display_LocationsObserver();

    hitFlagObstruction = MESH_LOCATION_NO_DATA;
    Display_LocationsObstruction();

    Get_Locations2();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetViewpoint1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (hitFlag1 == MESH_LOCATION_HAS_XYZ)
  {
    At3d_SetCameraViewPoint(xyzArray1[0], xyzArray1[1], xyzArray1[2]);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetViewpoint2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (hitFlag2 == MESH_LOCATION_HAS_XYZ)
  {
    At3d_SetCameraViewPoint(xyzArray2[0], xyzArray2[1], xyzArray2[2]);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Zoom_Lock_NoneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  roamingZoomLock = ROAMING_LOCK_ZOOM_NONE;

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Zoom_Lock_InClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  roamingZoomLock = ROAMING_LOCK_ZOOM_IN;

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Zoom_Lock_OutClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  roamingZoomLock = ROAMING_LOCK_ZOOM_OUT;

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Zoom_Lock_BothClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  roamingZoomLock = ROAMING_LOCK_ZOOM_BOTH;

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateX1ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectX_90Minus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateX2ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectX_90Plus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateX3ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectX_180();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateY1ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectY_90Minus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateY2ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectY_90Plus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateY3ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectY_180();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateZ1ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectZ_90Minus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateZ2ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectZ_90Plus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateZ3ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectZ_180();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Move_AheadBack_Step(void)
{
  try
  {
    moveAheadBack_Step = fabs( Move_AheadBack_Step->Text.ToDouble() );
  }
  catch(...)
  {
    moveAheadBack_Step = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_AheadClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Get_Move_AheadBack_Step();
  At3d_Move_Roaming_AheadBack(moveAheadBack_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_BackClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Get_Move_AheadBack_Step();
  At3d_Move_Roaming_AheadBack(-moveAheadBack_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Move_RightLeft_Step(void)
{
  try
  {
    moveRightLeft_Step = fabs( Move_RightLeft_Step->Text.ToDouble() );
  }
  catch(...)
  {
    moveRightLeft_Step = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_RightClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Get_Move_RightLeft_Step();
  At3d_Move_Roaming_RightLeft(moveRightLeft_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_LeftClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Get_Move_RightLeft_Step();
  At3d_Move_Roaming_RightLeft(-moveRightLeft_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Move_UpDown_Step(void)
{
  try
  {
    moveUpDown_Step = fabs( Move_UpDown_Step->Text.ToDouble() );
  }
  catch(...)
  {
    moveUpDown_Step = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_UpClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Get_Move_UpDown_Step();
  At3d_Move_Roaming_UpDown(moveUpDown_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_DownClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Get_Move_UpDown_Step();
  At3d_Move_Roaming_UpDown(-moveUpDown_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Move_Turn_Step(void)
{
  try
  {
    moveTurn_Step = fabs( Move_Turn_Step->Text.ToDouble() );
  }
  catch(...)
  {
    moveTurn_Step = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_Turn_RightClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Get_Move_Turn_Step();
  At3d_Move_Roaming_Turn(moveTurn_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_Turn_LeftClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Get_Move_Turn_Step();
  At3d_Move_Roaming_Turn(-moveTurn_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Roaming_Parameters(void)
{
  Roaming_Eye_X->Text = roaming_Eye[0];
  Roaming_Eye_Y->Text = roaming_Eye[1];
  Roaming_Eye_Z->Text = roaming_Eye[2];

  Roaming_Target_X->Text = roaming_Target[0];
  Roaming_Target_Y->Text = roaming_Target[1];
  Roaming_Target_Z->Text = roaming_Target[2];

  Roaming_Up_Rotation->Text = roaming_Up_Rotation;

  Roaming_Up_FieldOfView->Text = roaming_FieldOfView;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Roaming_Parameters(void)
{
  try
  {
    roaming_Eye[0] = Roaming_Eye_X->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Eye[0] = 0.0;
  }

  try
  {
    roaming_Eye[1] = Roaming_Eye_Y->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Eye[1] = 0.0;
  }

  try
  {
    roaming_Eye[2] = Roaming_Eye_Z->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Eye[2] = 0.0;
  }

  // --

  try
  {
    roaming_Target[0] = Roaming_Target_X->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Target[0] = 0.0;
  }

  try
  {
    roaming_Target[1] = Roaming_Target_Y->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Target[1] = 0.0;
  }

  try
  {
    roaming_Target[2] = Roaming_Target_Z->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Target[2] = 0.0;
  }

  // --

  try
  {
    roaming_Up_Rotation = Roaming_Up_Rotation->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Up_Rotation = 0.0;
  }

  // --

  try
  {
    roaming_FieldOfView = Roaming_Up_FieldOfView->Text.ToDouble();
  }
  catch(...)
  {
    roaming_FieldOfView = 65.0;
  }

  // --

  // just displaying again to check for errors
  //
  Display_Roaming_Parameters();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_GetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 index = CameraID_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CameraID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    if (At3d_Get_Roaming_Data(nameBuffer, &roaming_Eye[0], &roaming_Target[0],
                              &roaming_Up_Rotation, &roaming_FieldOfView,
                              roaming_asDisplayed_Flag) > 0)
    {
      Display_Roaming_Parameters();
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_As_DisplayedClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  roaming_asDisplayed_Flag = (Roaming_As_Displayed->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_Get_FromClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = Get_From_CameraID->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Get_From_CameraID->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    if (At3d_Get_Roaming_Data(nameBuffer, &roaming_Eye[0], &roaming_Target[0],
                              &roaming_Up_Rotation, &roaming_FieldOfView,
                              roaming_asDisplayed_Flag) > 0)
    {
      Display_Roaming_Parameters();
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_SetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = CameraID_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CameraID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    Get_Roaming_Parameters();

    At3d_Set_Roaming_Data(nameBuffer, &roaming_Eye[0], &roaming_Target[0],
                          &roaming_Up_Rotation, &roaming_FieldOfView);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_CameraClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, NewCameraName->Text);

  char nameBuffer[LEN_LAYER_NAME];
  my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  nameBuffer[LEN_LAYER_NAME - 1] = '\0';

  if (At3d_Add_Roaming_Camera(nameBuffer) > 0)
  {
    Total_Cameras->Text = cameraTotal;

    Set_Camera_Lists();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Free_Roaming_CamerasClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_Free_All_Roaming_Cameras();

  Set_Camera_Lists();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Free_One_CameraClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 index = CameraID_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CameraID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    if (At3d_Free_One_Roaming_Camera(nameBuffer) > 0)
    {
      cameraID            = 0;
      activeCameraID      = 0;
      getFromCameraID     = 0;
      topLeftCameraID     = 0;
      topRightCameraID    = 0;
      bottomLeftCameraID  = 0;
      bottomRightCameraID = 0;

      Set_Camera_Lists();
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read_Cameras_FileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (camerasFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(camerasFilename, MAX_PATH, "%s%sCameras-%s.txt", drive0, dir0, name0);
  }

  if (camerasFilename[0] != '\0')
  {
    my_splitpath(camerasFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Cameras";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  Int32 result = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(camerasFilename, MAX_PATH, fileName.c_str());

    Int32 skipDefaultCameraFlag = (Skip_Default_Camera->Checked) ? 1 : 0;

    Int32 skipManyViewsFlag = (Skip_Many_Views->Checked) ? 1 : 0;

    result = At3d_View_Cameras_Read_File(camerasFilename, skipDefaultCameraFlag,
                                         skipManyViewsFlag);
  }

  delete dlg;

  if (result)
  {
    Set_Camera_Lists();

    Many_Views_Get_NamesClick(NULL);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Save_Cameras_FileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (camerasFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(camerasFilename, MAX_PATH, "%s%sCameras-%s.txt", drive0, dir0, name0);
  }

  if (camerasFilename[0] != '\0')
  {
    my_splitpath(camerasFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Cameras";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TSaveDialog* dlg;
  try
  {
    dlg = new TSaveDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofNoReadOnlyReturn << ofPathMustExist << ofOverwritePrompt;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(camerasFilename, MAX_PATH, fileName.c_str());

    Int32 saveManyViewsFlag = (Save_Many_Views->Checked) ? 1 : 0;

    At3d_View_Cameras_Save_File(camerasFilename, saveManyViewsFlag);
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Camera_Lists(void)
{
  cameraTotal = At3d_Roaming_Camera_Count();
  Total_Cameras->Text = cameraTotal;

  // --

  CameraID_List->Items->BeginUpdate();
  Active_CameraID_List->Items->BeginUpdate();

  try
  {
    CameraID_List->Clear();
    Active_CameraID_List->Clear();

    Get_From_CameraID->Clear();

    for (Int32 i=0; i<cameraTotal; i++)
    {
      char buffer[LEN_LAYER_NAME];
      if (At3d_Roaming_Camera_Get_Name(i, buffer) > 0)
      {
        CameraID_List->Items->Add(buffer);
        Active_CameraID_List->Items->Add(buffer);

        Get_From_CameraID->Items->Add(buffer);
      }
    }

    if (!cameraTotal)
    {
      cameraID = -1;
      CameraID_List->ItemIndex = -1;

      activeCameraID = -1;
      Active_CameraID_List->ItemIndex = -1;

      getFromCameraID = -1;
      Get_From_CameraID->ItemIndex = -1;
    }
    else
    {
      if (cameraID >= cameraTotal) cameraID = 0;
      CameraID_List->ItemIndex = cameraID;

      if (activeCameraID >= cameraTotal) activeCameraID = 0;
      Active_CameraID_List->ItemIndex = activeCameraID;

      if (getFromCameraID >= cameraTotal) getFromCameraID = 0;
      Get_From_CameraID->ItemIndex = getFromCameraID;
    }

    CameraID_List->Items->EndUpdate();
    Active_CameraID_List->Items->EndUpdate();
  }
  catch(...)
  {
    CameraID_List->Items->EndUpdate();
    Active_CameraID_List->Items->EndUpdate();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CameraID_ListClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  cameraID = CameraID_List->ItemIndex;

  Get_Roaming_AxisClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_From_CameraIDChange(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  getFromCameraID = Get_From_CameraID->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Active_CameraID_ListClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = Active_CameraID_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Active_CameraID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    At3d_Switch_OGL_Camera(nameBuffer);

    Get_All_ActiveCamera_Data();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Roaming_AxisClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Roaming_GetClick(MY_NULL_PTR);
  Get_Axis_XYZClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Roaming_AxisClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Roaming_SetClick(MY_NULL_PTR);
  Set_Axis_XYZClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_Up_DefaultClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  roaming_Up_Rotation = 0.0;

  Display_Roaming_Parameters();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetGPUMaxTextureSizeClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  gpu_Max_Texture_Size = At3d_Get_GPU_Max_Texture_Size();

  GPUMaxTextureSize->Text = gpu_Max_Texture_Size;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetUserMaxTextureSizeClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  user_Max_Texture_Size = At3d_Get_Max_Texture_Size();

  UserMaxTextureSize->Text = user_Max_Texture_Size;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetUserMaxTextureSizeClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  powerOf2_Max_Texture_Size = PowerOf2_MaxTextureSize->Text.ToInt();

  Int32 result = At3d_Set_Max_Texture_Size(powerOf2_Max_Texture_Size);
  if (result) GetUserMaxTextureSizeClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_DragClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_SELECT, DISPLAY_MESH_SELECT_DRAG);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_ClickClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_SELECT, DISPLAY_MESH_SELECT_CLICK);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Line_Panel_MLClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_LINE_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Vertex1_Panel_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_VERTEX1_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Vertex2_Panel_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_VERTEX2_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UseDepthTest_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  depthTestFlag_ML = (UseDepthTest_ML->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayVertices_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  pointsFlag_ML = (DisplayVertices_ML->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetRubberbandParameters_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  try
  {
    lineWidth_ML = my_double_to_float(LineWidth_ML->Text.ToDouble());
  }
  catch(...)
  {
    lineWidth_ML = 1.0f;
  }

  try
  {
    pointSize_ML = my_double_to_float(PointSize_ML->Text.ToDouble());
  }
  catch(...)
  {
    pointSize_ML = 1.0f;
  }

  // --

  At3d_MeasureList_Set_Parameters(&rgbLineArray_ML[0], &rgbPoint1Array_ML[0],
                                  &rgbPoint2Array_ML[0],
                                  lineWidth_ML, pointSize_ML,
                                  depthTestFlag_ML, pointsFlag_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Obstruction_Panel_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSTRUCTION_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObstructionVertex_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  obstructionDisplayFlag_ML = (DisplayObstructionVertex_ML->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetObstructionParameters_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  try
  {
    pointObstructionSize_ML = my_double_to_float(ObstructionPointSize_ML->Text.ToDouble());
  }
  catch(...)
  {
    pointObstructionSize_ML = 1.0f;
  }

  // --

  At3d_MeasureList_Set_Obstruction_Parameters(rgbObstructionArray_ML,
                                              pointObstructionSize_ML,
                                              obstructionDisplayFlag_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ObserverLine_Panel_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSERVER_LINE_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ObserverVertex_Panel_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSERVER_VERTEX_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObserverLine_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  observerLineFlag_ML = (DisplayObserverLine_ML->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObserverVertex_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  observerPointFlag_ML = (DisplayObserverVertex_ML->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetObserverParameters_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  try
  {
    observerWidth_ML = my_double_to_float(ObserverLineWidth_ML->Text.ToDouble());
  }
  catch(...)
  {
    observerWidth_ML = 1.0f;
  }

  try
  {
    observerSize_ML = my_double_to_float(ObserverPointSize_ML->Text.ToDouble());
  }
  catch(...)
  {
    observerSize_ML = 1.0f;
  }

  // --

  At3d_MeasureList_Set_Observer_Parameters( rgbLineObserverArray_ML, rgbPointObserverArray_ML,
                                            observerWidth_ML, observerSize_ML,
                                            observerLineFlag_ML, observerPointFlag_ML );
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayMeasureListClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_DISPLAY_MEASURE_LIST,
                 (DisplayMeasureList->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AddCurrentMeasureClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, MeasureID->Text);

  char buffer[LEN_LAYER_NAME];
  my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  buffer[LEN_LAYER_NAME - 1] = '\0';

  float width;
  float rgbColor[3];

  if (!measureColorFlag)
  {
    width = lineWidth_ML;

    rgbColor[0] = rgbLineArray_ML[0];
    rgbColor[1] = rgbLineArray_ML[1];
    rgbColor[2] = rgbLineArray_ML[2];
  }
  else if (measureColorFlag == 1)
  {
    width = 3.0f;

    rgbColor[0] = 1.0f;
    rgbColor[1] = 0.5f;
    rgbColor[2] = 0.0f;
  }
  else
  {
    width = 4.0f;

    rgbColor[0] = 1.0f;
    rgbColor[1] = 0.0f;
    rgbColor[2] = 0.0f;
  }

  if (At3d_MeasureList_Add_Current(buffer, width, &rgbColor[0]) > -1)
  {
    Measure_ID_List_Add(buffer);
  }

  GetMeasureCountClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SaveImageClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if ( (!fileLoaded) ||
       (inName[0] == '\0') )
  {
    return;
  }

  // --

  Int32 width;
  Int32 height;
  Int32 maxSize = At3d_Get_GPU_Max_Texture_Size();

  try
  {
    width = ImageWidth->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  try
  {
    height = ImageHeight->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  float line_width;
  float point_size;

  try
  {
    line_width = my_double_to_float(ImageLineWidth->Text.ToDouble());
  }
  catch(...)
  {
    line_width = 1.0f;
  }

  try
  {
    point_size = my_double_to_float(ImagePointSize->Text.ToDouble());
  }
  catch(...)
  {
    point_size = 1.0f;
  }

  if ( (width > 0) &&
       (width <= maxSize) &&
       (height > 0) &&
       (height <= maxSize) &&
       (line_width > 0.0f) &&
       (point_size > 0.0f) )
  {
    char filename[_MAX_PATH];
    char drive0[_MAX_DRIVE];
    char dir0[_MAX_DIR];
    char name0[_MAX_FNAME];
    char ext0[_MAX_EXT];

    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(filename, _MAX_PATH, drive0);
    my_strcat(filename, _MAX_PATH, dir0);
    my_strcat(filename, _MAX_PATH, name0);

    // adding a suffix to avoid overwriting a JPG file used as a UV texture
    // if it had the same name as the model file
    //
    my_strcat(filename, _MAX_PATH, "-view_window");

    my_strcat(filename, _MAX_PATH, ".jpg");

    At3d_Save_Thumbnail_LineWidth_PointSize(line_width, point_size);

    At3d_Save_Thumbnail_Image(filename, width, height);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::StatusBar_ShowClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 height;

  try
  {
    height = StatusBar_Height->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  At3d_StatusBar_Show( (StatusBar_Top->Checked) ? 1 : 0, height );
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::StatusBar_HideClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_StatusBar_Hide();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::FontColorPanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  TColorDialog* ColorDlg;
  try
  {
    ColorDlg = new TColorDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    ColorDlg = MY_NULL_PTR;
  }

  if (ColorDlg)
  {
    ColorDlg->Options = ColorDlg->Options << cdFullOpen;
    ColorDlg->Color = static_cast<TColor>(fontColor);

    ColorDlg->Execute();

    fontColor = ColorDlg->Color;
    FontColorPanel->Color = static_cast<TColor>(fontColor);

    delete ColorDlg;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetFontColorClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_StatusBar_Set_Font_Color(fontColor);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetFontNameClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, FontName->Text);

  char buffer[LEN_LAYER_NAME];
  my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  buffer[LEN_LAYER_NAME - 1] = '\0';

  At3d_StatusBar_Set_Font_Name(buffer);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetFontSizeClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 size;

  try
  {
    size = FontSize->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  At3d_StatusBar_Set_Font_Size(size);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetFontStyleClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 boldFlag      = (FontBold->Checked)      ? 1 : 0;
  Int32 italicFlag    = (FontItalic->Checked)    ? 1 : 0;
  Int32 underlineFlag = (FontUnderline->Checked) ? 1 : 0;
  Int32 strikeOutFlag = (FontStrikeOut->Checked) ? 1 : 0;

  At3d_StatusBar_Set_Font_Style(boldFlag, italicFlag, underlineFlag, strikeOutFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetStatusBarTextClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, StatusBar_Text->Text);

  char buffer[LEN_LAYER_NAME];
  my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  buffer[LEN_LAYER_NAME - 1] = '\0';

  At3d_StatusBar_Set_Text(buffer);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetMeasureCountClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 count = At3d_MeasureList_Get_Count();

  MeasureCount->Text = count;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ClearMeasureListClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_MeasureList_Clear();

  Measure_ID_List_Clear();

  GetMeasureCountClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetMeasurementClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 index;
  MEASURE_DATA data;

  try
  {
    index = Get_Index->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  if (index == -1)
  {
    AnsiString tempID;
    my_copystring_to_ansi_from_vcl(&tempID, Get_ID->Text);

    my_strncpy(data.id, LEN_LAYER_NAME, tempID.c_str(), LEN_LAYER_NAME - 1);
    data.id[LEN_LAYER_NAME - 1] = '\0';
  }

  // --

  Int32 result = Get_MeasureList_Item(index, &data);

  Display_Measure_Data(result, &data);
}
//---------------------------------------------------------------------------

MEASURE_DATA* __fastcall TTest_At3dx_LocationsForm::Alloc_Measure_Data(void)
{
  MEASURE_DATA* ptr = MY_NULL_PTR;

  try
  {
    ptr = new MEASURE_DATA;

    if (ptr)
    {
      my_memset(ptr, 0, sizeof(MEASURE_DATA));
    }
  }
  catch(...)
  {
    ptr = MY_NULL_PTR;
  }

  return(ptr);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Clear_measureData_List(void)
{
  if (!measureData_ListPtr) return;

  // --

  Int32 iEnd = measureData_ListPtr->Count;
  if (iEnd)
  {
    for(Int32 i=0; i<iEnd; i++)
    {
      MEASURE_DATA* ptr = reinterpret_cast<MEASURE_DATA*>(measureData_ListPtr->Items[i]);

      if (ptr)
      {
        delete ptr;
      }
    }

    measureData_ListPtr->Clear();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Measure_Data(Int32 index,
                                                         MEASURE_DATA* dataPtr)
{
  if (!dataPtr) return;

  // --

  if (index == -1)
  {
    my_memset(dataPtr, 0, sizeof(MEASURE_DATA));
  }

  Measure_Index->Text = index;

  Measure_ID->Text = dataPtr->id;

  Measure_Visible_Flag->Checked     = (dataPtr->visible_Flag) ? 1 : 0;
  Measure_Observer_Flag->Checked    = (dataPtr->observer_Flag) ? 1 : 0;
  Measure_Obstruction_Flag->Checked = (dataPtr->obstruction_Flag) ? 1 : 0;

  // --

  Measure_Vertex1_X->Caption = dataPtr->vertex1[0];
  Measure_Vertex1_Y->Caption = dataPtr->vertex1[1];
  Measure_Vertex1_Z->Caption = dataPtr->vertex1[2];

  Measure_Vertex2_X->Caption = dataPtr->vertex2[0];
  Measure_Vertex2_Y->Caption = dataPtr->vertex2[1];
  Measure_Vertex2_Z->Caption = dataPtr->vertex2[2];

  // --

  Measure_Distance->Caption = dataPtr->distance;

  // --

  Measure_Line_Width->Text = static_cast<double>(dataPtr->lineWidth);

  Int32 red   = my_change_float_to_256_int(dataPtr->rgb_Line_Array[0]);
  Int32 green = my_change_float_to_256_int(dataPtr->rgb_Line_Array[1]);
  Int32 blue  = my_change_float_to_256_int(dataPtr->rgb_Line_Array[2]);

  Measure_Line_Color->Color = static_cast<TColor>(red + (green << 8) + (blue << 16));

  // --

  Measure_Observer_X->Caption = dataPtr->observer[0];
  Measure_Observer_Y->Caption = dataPtr->observer[1];
  Measure_Observer_Z->Caption = dataPtr->observer[2];

  // --

  Measure_Obstruction_X->Caption = dataPtr->obstruction[0];
  Measure_Obstruction_Y->Caption = dataPtr->obstruction[1];
  Measure_Obstruction_Z->Caption = dataPtr->obstruction[2];

  // --

  Measure_Obstruction_Distance->Caption = dataPtr->obstruction_Distance;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Transfer_All_FromClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!measureData_ListPtr) return;

  // --

  Int32 fromCount = At3d_MeasureList_Get_Count();

  if (!fromCount)
  {
    return;
  }

  // --

  for (Int32 i=0; i<fromCount; i++)
  {
    MEASURE_DATA* ptr = Alloc_Measure_Data();

    if (ptr)
    {
      measureData_ListPtr->Add(ptr);

      Get_MeasureList_Item(i, ptr);
    }
  }

  Transfer_Count->Text = measureData_ListPtr->Count;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Transfer_Random5Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!measureData_ListPtr) return;

  // --

  Int32 iEnd = measureData_ListPtr->Count;

  if (iEnd < 5)
  {
    return;
  }

  for (Int32 i=0; i<5; i++)
  {
    Int32 index;

    if (!i)
    {
      index = 3;
    }
    else if (i == 1)
    {
      index = 1;
    }
    else if (i == 2)
    {
      index = 4;
    }
    else if (i == 3)
    {
      index = 2;
    }
    else
    {
      index = 0;
    }

    MEASURE_DATA* ptr = reinterpret_cast<MEASURE_DATA*>(measureData_ListPtr->Items[index]);

    if ( Add_MeasureList_Item(ptr) > -1 )
    {
      Measure_ID_List_Add(ptr->id);
    }
  }

  // --

  GetMeasureCountClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Transfer_All_ToClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!measureData_ListPtr) return;

  // --

  Int32 iEnd = measureData_ListPtr->Count;

  if (!iEnd)
  {
    return;
  }

  // --

  for (Int32 i=0; i<iEnd; i++)
  {
    MEASURE_DATA* ptr = reinterpret_cast<MEASURE_DATA*>(measureData_ListPtr->Items[i]);

    if (ptr)
    {
      if ( Add_MeasureList_Item(ptr) > -1 )
      {
        Measure_ID_List_Add(ptr->id);
      }
    }
  }

  GetMeasureCountClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Transfer_ClearClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!measureData_ListPtr) return;

  // --

  Clear_measureData_List();
  Transfer_Count->Text = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::No_Ctrl_Key_PanClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_NO_CTRL_KEY_PAN, (No_Ctrl_Key_Pan->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Measure_ID_List_Clear(void)
{
  Measure_ID_List->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Measure_ID_List_Add(char* idPtr)
{
  Measure_ID_List->Items->Add(idPtr);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Visible_AllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_MeasureList_Set_Visible_All(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Visible_NoneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_MeasureList_Set_Visible_All(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Visible_Set_OneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Measure_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      At3d_MeasureList_Set_Visible_One(-1, buffer, (Measure_ID_Visible->Checked) ? 1 : 0);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Only_One_VisibleClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    At3d_WriteFlag(LIBFLAG_NO_DRAW, 1);

    At3d_MeasureList_Set_Visible_All(0);

    At3d_MeasureList_Set_Visible_One(itemIndex, MY_NULL_PTR, 1);

    At3d_WriteFlag(LIBFLAG_NO_DRAW, 0);

    At3d_DrawWireframe();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DeleteTest1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Measure_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if ( At3d_MeasureList_Delete_Item(-1, buffer) > 0 )
      {
        Measure_ID_List->Items->Delete(itemIndex);
        GetMeasureCountClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DeleteTest2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    if ( At3d_MeasureList_Delete_Item(itemIndex, MY_NULL_PTR) > 0 )
    {
      Measure_ID_List->Items->Delete(itemIndex);
      GetMeasureCountClick(MY_NULL_PTR);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReplaceTest1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Measure_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      MEASURE_DATA data;

      my_strncpy(data.id, LEN_LAYER_NAME, buffer, LEN_LAYER_NAME - 1);

      if ( Get_MeasureList_Item(-1, &data) == itemIndex)
      {
        if (data.observer_Flag)
        {
          data.observer_DeltaXYZ[2] += 1.5;

          Replace_MeasureList_Item( -1, buffer, &data );
        }
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReplaceTest2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    MEASURE_DATA data;

    if ( Get_MeasureList_Item(itemIndex, &data) == itemIndex)
    {
      if (data.observer_Flag)
      {
        if (data.observer_DeltaXYZ[2] - 1.5 < 0.0)
        {
        }
        else
        {
          data.observer_DeltaXYZ[2] -= 1.5;

          my_strcat(data.id, LEN_LAYER_NAME, "7");

          if ( Replace_MeasureList_Item( itemIndex, MY_NULL_PTR, &data ) > 0 )
          {
            Measure_ID_List->Items->Strings[itemIndex] = data.id;
          }
        }
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SelectTest1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Measure_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      float rgbArray[3];

      rgbArray[0] = 0.0f;
      rgbArray[1] = 1.0f;
      rgbArray[2] = 0.0f;

      At3d_MeasureList_Set_Selected_Color(&rgbArray[0]);

      Int32 result = At3d_MeasureList_Set_Selected_Line(-1, buffer);

      if ( result > -1 )
      {
        MEASURE_DATA data;

        result = Get_MeasureList_Item(result, &data);

        Display_Measure_Data(result, &data);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SelectTest2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    float rgbArray[3];

    rgbArray[0] = 1.0f;
    rgbArray[1] = 1.0f;
    rgbArray[2] = 1.0f;

    At3d_MeasureList_Set_Selected_Color(&rgbArray[0]);

    Int32 result = At3d_MeasureList_Set_Selected_Line(itemIndex, MY_NULL_PTR);

    if ( result > -1 )
    {
      MEASURE_DATA data;

      result = Get_MeasureList_Item(result, &data);

      Display_Measure_Data(result, &data);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SelectCancelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_MeasureList_Cancel_Selected_Line();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SelectMeasureLineClick(TObject *Sender)

{
   sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_OGL_PICK_MEASURE_LINE, (SelectMeasureLine->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CenterAtOriginClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_SetObject_XYZCenter();
}
//---------------------------------------------------------------------------

double __fastcall TTest_At3dx_LocationsForm::Calculate_Length(double* vertex1_Ptr,
                                                              double* vertex2_Ptr)
{
  double distance = 0.0;

  if ( (!vertex1_Ptr) ||
       (!vertex2_Ptr) )
  {
    return(distance);
  }

  // --

  double x1 = *vertex1_Ptr++;
  double y1 = *vertex1_Ptr++;
  double z1 = *vertex1_Ptr;

  double x2 = *vertex2_Ptr++;
  double y2 = *vertex2_Ptr++;
  double z2 = *vertex2_Ptr;

  double x = x2 - x1;
  double y = y2 - y1;
  double z = z2 - z1;

  distance = sqrt( x * x + y * y + z * z );

  return(distance);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Calculate_Distances(void)
{
  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if (hitFlagObserver == MESH_LOCATION_HAS_XYZ)
    {
      rubberbandDistance = Calculate_Length(&xyzArrayObserver[0],
                                            &xyzArray2[0]);
    }
    else
    {
      rubberbandDistance = Calculate_Length(&xyzArray1[0],
                                            &xyzArray2[0]);
    }

    // --

    if (hitFlagObstruction == MESH_LOCATION_HAS_XYZ)
    {
      if (hitFlagObserver == MESH_LOCATION_HAS_XYZ)
      {
        obstructionDistance = Calculate_Length(&xyzArrayObserver[0],
                                               &xyzArrayObstruction[0]);
      }
      else
      {
        obstructionDistance = Calculate_Length(&xyzArray1[0],
                                               &xyzArrayObstruction[0]);
      }
    }
    else
    {
      obstructionDistance = 0.0;
    }
  }
  else
  {
    rubberbandDistance  = 0.0;
    obstructionDistance = 0.0;
  }
}
//---------------------------------------------------------------------------

Int32 __fastcall TTest_At3dx_LocationsForm::Add_MeasureList_Item(MEASURE_DATA* dataPtr)
{
  Int32 result = -1;

  if ( (!dllOpen) ||
       (!dataPtr) )
  {
    return(result);
  }

  // --

  result = At3d_MeasureList_Add_Item(dataPtr->id,
                                     dataPtr->visible_Flag,
                                     dataPtr->observer_Flag,
                                     dataPtr->obstruction_Flag,
                                     dataPtr->lineWidth,
                                     &dataPtr->rgb_Line_Array[0],
                                     &dataPtr->vertex1[0],
                                     &dataPtr->vertex2[0],
                                     &dataPtr->observer_DeltaXYZ[0],
                                     &dataPtr->obstruction[0]);

  return(result);
}
//---------------------------------------------------------------------------

Int32 __fastcall TTest_At3dx_LocationsForm::Get_MeasureList_Item(Int32 index, MEASURE_DATA* dataPtr)
{
  Int32 result = -1;

  if ( (!dllOpen) ||
       (!dataPtr) )
  {
    return(result);
  }

  // --

  result = At3d_MeasureList_Get_Item(index, dataPtr->id,
                                     &dataPtr->visible_Flag,
                                     &dataPtr->observer_Flag,
                                     &dataPtr->obstruction_Flag,
                                     &dataPtr->lineWidth,
                                     &dataPtr->rgb_Line_Array[0],
                                     &dataPtr->vertex1[0],
                                     &dataPtr->vertex2[0],
                                     &dataPtr->observer_DeltaXYZ[0],
                                     &dataPtr->obstruction[0]);

  if (result > -1)
  {
    if (dataPtr->observer_Flag)
    {
      dataPtr->observer[0] = dataPtr->vertex1[0] + dataPtr->observer_DeltaXYZ[0];
      dataPtr->observer[1] = dataPtr->vertex1[1] + dataPtr->observer_DeltaXYZ[1];
      dataPtr->observer[2] = dataPtr->vertex1[2] + dataPtr->observer_DeltaXYZ[2];

      dataPtr->distance = Calculate_Length(&dataPtr->observer[0],
                                           &dataPtr->vertex2[0]);
    }
    else
    {
      dataPtr->observer[0] = 0.0;
      dataPtr->observer[1] = 0.0;
      dataPtr->observer[2] = 0.0;

      dataPtr->distance = Calculate_Length(&dataPtr->vertex1[0],
                                           &dataPtr->vertex2[0]);
    }

    // --

    if (dataPtr->obstruction_Flag)
    {
      if (dataPtr->observer_Flag)
      {
        dataPtr->obstruction_Distance = Calculate_Length(&dataPtr->observer[0],
                                                         &dataPtr->obstruction[0]);
      }
      else
      {
        dataPtr->obstruction_Distance = Calculate_Length(&dataPtr->vertex1[0],
                                                         &dataPtr->obstruction[0]);
      }
    }
    else
    {
      dataPtr->obstruction_Distance = 0.0;
    }
  }



  return(result);
}
//---------------------------------------------------------------------------

Int32 __fastcall TTest_At3dx_LocationsForm::Replace_MeasureList_Item(Int32 index,
                                         char* bufferPtr, MEASURE_DATA* dataPtr)
{
  Int32 result = -1;

  if ( (!dllOpen) ||
       (!dataPtr) )
  {
    return(result);
  }

  // --

  result =  At3d_MeasureList_Replace_Item(index, bufferPtr,
                                          dataPtr->id,
                                          dataPtr->visible_Flag,
                                          dataPtr->observer_Flag,
                                          dataPtr->obstruction_Flag,
                                          dataPtr->lineWidth,
                                          &dataPtr->rgb_Line_Array[0],
                                          &dataPtr->vertex1[0],
                                          &dataPtr->vertex2[0],
                                          &dataPtr->observer_DeltaXYZ[0],
                                          &dataPtr->obstruction[0]);

  return(result);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MeasureColor_DefaultClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  measureColorFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MeasureColor_Sniper1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  measureColorFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MeasureColor_Sniper2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  measureColorFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetStatusbarTextClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  char buffer[80];
  buffer[0] = '\0';

  At3d_StatusBar_Get_Text(buffer, 80);

  if (buffer[0] == '\0')
  {
    Statusbar_ReadText->Clear();
  }
  else
  {
    Statusbar_ReadText->Text = buffer;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read_Measurements_FileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (measurementsFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(measurementsFilename, MAX_PATH, "%s%sMeasurements-%s.txt", drive0, dir0, name0);
  }

  if (measurementsFilename[0] != '\0')
  {
    my_splitpath(measurementsFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Measurements";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  Int32 result = 0;
  Int32 oldCount = At3d_MeasureList_Get_Count();

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(measurementsFilename, MAX_PATH, fileName.c_str());


    result = At3d_Measurements_Read_File(measurementsFilename);
  }

  delete dlg;

  if (result)
  {
    Int32 newCount = At3d_MeasureList_Get_Count();

    char name[LEN_LAYER_NAME];

    if (newCount > oldCount)
    {
      for (Int32 i=oldCount; i<newCount; i++)
      {
        if (At3d_MeasureList_Get_Name(i, name) > 0)
        {
          Measure_ID_List->Items->Add(name);
        }
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Save_Measurements_FileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) ||
       (!At3d_MeasureList_Get_Count()) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (measurementsFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(measurementsFilename, MAX_PATH, "%s%sMeasurements-%s.txt", drive0, dir0, name0);
  }

  if (measurementsFilename[0] != '\0')
  {
    my_splitpath(measurementsFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Measurements";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TSaveDialog* dlg;
  try
  {
    dlg = new TSaveDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofNoReadOnlyReturn << ofPathMustExist << ofOverwritePrompt;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(measurementsFilename, MAX_PATH, fileName.c_str());


    At3d_Measurements_Save_File(measurementsFilename);
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::No_Reduce_Vert_Edge_PolyClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_NO_DUPLICATE_VERT_EDGE_POLY_REDUCE,
                 (No_Reduce_Vert_Edge_Poly->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::InvertSelectedColorClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_INVERT_COLOR_SELECTED_MEASURE_LINE,
                 (InvertSelectedColor->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayBillboardsClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_DISPLAY_MEASURE_BILLBOARDS,
                 (DisplayBillboards->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MakeBillboardClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, BillboardText->Text);

  Graphics::TBitmap* bmpPtr;

  try
  {
    bmpPtr = new Graphics::TBitmap;
  }
  catch(...)
  {
    return;
  }

  bmpPtr->PixelFormat = pf24bit;

  bmpPtr->Canvas->Font->Size = -48;

  if (BillboardFontColor1->Checked)
  {
    bmpPtr->Canvas->Font->Color = clRed;
  }
  else if (BillboardFontColor2->Checked)
  {
    bmpPtr->Canvas->Font->Color = clGreen;
  }
  else
  {
    bmpPtr->Canvas->Font->Color = clBlue;
  }

  bmpPtr->Canvas->Font->Style= TFontStyles() << System::Uitypes::fsBold;
  bmpPtr->Canvas->Brush->Style=bsSolid;
  bmpPtr->Canvas->Brush->Color=clBlack;

  Int32 imageWidth  = bmpPtr->Canvas->TextWidth(tempName.c_str());
  Int32 imageHeight = bmpPtr->Canvas->TextHeight(tempName.c_str());

  bmpPtr->Width  = imageWidth;
  bmpPtr->Height = imageHeight;

  bmpPtr->Canvas->TextOut(0, 0,tempName.c_str());

  // --

  Int32 alphaFlag = (BillboardAlpha->Checked) ? 1 : 0;

  Int32 rowBytes;

  if (alphaFlag)
  {
    rowBytes = imageWidth * 4;
  }
  else
  {
    rowBytes = ((3 * imageWidth + 3) / 4) * 4;
  }

  UInt8* imagePtr;

  try
  {
    imagePtr = new UInt8[static_cast<size_t>(rowBytes * imageHeight)];
  }
  catch(...)
  {
    imagePtr = MY_NULL_PTR;;
  }

  if (imagePtr)
  {
    for (Int32 y=0; y<imageHeight; y++)
    {
      UInt8* ptr0 = reinterpret_cast<UInt8*>(bmpPtr->ScanLine[y]);
      UInt8* ptr1 = imagePtr + y * rowBytes;

      for (Int32 x=0; x<imageWidth; x++)
      {
        Int32 count = *ptr0;
        *ptr1++ = *ptr0++;

        count += *ptr0;
        *ptr1++ = *ptr0++;

        count += *ptr0;
        *ptr1++ = *ptr0++;

        if (alphaFlag)
        {
          *ptr1++ = (count < 10) ? 0 : 255;
        }
      }
    }

    // --

    float rgb_Line_Array[3];

    if (BillboardFontColor1->Checked)
    {
      rgb_Line_Array[0] = 1.0f;
      rgb_Line_Array[1] = 0.0f;
      rgb_Line_Array[2] = 0.0f;
    }
    else if (BillboardFontColor2->Checked)
    {
      rgb_Line_Array[0] = 0.0f;
      rgb_Line_Array[1] = 1.0f;
      rgb_Line_Array[2] = 0.0f;
    }
    else
    {
      rgb_Line_Array[0] = 0.0f;
      rgb_Line_Array[1] = 0.0f;
      rgb_Line_Array[2] = 1.0f;
    }

    // --

    float billboardLineWidth;

    try
    {
      billboardLineWidth = my_double_to_float(BillboardLineWidth->Text.ToDouble());
    }
    catch(...)
    {
      billboardLineWidth = 1.0f;
    }

    // --

    double height;

    try
    {
      height = BillboardHeight->Text.ToDouble();
    }
    catch(...)
    {
      height = 1.0;
    }

    // --

    double position[3];

    try
    {
      position[0] = BillboardPositionX->Text.ToDouble();
    }
    catch(...)
    {
      position[0] = 0.0;
    }

    try
    {
      position[1] = BillboardPositionY->Text.ToDouble();
    }
    catch(...)
    {
      position[1] = 0.0;
    }

    try
    {
      position[2] = BillboardPositionZ->Text.ToDouble();
    }
    catch(...)
    {
      position[2] = 0.0;
    }

    // --

    double pivot[3];

    try
    {
      pivot[0] = BillboardPivotX->Text.ToDouble();
    }
    catch(...)
    {
      pivot[0] = 0.0;
    }

    try
    {
      pivot[1] = BillboardPivotY->Text.ToDouble();
    }
    catch(...)
    {
      pivot[1] = 0.0;
    }

    try
    {
      pivot[2] = BillboardPivotZ->Text.ToDouble();
    }
    catch(...)
    {
      pivot[2] = 0.0;
    }


    // --

    double rotation[3];

    try
    {
      rotation[0] = BillboardRotateX->Text.ToDouble();
    }
    catch(...)
    {
      rotation[0] = 0.0;
    }

    try
    {
      rotation[1] = BillboardRotateY->Text.ToDouble();
    }
    catch(...)
    {
      rotation[1] = 0.0;
    }

    try
    {
      rotation[2] = BillboardRotateZ->Text.ToDouble();
    }
    catch(...)
    {
      rotation[2] = 0.0;
    }

    // --

    Int32 typeFlag;

    try
    {
      typeFlag = BillboardType->Text.ToInt();
    }
    catch(...)
    {
      typeFlag = 0;
    }

    // --

    my_copystring_to_ansi_from_vcl(&tempName, BillboardID->Text);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    // --

    if (At3d_Measure_Billboard_Add_Item(buffer, imagePtr, imageWidth,
                                        imageHeight, alphaFlag,
                                        BillboardPivotTypeList->ItemIndex,
                                        (BillboardStatic->Checked) ? 1 : 0,
                                        (BillboardVisible->Checked) ? 1 : 0,
                                        (BillboardOglReplace->Checked) ? 1 : 0,
                                        typeFlag,
                                        &rgb_Line_Array[0], billboardLineWidth,
                                        height, &position[0], &pivot[0],
                                        &rotation[0]) > -1)
    {
      if (Billboard_ID_List->Items->Add(buffer) > -1)
      {
        Billboard_Get_CountClick(MY_NULL_PTR);
      }
    }
  }

  // --

  if (imagePtr) delete[] imagePtr;

  if (bmpPtr) delete bmpPtr;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DeleteAllBillboardsClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (At3d_Measure_Billboard_Delete_All() > 0)
  {
    Billboard_ID_List->Clear();
    Billboard_Get_CountClick(MY_NULL_PTR);
  }
}
//---------------------------------------------------------------------------


void __fastcall TTest_At3dx_LocationsForm::No_Ctrl_Key_ZoomClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_NO_CTRL_KEY_ZOOM, (No_Ctrl_Key_Zoom->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraRotateDegreesClick(TObject *Sender)
{
   sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  double degrees;

  try
  {
    degrees = CameraRotateDegrees->Text.ToDouble();
  }
  catch(...)
  {
    degrees = 0.0;
  }

  At3d_Camera_Rotate_Set_Speed(degrees);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::TwoSidedLightingClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_OGL_2SIDED_LIGHTING, (TwoSidedLighting->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CullBackFacesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_OGL_CULL_BACKFACES, (CullBackFaces->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::NoMouseButtonMoveActivityClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_NO_MOUSE_BOTTON_MOVE_ACTIVITY, (NoMouseButtonMoveActivity->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::NoMouseWheelActivityClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_NO_MOUSE_WHEEL_ACTIVITY, (NoMouseWheelActivity->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::No_Ctrl_Key_FOVClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_NO_CTRL_KEY_FOV, (No_Ctrl_Key_FOV->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetNoCtrlStepFOVClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  float step = At3d_No_Ctrl_Key_FOV_Get_Step();

  NoCtrlStepFOV->Text = static_cast<double>(step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetNoCtrlStepFOVClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  float step;

  try
  {
    step = my_double_to_float( NoCtrlStepFOV->Text.ToDouble() );
  }
  catch(...)
  {
    step = 0.1f;
  }

  At3d_No_Ctrl_Key_FOV_Set_Step(step);
}
//---------------------------------------------------------------------------
void __fastcall TTest_At3dx_LocationsForm::Billboard_Visible_AllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_Measure_Billboard_Set_Visible_All(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Visible_OneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_Measure_Billboard_Set_Visible_All(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Visible_Set_OneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      At3d_Measure_Billboard_Set_Visible_One(-1, buffer, (Billboard_ID_Visible->Checked) ? 1 : 0);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Only_One_VisibleClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    At3d_WriteFlag(LIBFLAG_NO_DRAW, 1);

    At3d_Measure_Billboard_Set_Visible_All(0);

    At3d_Measure_Billboard_Set_Visible_One(itemIndex, MY_NULL_PTR, 1);

    At3d_WriteFlag(LIBFLAG_NO_DRAW, 0);

    At3d_DrawWireframe();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_DeleteTest1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if ( At3d_Measure_Billboard_Delete_Item(-1, buffer) > 0 )
      {
        Billboard_ID_List->Items->Delete(itemIndex);
        Billboard_Get_CountClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_DeleteTest2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    if ( At3d_Measure_Billboard_Delete_Item(itemIndex, MY_NULL_PTR) > 0 )
    {
      Billboard_ID_List->Items->Delete(itemIndex);
      Billboard_Get_CountClick(MY_NULL_PTR);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetPivot1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double position[3];
      double pivot[3];

      if (At3d_Measure_Billboard_Get_Pivot(-1, buffer, &position[0], &pivot[0]) > 0)
      {
        Billboard_PositionX->Text = position[0];
        Billboard_PositionY->Text = position[1];
        Billboard_PositionZ->Text = position[2];

        Billboard_PivotX->Text = pivot[0];
        Billboard_PivotY->Text = pivot[1];
        Billboard_PivotZ->Text = pivot[2];
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetPivot2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    double position[3];
    double pivot[3];

    if (At3d_Measure_Billboard_Get_Pivot(itemIndex, MY_NULL_PTR, &position[0], &pivot[0]) > 0)
    {
      Billboard_PositionX->Text = position[0];
      Billboard_PositionY->Text = position[1];
      Billboard_PositionZ->Text = position[2];

      Billboard_PivotX->Text = pivot[0];
      Billboard_PivotY->Text = pivot[1];
      Billboard_PivotZ->Text = pivot[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetPivot1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double position[3];

      try
      {
        position[0] = Billboard_PositionX->Text.ToDouble();
      }
      catch(...)
      {
        position[0] = 0.0;
      }

      try
      {
        position[1] = Billboard_PositionY->Text.ToDouble();
      }
      catch(...)
      {
        position[1] = 0.0;
      }

      try
      {
        position[2] = Billboard_PositionZ->Text.ToDouble();
      }
      catch(...)
      {
        position[2] = 0.0;
      }

      // --

      double pivot[3];

      try
      {
        pivot[0] = Billboard_PivotX->Text.ToDouble();
      }
      catch(...)
      {
        pivot[0] = 0.0;
      }

      try
      {
        pivot[1] = Billboard_PivotY->Text.ToDouble();
      }
      catch(...)
      {
        pivot[1] = 0.0;
      }

      try
      {
        pivot[2] = Billboard_PivotZ->Text.ToDouble();
      }
      catch(...)
      {
        pivot[2] = 0.0;
      }

      // --

      At3d_Measure_Billboard_Set_Pivot(-1, buffer, &position[0], &pivot[0]);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetPivot2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    double position[3];

    try
    {
      position[0] = Billboard_PositionX->Text.ToDouble();
    }
    catch(...)
    {
      position[0] = 0.0;
    }

    try
    {
      position[1] = Billboard_PositionY->Text.ToDouble();
    }
    catch(...)
    {
      position[1] = 0.0;
    }

    try
    {
      position[2] = Billboard_PositionZ->Text.ToDouble();
    }
    catch(...)
    {
      position[2] = 0.0;
    }

    // --

    double pivot[3];

    try
    {
      pivot[0] = Billboard_PivotX->Text.ToDouble();
    }
    catch(...)
    {
      pivot[0] = 0.0;
    }

    try
    {
      pivot[1] = Billboard_PivotY->Text.ToDouble();
    }
    catch(...)
    {
      pivot[1] = 0.0;
    }

    try
    {
      pivot[2] = Billboard_PivotZ->Text.ToDouble();
    }
    catch(...)
    {
      pivot[2] = 0.0;
    }

    // --

    At3d_Measure_Billboard_Set_Pivot(itemIndex, MY_NULL_PTR, &position[0], &pivot[0]);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetHeight1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double height;

      if (At3d_Measure_Billboard_Get_Height(-1, buffer, &height) > 0)
      {
        Billboard_Height->Text = height;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetHeight2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    double height;

    if (At3d_Measure_Billboard_Get_Height(itemIndex, MY_NULL_PTR, &height) > 0)
    {
      Billboard_Height->Text = height;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetHeight1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double height;

      try
      {
        height = Billboard_Height->Text.ToDouble();
      }
      catch(...)
      {
        height = 1.0;
      }

      At3d_Measure_Billboard_Set_Height(-1, buffer, height);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetHeight2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    double height;

    try
    {
      height = Billboard_Height->Text.ToDouble();
    }
    catch(...)
    {
      height = 1.0;
    }

    At3d_Measure_Billboard_Set_Height(itemIndex, MY_NULL_PTR, height);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetIDClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    char buffer[LEN_LAYER_NAME];

    if (At3d_Measure_Billboard_Get_ID(itemIndex, buffer) > 0)
    {
      Billboard_ID->Text = AnsiString(buffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetIDClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID->Text);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if ( At3d_Measure_Billboard_Set_ID(itemIndex, buffer) > 0 )
    {
      Billboard_ID_List->Items->Strings[itemIndex] = buffer;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Image_From_DLL(Int32 index, char* idPtr)
{
  if (!dllOpen) return;

  // --

  if (billboardImagePtr)
  {
    delete[] billboardImagePtr;
    billboardImagePtr = MY_NULL_PTR;
  }

  if ( At3d_Measure_Billboard_Get_Image_Size(index, idPtr, &billboardImageWidth,
                                             &billboardImageHeight,
                                             &billboardAlphaFlag) > 0 )
  {
    Int32 rowBytes;

    if (billboardAlphaFlag)
    {
      rowBytes = billboardImageWidth * 4;
    }
    else
    {
      rowBytes = ((3 * billboardImageWidth + 3) / 4) * 4;
    }

    Int32 imageSize = rowBytes * billboardImageHeight;

    try
    {
      billboardImagePtr = new UInt8[static_cast<size_t>(imageSize)];
    }
    catch(...)
    {
      billboardImagePtr = MY_NULL_PTR;;
    }

    if (billboardImagePtr)
    {
      At3d_Measure_Billboard_Get_Image(index, idPtr, billboardImagePtr, imageSize);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetImage1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Get_Image_From_DLL(-1, buffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetImage2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Get_Image_From_DLL(itemIndex, MY_NULL_PTR);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetImage1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (!billboardImagePtr)
  {
    return;
  }

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      At3d_Measure_Billboard_Set_Image(-1, buffer, billboardImagePtr,
                                       billboardImageWidth, billboardImageHeight,
                                       billboardAlphaFlag);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetImage2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (!billboardImagePtr)
  {
    return;
  }

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    At3d_Measure_Billboard_Set_Image(itemIndex, MY_NULL_PTR, billboardImagePtr,
                                     billboardImageWidth, billboardImageHeight,
                                     billboardAlphaFlag);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::BillboardPosition_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      BillboardPositionX->Text = xyzArrayObserver[0];
      BillboardPositionY->Text = xyzArrayObserver[1];
      BillboardPositionZ->Text = xyzArrayObserver[2];
    }
    else
    {
      BillboardPositionX->Text = xyzArray1[0];
      BillboardPositionY->Text = xyzArray1[1];
      BillboardPositionZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::BillboardPosition_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    BillboardPositionX->Text = xyzArray2[0];
    BillboardPositionY->Text = xyzArray2[1];
    BillboardPositionZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::BillboardPivot_V1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      BillboardPivotX->Text = xyzArrayObserver[0];
      BillboardPivotY->Text = xyzArrayObserver[1];
      BillboardPivotZ->Text = xyzArrayObserver[2];
    }
    else
    {
      BillboardPivotX->Text = xyzArray1[0];
      BillboardPivotY->Text = xyzArray1[1];
      BillboardPivotZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::BillboardPivot_V2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    BillboardPivotX->Text = xyzArray2[0];
    BillboardPivotY->Text = xyzArray2[1];
    BillboardPivotZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_PivotTypeGet1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 pivotType = 0;

      if (At3d_Measure_Billboard_Get_Pivot_Type(-1, buffer, &pivotType) > 0)
      {
        Billboard_PivotTypeList->ItemIndex = pivotType;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_PivotTypeGet2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Int32 pivotType = 0;

    if (At3d_Measure_Billboard_Get_Pivot_Type(itemIndex, MY_NULL_PTR, &pivotType) > 0)
    {
      Billboard_PivotTypeList->ItemIndex = pivotType;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_PivotTypeSet1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 pivotType = Billboard_PivotTypeList->ItemIndex;

      if (pivotType > -1)
      {
        At3d_Measure_Billboard_Set_Pivot_Type(-1, buffer, pivotType);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_PivotTypeSet2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Int32 pivotType = Billboard_PivotTypeList->ItemIndex;

    if (pivotType > -1)
    {
      At3d_Measure_Billboard_Set_Pivot_Type(itemIndex, MY_NULL_PTR, pivotType);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetOglReplace1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 oglReplace = 0;

      if (At3d_Measure_Billboard_Get_OglReplace(-1, buffer, &oglReplace) > 0)
      {
        Billboard_OglReplace->Checked = oglReplace;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetOglReplace2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Int32 oglReplace;

    if (At3d_Measure_Billboard_Get_OglReplace(itemIndex, MY_NULL_PTR, &oglReplace) > 0)
    {
      Billboard_OglReplace->Checked = oglReplace;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetOglReplace1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      At3d_Measure_Billboard_Set_OglReplace(-1, buffer, (Billboard_OglReplace->Checked) ? 1 : 0);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetOglReplace2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    At3d_Measure_Billboard_Set_OglReplace(itemIndex, MY_NULL_PTR, (Billboard_OglReplace->Checked) ? 1 : 0);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetStatic1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 staticFlag = 0;

      if (At3d_Measure_Billboard_Get_Static(-1, buffer, &staticFlag) > 0)
      {
        Billboard_Static->Checked = staticFlag;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetStatic2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Int32 staticFlag;

    if (At3d_Measure_Billboard_Get_Static(itemIndex, MY_NULL_PTR, &staticFlag) > 0)
    {
      Billboard_Static->Checked = staticFlag;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetStatic1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      At3d_Measure_Billboard_Set_Static(-1, buffer, (Billboard_Static->Checked) ? 1 : 0);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetStatic2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    At3d_Measure_Billboard_Set_Static(itemIndex, MY_NULL_PTR, (Billboard_Static->Checked) ? 1 : 0);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetRotation1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double rotation[3];

      if (At3d_Measure_Billboard_Get_Static_Angles(-1, buffer, &rotation[0]) > 0)
      {
        Billboard_RotateX->Text = rotation[0];
        Billboard_RotateY->Text = rotation[1];
        Billboard_RotateZ->Text = rotation[2];
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetRotation2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    double rotation[3];

    if (At3d_Measure_Billboard_Get_Static_Angles(itemIndex, MY_NULL_PTR, &rotation[0]) > 0)
    {
      Billboard_RotateX->Text = rotation[0];
      Billboard_RotateY->Text = rotation[1];
      Billboard_RotateZ->Text = rotation[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetRotation1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double rotation[3];

      try
      {
        rotation[0] = Billboard_RotateX->Text.ToDouble();
      }
      catch(...)
      {
        rotation[0] = 0.0;
      }

      try
      {
        rotation[1] = Billboard_RotateY->Text.ToDouble();
      }
      catch(...)
      {
        rotation[1] = 0.0;
      }

      try
      {
        rotation[2] = Billboard_RotateZ->Text.ToDouble();
      }
      catch(...)
      {
        rotation[2] = 0.0;
      }

      At3d_Measure_Billboard_Set_Static_Angles(-1, buffer, &rotation[0]);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetRotation2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    double rotation[3];

    try
    {
      rotation[0] = Billboard_RotateX->Text.ToDouble();
    }
    catch(...)
    {
      rotation[0] = 0.0;
    }

    try
    {
      rotation[1] = Billboard_RotateY->Text.ToDouble();
    }
    catch(...)
    {
      rotation[1] = 0.0;
    }

    try
    {
      rotation[2] = Billboard_RotateZ->Text.ToDouble();
    }
    catch(...)
    {
      rotation[2] = 0.0;
    }

    At3d_Measure_Billboard_Set_Static_Angles(itemIndex, MY_NULL_PTR, &rotation[0]);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Copy_DeleteClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  // This function copies the billboard data from the DLL
  // and then the billboard is deleted from the DLL
  //
  if (billboardData.imagePtr)
  {
    delete[] billboardData.imagePtr;
    billboardData.imagePtr = MY_NULL_PTR;
    billboardData.imageSize = 0;
  }

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    my_strncpy(billboardData.id, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    billboardData.id[LEN_LAYER_NAME - 1] = '\0';

    Int32 index = (Billboard_Copy_Index->Checked) ? itemIndex : -1;

    if (index > -1) my_memset(billboardData.id, 0, LEN_LAYER_NAME);

    if (Billboard_Copy_Image->Checked)
    {
      Int32 width;
      Int32 height;
      Int32 alphaFlag;

      if ( At3d_Measure_Billboard_Get_Image_Size(index, billboardData.id,
                                                 &width, &height, &alphaFlag) > 0 )
      {
        Int32 rowBytes;

        if (alphaFlag)
        {
          rowBytes = width * 4;
        }
        else
        {
          rowBytes = ((3 * width + 3) / 4) * 4;
        }

        UInt8* imagePtr;
        Int32 imageSize = rowBytes * height;

        try
        {
          imagePtr = new UInt8[static_cast<size_t>(imageSize)];
        }
        catch(...)
        {
          imagePtr = MY_NULL_PTR;;
        }

        if (imagePtr)
        {
          billboardData.imagePtr  = imagePtr;
          billboardData.imageSize = imageSize;
        }
      }
    }

    if (At3d_Measure_Billboard_Get_Item(index, billboardData.id,
                                        billboardData.imagePtr,
                                        billboardData.imageSize,
                                        &billboardData.imageWidth,
                                        &billboardData.imageHeight,
                                        &billboardData.alphaFlag,
                                        &billboardData.pivotTypeFlag,
                                        &billboardData.staticFlag,
                                        &billboardData.visibleFlag,
                                        &billboardData.oglReplaceFlag,
                                        &billboardData.typeFlag,
                                        &billboardData.rgb_Line_Array[0],
                                        &billboardData.lineWidth,
                                        &billboardData.height,
                                        &billboardData.position[0],
                                        &billboardData.pivot[0],
                                        &billboardData.rotationStatic[0]) > 0)
    {
      if ( At3d_Measure_Billboard_Delete_Item(itemIndex, MY_NULL_PTR) > 0 )
      {
        Billboard_ID_List->Items->Delete(itemIndex);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_ReplaceClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  // This function copies the billboard data to an existing billboard in the DLL
  //
  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    Int32 index = (Billboard_Copy_Index->Checked) ? -1 : itemIndex;

    if (At3d_Measure_Billboard_Replace_Item(index, buffer,
                                            billboardData.id,
                                            billboardData.imagePtr,
                                            billboardData.imageWidth,
                                            billboardData.imageHeight,
                                            billboardData.alphaFlag,
                                            billboardData.pivotTypeFlag,
                                            billboardData.staticFlag,
                                            billboardData.visibleFlag,
                                            billboardData.oglReplaceFlag,
                                            billboardData.typeFlag,
                                            &billboardData.rgb_Line_Array[0],
                                            billboardData.lineWidth,
                                            billboardData.height,
                                            &billboardData.position[0],
                                            &billboardData.pivot[0],
                                            &billboardData.rotationStatic[0]) > 0)
    {
     Billboard_ID_List->Items->Strings[itemIndex] = billboardData.id;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Cancel_SelectedClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_Measure_Billboard_Cancel_Selected();
  Billboard_Selected_ID->Text = AnsiString();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_InvertSelectedColorClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_INVERT_COLOR_SELECTED_MEASURE_BILLBOARD,
                 (Billboard_InvertSelectedColor->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetColorClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  float color[3];

  try
  {
    color[0] = my_double_to_float(Billboard_Selected_Red->Text.ToDouble());
  }
  catch(...)
  {
    color[0] = 1.0f;
  }

  try
  {
    color[1] = my_double_to_float(Billboard_Selected_Green->Text.ToDouble());
  }
  catch(...)
  {
    color[1] = 1.0f;
  }

  try
  {
    color[2] = my_double_to_float(Billboard_Selected_Blue->Text.ToDouble());
  }
  catch(...)
  {
    color[2] = 1.0f;
  }

  At3d_Measure_Billboard_Set_Selected_Color(&color[0]);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetWidthClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  float width;

  try
  {
    width = my_double_to_float(Billboard_Selected_Width->Text.ToDouble());
  }
  catch(...)
  {
    width = 1.0f;
  }

  At3d_Measure_Billboard_Set_Selected_Width(width);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Line_Get1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      float rgbArray[3];
      float width;

      if (At3d_Measure_Billboard_Get_Line(-1, buffer, &rgbArray[0],
                                          &width) > 0)
      {
        Billboard_Line_Red->Text   = static_cast<double>(rgbArray[0]);
        Billboard_Line_Green->Text = static_cast<double>(rgbArray[1]);
        Billboard_Line_Blue->Text  = static_cast<double>(rgbArray[2]);

        Billboard_Line_Width->Text = static_cast<double>(width);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Line_Get2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    float rgbArray[3];
    float width;;

    if (At3d_Measure_Billboard_Get_Line(itemIndex, MY_NULL_PTR, &rgbArray[0], &width) > 0)
    {
      Billboard_Line_Red->Text   = static_cast<double>(rgbArray[0]);
      Billboard_Line_Green->Text = static_cast<double>(rgbArray[1]);
      Billboard_Line_Blue->Text  = static_cast<double>(rgbArray[2]);

      Billboard_Line_Width->Text = static_cast<double>(width);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Line_Set1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      float rgbArray[3];

      try
      {
        rgbArray[0] = my_double_to_float(Billboard_Line_Red->Text.ToDouble());
      }
      catch(...)
      {
        rgbArray[0] = 0.0f;
      }

      try
      {
        rgbArray[1] = my_double_to_float(Billboard_Line_Green->Text.ToDouble());
      }
      catch(...)
      {
        rgbArray[1] = 0.0f;
      }

      try
      {
        rgbArray[2] = my_double_to_float(Billboard_Line_Blue->Text.ToDouble());
      }
      catch(...)
      {
        rgbArray[2] = 0.0f;
      }

      // --

      float width;

      try
      {
        width = my_double_to_float(Billboard_Line_Width->Text.ToDouble());
      }
      catch(...)
      {
        width = 1.0f;
      }

      // --

      At3d_Measure_Billboard_Set_Line(-1, buffer, &rgbArray[0], width);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Line_Set2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    float rgbArray[3];

    try
    {
      rgbArray[0] = my_double_to_float(Billboard_Line_Red->Text.ToDouble());
    }
    catch(...)
    {
      rgbArray[0] = 0.0f;
    }

    try
    {
      rgbArray[1] = my_double_to_float(Billboard_Line_Green->Text.ToDouble());
    }
    catch(...)
    {
      rgbArray[1] = 0.0f;
    }

    try
    {
      rgbArray[2] = my_double_to_float(Billboard_Line_Blue->Text.ToDouble());
    }
    catch(...)
    {
      rgbArray[2] = 0.0f;
    }

    // --

    float width;

    try
    {
      width = my_double_to_float(Billboard_Line_Width->Text.ToDouble());
    }
    catch(...)
    {
      width = 1.0f;
    }

    // --

    At3d_Measure_Billboard_Set_Line(itemIndex, MY_NULL_PTR, &rgbArray[0], width);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Get_CountClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 count = At3d_Measure_Billboard_Get_Count();

  Billboard_Count->Text = count;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetType1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 type;

      if (At3d_Measure_Billboard_Get_Type(-1, buffer, &type) > 0)
      {
        Billboard_Type->Text = type;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetType2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Int32 type;

    if (At3d_Measure_Billboard_Get_Type(itemIndex, MY_NULL_PTR, &type) > 0)
    {
      Billboard_Type->Text = type;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetType1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 type;

      try
      {
        type = Billboard_Type->Text.ToInt();
      }
      catch(...)
      {
        type = 0;
      }

      At3d_Measure_Billboard_Set_Type(-1, buffer, type);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetType2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Int32 type;

    try
    {
      type = Billboard_Type->Text.ToInt();
    }
    catch(...)
    {
      type = 0;
    }

    At3d_Measure_Billboard_Set_Type(itemIndex, MY_NULL_PTR, type);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_LocationClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 screenX;
  Int32 screenY;

  try
  {
    screenX = Get_ScreenX->Text.ToInt();
  }
  catch(...)
  {
    screenX = 0;
  }

  try
  {
    screenY = Get_ScreenY->Text.ToInt();
  }
  catch(...)
  {
    screenY = 0;
  }

  Int32 hitFlag;
  double xyzArray[3];
  float rgbArray[3];
  char layerName[LEN_LAYER_NAME];
  char cloneItemName[LEN_LAYER_NAME];

  Int32 result = At3d_Get_Mesh_Location_At_Point(screenX, screenY, &hitFlag,
                                                 &xyzArray[0], &layerName[0],
                                                 &cloneItemName[0], &rgbArray[0]);

  if (result)
  {
    if (hitFlag == 2)
    {
      Get_MeshX->Caption = xyzArray[0];
      Get_MeshY->Caption = xyzArray[1];
      Get_MeshZ->Caption = xyzArray[2];

      Get_LayerName->Caption = layerName;
      Get_CloneItemName->Caption = cloneItemName;

      Get_Red1->Text   = my_change_float_to_256_int(rgbArray[0]);
      Get_Green1->Text = my_change_float_to_256_int(rgbArray[1]);
      Get_Blue1->Text  = my_change_float_to_256_int(rgbArray[2]);

      Get_Red2->Text   = rgbArray[0];
      Get_Green2->Text = rgbArray[1];
      Get_Blue2->Text  = rgbArray[2];
    }
    else
    {
      Get_MeshX->Caption = "";
      Get_MeshY->Caption = "";
      Get_MeshZ->Caption = "";

      Get_LayerName->Caption = "";
      Get_CloneItemName->Caption = "";

      Get_Red1->Text   = "";
      Get_Green1->Text = "";
      Get_Blue1->Text  = "";

      Get_Red2->Text   = "";
      Get_Green2->Text = "";
      Get_Blue2->Text  = "";
    }
  }
  else
  {
    Get_MeshX->Caption = "";
    Get_MeshY->Caption = "";
    Get_MeshZ->Caption = "";

    Get_LayerName->Caption = "";
    Get_CloneItemName->Caption = "";

    Get_Red1->Text   = "";
    Get_Green1->Text = "";
    Get_Blue1->Text  = "";

    Get_Red2->Text   = "";
    Get_Green2->Text = "";
    Get_Blue2->Text  = "";
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Integrated_GPUClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_INTEGRATED_GPU, (Integrated_GPU->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_Depth_TestClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_DEPTH_TEST, (Locations_Depth_Test->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Intersection_PointClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  double v1[3];

  try
  {
    v1[0] = LineMesh_V1_X->Text.ToDouble();
  }
  catch(...)
  {
    v1[0] = 0.0;
  }

  try
  {
    v1[1] = LineMesh_V1_Y->Text.ToDouble();
  }
  catch(...)
  {
    v1[1] = 0.0;
  }

  try
  {
    v1[2] = LineMesh_V1_Z->Text.ToDouble();
  }
  catch(...)
  {
    v1[2] = 0.0;
  }

  // --
  double v2[3];

  try
  {
    v2[0] = LineMesh_V2_X->Text.ToDouble();
  }
  catch(...)
  {
    v2[0] = 0.0;
  }

  try
  {
    v2[1] = LineMesh_V2_Y->Text.ToDouble();
  }
  catch(...)
  {
    v2[1] = 0.0;
  }

  try
  {
    v2[2] = LineMesh_V2_Z->Text.ToDouble();
  }
  catch(...)
  {
    v2[2] = 0.0;
  }

  // --

  double skipDistance;

  try
  {
    skipDistance = LineMesh_SkipDistance->Text.ToDouble();
  }
  catch(...)
  {
    skipDistance = 0.0;
  }

  // --

  double intersection[3];

  if ( At3d_Get_Line_Mesh_Intersection_Point(&v1[0], &v2[0], skipDistance, &intersection[0]) > 0 )
  {
    LineMesh_Intersection_X->Text = intersection[0];
    LineMesh_Intersection_Y->Text = intersection[1];
    LineMesh_Intersection_Z->Text = intersection[2];
  }
  else
  {
    LineMesh_Intersection_X->Text = AnsiString();
    LineMesh_Intersection_Y->Text = AnsiString();
    LineMesh_Intersection_Z->Text = AnsiString();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReadAddPosition_V1Click(TObject *Sender)
{
   sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      ReadAddPositionX->Text = xyzArrayObserver[0];
      ReadAddPositionY->Text = xyzArrayObserver[1];
      ReadAddPositionZ->Text = xyzArrayObserver[2];
    }
    else
    {
      ReadAddPositionX->Text = xyzArray1[0];
      ReadAddPositionY->Text = xyzArray1[1];
      ReadAddPositionZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReadAddPosition_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    ReadAddPositionX->Text = xyzArray2[0];
    ReadAddPositionY->Text = xyzArray2[1];
    ReadAddPositionZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read_Add_Static_MeshClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Screen->Cursor = TCursor(crHourGlass);
  Read_Add_File();
  Screen->Cursor = TCursor(crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read_Add_File(void)
{
  if (!dllOpen) return;

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if (inName[0] != 0)
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ext0);
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr;
  const char* inputExtensionPtr;
  Int32 inputType;

  if (!inputFileType)
  {
    inputNamePtr      = "Wavefront";
    inputExtensionPtr = ".obj";
    inputType         = INTYPE_WAVEFRONT;
  }
  else if (inputFileType == 1)
  {
    inputNamePtr      = "VRML";
    inputExtensionPtr = ".wrl";
    inputType         = INTYPE_VRML;
  }
  else
  {
    inputNamePtr      = "AT3D Mesh";
    inputExtensionPtr = ".atm";
    inputType         = INTYPE_AT3D_MESH;
  }

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(inName, MAX_PATH, fileName.c_str());

    // before reading make sure flags are set properly
    // (note: flags maintain their setting until changed with this function)
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_VERTICES, 1);
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_FACES, 1);
    At3d_WriteFlag(LIBFLAG_SHOW_READ_MESSAGES, 0);

    // --

    double scaleFactor;

    try
    {
      scaleFactor = ReadAddScaleFactor->Text.ToDouble();
    }
    catch(...)
    {
      scaleFactor = 1.0;
    }

    // --

    Int32 xRotationFlag;

    if (ReadAddRotateX_No->Checked)
    {
      xRotationFlag = ROTATE_ON_X_AXIS_NO;
    }
    else if (ReadAddRotateX_Minus90->Checked)
    {
      xRotationFlag = ROTATE_ON_X_AXIS_MINUS_90;
    }
    else if (ReadAddRotateX_Plus90->Checked)
    {
      xRotationFlag = ROTATE_ON_X_AXIS_PLUS_90;
    }
    else
    {
      xRotationFlag = ROTATE_ON_X_AXIS_180;
    }

    // --

    double position[3];

    try
    {
      position[0] = ReadAddPositionX->Text.ToDouble();
    }
    catch(...)
    {
      position[0] = 0.0;
    }

    try
    {
      position[1] = ReadAddPositionY->Text.ToDouble();
    }
    catch(...)
    {
      position[1] = 0.0;
    }

    try
    {
      position[2] = ReadAddPositionZ->Text.ToDouble();
    }
    catch(...)
    {
      position[2] = 0.0;
    }

    // --

    AnsiString tempPrefix;
    my_copystring_to_ansi_from_vcl(&tempPrefix, ReadAddPrefix->Text);

    char prefixBuffer[LEN_LAYER_NAME];
    my_strncpy(prefixBuffer, LEN_LAYER_NAME, tempPrefix.c_str(), LEN_LAYER_NAME - 1);
    prefixBuffer[LEN_LAYER_NAME - 1] = '\0';

    // --

    // read and add file
    //
    // the file read can contain more than one layer
    // all layers read can be saved to an OBJ or other 3D file if the layers are turned on
    //
    // a user defined prefix is added to every layer name
    // an underscore, '_', is automatically added to the prefix
    // this prefex is used with all functions that work with 'mesh static'
    //
    // the local origin of the object is placed at 'position'
    //
    // return value for all 'mesh static' functions
    // 0 = failure
    // 1 = success
    //
    if (At3d_Mesh_Static_Read_And_Add(inputType, inName, scaleFactor,
                                      xRotationFlag, &position[0], prefixBuffer))
    {
      AdjustMesh_List->Items->Add(prefixBuffer);
    }
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Pivot_CalculateClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      // when the pivot is calculated, X and Y are placed at the mid-point on
      // each axis and Z is placed at the lowest Z elevation
      //
      // if the original location of the pivot point is required, all
      // changes for it should be tracked and saved by the app as it can not
      // be saved to an OBJ or other 3D file.
      //
      double pivot[3];
      if (At3d_Mesh_Static_Calculate_Pivot(buffer, &pivot[0]) > 0)
      {
        AdjustMesh_PivotX->Text = pivot[0];
        AdjustMesh_PivotY->Text = pivot[1];
        AdjustMesh_PivotZ->Text = pivot[2];
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Pivot_GetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double pivot[3];
      if (At3d_Mesh_Static_Get_Pivot(buffer, &pivot[0]) > 0)
      {
        AdjustMesh_PivotX->Text = pivot[0];
        AdjustMesh_PivotY->Text = pivot[1];
        AdjustMesh_PivotZ->Text = pivot[2];
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Pivot_SetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  double pivot[3];

  try
  {
    pivot[0] = AdjustMesh_PivotX->Text.ToDouble();
  }
  catch(...)
  {
    pivot[0] = 0.0;
  }

  try
  {
    pivot[1] = AdjustMesh_PivotY->Text.ToDouble();
  }
  catch(...)
  {
    pivot[1] = 0.0;
  }

  try
  {
    pivot[2] = AdjustMesh_PivotZ->Text.ToDouble();
  }
  catch(...)
  {
    pivot[2] = 0.0;
  }

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Mesh_Static_Set_Pivot(buffer, &pivot[0]) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_RotateClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  double rotate[3];

  try
  {
    rotate[0] = AdjustMesh_RotateX->Text.ToDouble();
  }
  catch(...)
  {
    rotate[0] = 0.0;
  }

  try
  {
    rotate[1] = AdjustMesh_RotateY->Text.ToDouble();
  }
  catch(...)
  {
    rotate[1] = 0.0;
  }

  try
  {
    rotate[2] = AdjustMesh_RotateZ->Text.ToDouble();
  }
  catch(...)
  {
    rotate[2] = 0.0;
  }

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Mesh_Static_Rotate(buffer, &rotate[0]) > 0)
      {
        AdjustMesh_SizeClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_ScaleClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  double scale[3];

  try
  {
    scale[0] = AdjustMesh_ScaleX->Text.ToDouble();
  }
  catch(...)
  {
    scale[0] = 1.0;
  }

  try
  {
    scale[1] = AdjustMesh_ScaleY->Text.ToDouble();
  }
  catch(...)
  {
    scale[1] = 1.0;
  }

  try
  {
    scale[2] = AdjustMesh_ScaleZ->Text.ToDouble();
  }
  catch(...)
  {
    scale[2] = 1.0;
  }

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Mesh_Static_Scale(buffer, &scale[0]) > 0)
      {
        AdjustMesh_SizeClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_AdjustMesh_Translate(Int32 relativeFlag)
{
  if (!dllOpen) return;

  // --

  double translate[3];

  try
  {
    translate[0] = AdjustMesh_TranslateX->Text.ToDouble();
  }
  catch(...)
  {
    translate[0] = 0.0;
  }

  try
  {
    translate[1] = AdjustMesh_TranslateY->Text.ToDouble();
  }
  catch(...)
  {
    translate[1] = 1.0;
  }

  try
  {
    translate[2] = AdjustMesh_TranslateZ->Text.ToDouble();
  }
  catch(...)
  {
    translate[2] = 1.0;
  }

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Mesh_Static_Translate(buffer, relativeFlag, &translate[0]) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Translate_RelativeClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_AdjustMesh_Translate(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_TranslateClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_AdjustMesh_Translate(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_DeleteClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Mesh_Static_Delete(buffer) > 0)
      {
        AdjustMesh_List->Items->Delete(itemIndex);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_OnClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Mesh_Static_Set_Visible(buffer, 1) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_OffClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Mesh_Static_Set_Visible(buffer, 0) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_SizeClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double size[3];
      if (At3d_Mesh_Static_Get_Size(buffer, &size[0]) > 0)
      {
        AdjustMesh_SizeX->Text = size[0];
        AdjustMesh_SizeY->Text = size[1];
        AdjustMesh_SizeZ->Text = size[2];
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Pivot_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      AdjustMesh_PivotX->Text = xyzArrayObserver[0];
      AdjustMesh_PivotY->Text = xyzArrayObserver[1];
      AdjustMesh_PivotZ->Text = xyzArrayObserver[2];
    }
    else
    {
      AdjustMesh_PivotX->Text = xyzArray1[0];
      AdjustMesh_PivotY->Text = xyzArray1[1];
      AdjustMesh_PivotZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Pivot_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    AdjustMesh_PivotX->Text = xyzArray2[0];
    AdjustMesh_PivotY->Text = xyzArray2[1];
    AdjustMesh_PivotZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Translate_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      AdjustMesh_TranslateX->Text = xyzArrayObserver[0];
      AdjustMesh_TranslateY->Text = xyzArrayObserver[1];
      AdjustMesh_TranslateZ->Text = xyzArrayObserver[2];
    }
    else
    {
      AdjustMesh_TranslateX->Text = xyzArray1[0];
      AdjustMesh_TranslateY->Text = xyzArray1[1];
      AdjustMesh_TranslateZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Translate_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    AdjustMesh_TranslateX->Text = xyzArray2[0];
    AdjustMesh_TranslateY->Text = xyzArray2[1];
    AdjustMesh_TranslateZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_AddClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Screen->Cursor = TCursor(crHourGlass);
  Add_Clone_Mesh(0);
  Screen->Cursor = TCursor(crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_ChangeClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Screen->Cursor = TCursor(crHourGlass);
  Add_Clone_Mesh(1);
  Screen->Cursor = TCursor(crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_Clone_Mesh(Int32 changeFlag)
{
  if (!dllOpen) return;

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if (inName[0] != 0)
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ext0);
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr;
  const char* inputExtensionPtr;
  //Int32 inputType;

  if (!inputFileType)
  {
    inputNamePtr      = "Wavefront";
    inputExtensionPtr = ".obj";
    //inputType         = INTYPE_WAVEFRONT;
  }
  else if (inputFileType == 1)
  {
    inputNamePtr      = "VRML";
    inputExtensionPtr = ".wrl";
    //inputType         = INTYPE_VRML;
  }
  else
  {
    inputNamePtr      = "AT3D Mesh";
    inputExtensionPtr = ".atm";
    //inputType         = INTYPE_AT3D_MESH;
  }

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(inName, MAX_PATH, fileName.c_str());

    // before reading make sure flags are set properly
    // (note: flags maintain their setting until changed with this function)
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_VERTICES, 1);
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_FACES, 1);
    At3d_WriteFlag(LIBFLAG_SHOW_READ_MESSAGES, 0);

    // --

    double scaleFactor;

    try
    {
      scaleFactor = Add_CloneMesh_ScaleFactor->Text.ToDouble();
    }
    catch(...)
    {
      scaleFactor = 1.0;
    }

    // --

    Int32 xRotationFlag;

    if (Add_CloneMesh_RotateX_No->Checked)
    {
      xRotationFlag = ROTATE_ON_X_AXIS_NO;
    }
    else if (Add_CloneMesh_RotateX_Minus90->Checked)
    {
      xRotationFlag = ROTATE_ON_X_AXIS_MINUS_90;
    }
    else if (Add_CloneMesh_RotateX_Plus90->Checked)
    {
      xRotationFlag = ROTATE_ON_X_AXIS_PLUS_90;
    }
    else
    {
      xRotationFlag = ROTATE_ON_X_AXIS_180;
    }

    // --

    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Add_CloneMesh_Name->Text);

    char nameBuffer[LEN_LAYER_NAME];
    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';
    At3d_CheckName(nameBuffer);
    Add_CloneMesh_Name->Text = nameBuffer;

    // --

    // add clone mesh
    //
    // the file containing the clone mesh can contain more than one layer
    //
    // meshes are displayed using clone items
    //
    // return value for all 'clone' functions
    // 0 = failure
    // 1 = success
    //
    if (changeFlag)
    {
      if (At3d_Clone_Mesh_Change_File(nameBuffer, inName, scaleFactor, xRotationFlag))
      {
        //
      }
    }
    else
    {
      if (At3d_Clone_Mesh_Add(nameBuffer, inName, scaleFactor, xRotationFlag))
      {
        CloneMesh_List->Items->Add(nameBuffer);
      }
    }
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_DeleteOneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Clone_Mesh_Delete(buffer) > 0)
      {
        CloneMesh_List->Items->Delete(itemIndex);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_DeleteAllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (At3d_Clone_Mesh_Delete_All() > 0)
  {
    CloneMesh_List->Clear();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_Change_MeshNameClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_List->Items->Strings[itemIndex]);

    char meshNameBuffer[LEN_LAYER_NAME];
    my_strncpy(meshNameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    meshNameBuffer[LEN_LAYER_NAME - 1] = '\0';

    // --

    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_New_MeshName->Text);

    char newMeshName[LEN_LAYER_NAME];
    my_strncpy(newMeshName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    newMeshName[LEN_LAYER_NAME - 1] = '\0';
    At3d_CheckName(newMeshName);
    CloneMesh_New_MeshName->Text = newMeshName;

    if ( (*meshNameBuffer != '\0') &&
         (*newMeshName  != '\0') )
    {
      if (At3d_Clone_Mesh_Change_MeshName(meshNameBuffer, newMeshName) > 0)
      {
        CloneMesh_List->Items->Strings[itemIndex] = newMeshName;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_GetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      char filename[LEN_FILE_PATH];
      Int32 xRotationFlag;
      double scaleFactor;

      if (At3d_Clone_Mesh_Get_Data(buffer, filename, &scaleFactor, &xRotationFlag) > 0)
      {
        Data_CloneMesh_Filename->Text = filename;
        Data_CloneMesh_ScaleFactor->Text = scaleFactor;
        Data_CloneMesh_RotationX->Text = xRotationFlag;
      }
      else
      {
        Data_CloneMesh_Filename->Text = "";
        Data_CloneMesh_ScaleFactor->Text = "";
        Data_CloneMesh_RotationX->Text = "";
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_Size_GetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double size[3];
      double scaleFactor;

      if (At3d_Clone_Mesh_Get_Size(buffer, &size[0], &scaleFactor) > 0)
      {
        CloneMesh_SizeX->Text = size[0];
        CloneMesh_SizeY->Text = size[1];
        CloneMesh_SizeZ->Text = size[2];

        CloneMesh_Old_ScaleFactor->Text = scaleFactor;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_Size_SetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double scaleFactor;

      try
      {
        scaleFactor = CloneMesh_New_ScaleFactor->Text.ToDouble();
      }
      catch(...)
      {
        scaleFactor = 1.0;
      }

      if (At3d_Clone_Mesh_Set_ScaleFactor(buffer, scaleFactor) > 0)
      {
        CloneMesh_GetClick(MY_NULL_PTR);
        CloneMesh_Size_GetClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_RotateX_RotateClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 xRotationFlag;

      if (CloneMesh_RotateX_Minus90->Checked)
      {
        xRotationFlag = ROTATE_ON_X_AXIS_MINUS_90;
      }
      else if (CloneMesh_RotateX_Plus90->Checked)
      {
        xRotationFlag = ROTATE_ON_X_AXIS_PLUS_90;
      }
      else if (CloneMesh_RotateX_180->Checked)
      {
        xRotationFlag = ROTATE_ON_X_AXIS_180;
      }
      else
      {
        xRotationFlag = ROTATE_ON_X_AXIS_NO;
      }

      if (At3d_Clone_Mesh_RotateX(buffer, xRotationFlag) > 0)
      {
        CloneMesh_GetClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_CloneItem_GetMeshNameClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    Add_CloneItem_MeshName->Text = CloneMesh_List->Items->Strings[itemIndex];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_CloneItem_Location_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      Add_CloneItem_LocationX->Text = xyzArrayObserver[0];
      Add_CloneItem_LocationY->Text = xyzArrayObserver[1];
      Add_CloneItem_LocationZ->Text = xyzArrayObserver[2];
    }
    else
    {
      Add_CloneItem_LocationX->Text = xyzArray1[0];
      Add_CloneItem_LocationY->Text = xyzArray1[1];
      Add_CloneItem_LocationZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_CloneItem_Location_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    Add_CloneItem_LocationX->Text = xyzArray2[0];
    Add_CloneItem_LocationY->Text = xyzArray2[1];
    Add_CloneItem_LocationZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_CloneItem_AddClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Screen->Cursor = TCursor(crHourGlass);
  Add_Clone_Item();
  Screen->Cursor = TCursor(crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_Clone_Item(void)
{
  if (!dllOpen) return;

  // --

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, Add_CloneItem_ItemName->Text);

  char itemNameBuffer[LEN_LAYER_NAME];
  my_strncpy(itemNameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  itemNameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(itemNameBuffer);
  Add_CloneItem_ItemName->Text = itemNameBuffer;

  // --

  my_copystring_to_ansi_from_vcl(&tempName, Add_CloneItem_MeshName->Text);

  char meshNameBuffer[LEN_LAYER_NAME];
  my_strncpy(meshNameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  meshNameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(meshNameBuffer);
  Add_CloneItem_MeshName->Text = meshNameBuffer;

  // --

  double location[3];

  try
  {
    location[0] = Add_CloneItem_LocationX->Text.ToDouble();
  }
  catch(...)
  {
    location[0] = 0.0;
  }

  try
  {
    location[1] = Add_CloneItem_LocationY->Text.ToDouble();
  }
  catch(...)
  {
    location[1] = 0.0;
  }

  try
  {
    location[2] = Add_CloneItem_LocationZ->Text.ToDouble();
  }
  catch(...)
  {
    location[2] = 0.0;
  }

  // --

  double rotation[3];

  try
  {
    rotation[0] = Add_CloneItem_RotationX->Text.ToDouble();
  }
  catch(...)
  {
    rotation[0] = 0.0;
  }

  try
  {
    rotation[1] = Add_CloneItem_RotationY->Text.ToDouble();
  }
  catch(...)
  {
    rotation[1] = 0.0;
  }

  try
  {
    rotation[2] = Add_CloneItem_RotationZ->Text.ToDouble();
  }
  catch(...)
  {
    rotation[2] = 0.0;
  }

  // --

  double scaleFactor;

  try
  {
    scaleFactor = Add_CloneItem_ScaleFactor->Text.ToDouble();
  }
  catch(...)
  {
    scaleFactor = 1.0;
  }

  // --

  Int32 visibleFlag = (Add_CloneItem_Visible->Checked) ? 1 : 0;

  // --

  if ( (*itemNameBuffer != '\0') &&
       (*meshNameBuffer != '\0') )
  {
    if (At3d_Clone_Item_Add(itemNameBuffer, meshNameBuffer,
                            &location[0], &rotation[0], scaleFactor, visibleFlag))
    {
      CloneItem_List->Items->Add(itemNameBuffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Change_ItemNameClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemNameBuffer[LEN_LAYER_NAME];
    my_strncpy(itemNameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemNameBuffer[LEN_LAYER_NAME - 1] = '\0';

    // --

    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_New_ItemName->Text);

    char newItemName[LEN_LAYER_NAME];
    my_strncpy(newItemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    newItemName[LEN_LAYER_NAME - 1] = '\0';
    At3d_CheckName(newItemName);
    CloneItem_New_ItemName->Text = newItemName;

    if ( (*itemNameBuffer != '\0') &&
         (*newItemName  != '\0') )
    {
      if (At3d_Clone_Item_Change_ItemName(itemNameBuffer, newItemName) > 0)
      {
        CloneItem_List->Items->Strings[itemIndex] = newItemName;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_GetMeshNameClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  CloneItem_MeshName->Text = "";

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemName[LEN_LAYER_NAME];
    my_strncpy(itemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemName[LEN_LAYER_NAME - 1] = '\0';

    if (*itemName != '\0')
    {
      char meshName[LEN_LAYER_NAME];

      if (At3d_Clone_Item_Get_Mesh(itemName, meshName) > 0)
      {
        CloneItem_MeshName->Text = meshName;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Change_MeshNameClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemName[LEN_LAYER_NAME];
    my_strncpy(itemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemName[LEN_LAYER_NAME - 1] = '\0';

    // --

    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_MeshName->Text);

    char meshName[LEN_LAYER_NAME];
    my_strncpy(meshName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    meshName[LEN_LAYER_NAME - 1] = '\0';
    At3d_CheckName(meshName);
    CloneItem_MeshName->Text = meshName;

    if ( (*itemName != '\0') &&
         (*meshName != '\0') )
    {
      if (At3d_Clone_Item_Change_Mesh(itemName, meshName) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_DeleteOneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Clone_Item_Delete(buffer) > 0)
      {
        CloneItem_List->Items->Delete(itemIndex);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_DeleteAllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (At3d_Clone_Item_Delete_All() > 0)
  {
    CloneItem_List->Clear();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_GetDataClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemName[LEN_LAYER_NAME];
    my_strncpy(itemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemName[LEN_LAYER_NAME - 1] = '\0';

    if (*itemName != '\0')
    {
      char meshName[LEN_LAYER_NAME];
      Int32 visibleFlag;
      double location[3];
      double rotation[3];
      double scaleFactor;

      if (At3d_Clone_Item_Get_Data(itemName, meshName,
                                   &location[0], &rotation[0],
                                   &scaleFactor, &visibleFlag) > 0)
      {
        Data_CloneItem_MeshName->Text = meshName;

        Data_CloneItem_LocationX->Text = location[0];
        Data_CloneItem_LocationY->Text = location[1];
        Data_CloneItem_LocationZ->Text = location[2];

        Data_CloneItem_RotationX->Text = rotation[0];
        Data_CloneItem_RotationY->Text = rotation[1];
        Data_CloneItem_RotationZ->Text = rotation[2];

        Data_CloneItem_ScaleFactor->Text = scaleFactor;

        Data_CloneItem_Visible->Checked = (visibleFlag) ? 1 : 0;
      }
      else
      {
        Data_CloneItem_MeshName->Text = "";

        Data_CloneItem_LocationX->Text = "";
        Data_CloneItem_LocationY->Text = "";
        Data_CloneItem_LocationZ->Text = "";

        Data_CloneItem_RotationX->Text = "";
        Data_CloneItem_RotationY->Text = "";
        Data_CloneItem_RotationZ->Text = "";

        Data_CloneItem_ScaleFactor->Text = 1.0;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Rotate_RelativeClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_CloneItem_Rotate(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_RotateClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_CloneItem_Rotate(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_CloneItem_Rotate(Int32 relativeFlag)
{
  if (!dllOpen) return;

  // --

  double rotate[3];

  try
  {
    rotate[0] = CloneItem_RotateX->Text.ToDouble();
  }
  catch(...)
  {
    rotate[0] = 0.0;
  }

  try
  {
    rotate[1] = CloneItem_RotateY->Text.ToDouble();
  }
  catch(...)
  {
    rotate[1] = 0.0;
  }

  try
  {
    rotate[2] = CloneItem_RotateZ->Text.ToDouble();
  }
  catch(...)
  {
    rotate[2] = 0.0;
  }

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Clone_Item_Rotate(buffer, relativeFlag, &rotate[0]) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Translate_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      CloneItem_TranslateX->Text = xyzArrayObserver[0];
      CloneItem_TranslateY->Text = xyzArrayObserver[1];
      CloneItem_TranslateZ->Text = xyzArrayObserver[2];
    }
    else
    {
      CloneItem_TranslateX->Text = xyzArray1[0];
      CloneItem_TranslateY->Text = xyzArray1[1];
      CloneItem_TranslateZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Translate_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    CloneItem_TranslateX->Text = xyzArray2[0];
    CloneItem_TranslateY->Text = xyzArray2[1];
    CloneItem_TranslateZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Translate_RelativeClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_CloneItem_Translate(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_TranslateClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_CloneItem_Translate(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_CloneItem_Translate(Int32 relativeFlag)
{
  if (!dllOpen) return;

  // --

  double translate[3];

  try
  {
    translate[0] = CloneItem_TranslateX->Text.ToDouble();
  }
  catch(...)
  {
    translate[0] = 0.0;
  }

  try
  {
    translate[1] = CloneItem_TranslateY->Text.ToDouble();
  }
  catch(...)
  {
    translate[1] = 0.0;
  }

  try
  {
    translate[2] = CloneItem_TranslateZ->Text.ToDouble();
  }
  catch(...)
  {
    translate[2] = 0.0;
  }

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Clone_Item_Translate(buffer, relativeFlag, &translate[0]) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Size_CloneItem_GetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemName[LEN_LAYER_NAME];
    my_strncpy(itemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemName[LEN_LAYER_NAME - 1] = '\0';

    if (*itemName != '\0')
    {
      double originalSize[3];
      double currentSize[3];
      double scaleFactor;

      if (At3d_Clone_Item_Get_Size(itemName, &originalSize[0],
                                   &currentSize[0], &scaleFactor) > 0)
      {
        Size_CloneItem_OriginalX->Text = originalSize[0];
        Size_CloneItem_OriginalY->Text = originalSize[1];
        Size_CloneItem_OriginalZ->Text = originalSize[2];

        Size_CloneItem_CurrentX->Text = currentSize[0];
        Size_CloneItem_CurrentY->Text = currentSize[1];
        Size_CloneItem_CurrentZ->Text = currentSize[2];

        Size_CloneItem_Old_ScaleFactor->Text = scaleFactor;
      }
      else
      {
        Size_CloneItem_OriginalX->Text = "";
        Size_CloneItem_OriginalY->Text = "";
        Size_CloneItem_OriginalZ->Text = "";

        Size_CloneItem_CurrentX->Text = "";
        Size_CloneItem_CurrentY->Text = "";
        Size_CloneItem_CurrentZ->Text = "";

        Size_CloneItem_Old_ScaleFactor->Text = "";
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Size_CloneItem_SetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemName[LEN_LAYER_NAME];
    my_strncpy(itemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemName[LEN_LAYER_NAME - 1] = '\0';

    if (*itemName != '\0')
    {
      double scaleFactor;

      try
      {
        scaleFactor = Size_CloneItem_New_ScaleFactor->Text.ToDouble();
      }
      catch(...)
      {
        scaleFactor = 1.0;
      }

      if (At3d_Clone_Item_Set_ScaleFactor(itemName, scaleFactor) > 0)
      {
        Size_CloneItem_GetClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Snap_CloneItem_From_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      Snap_CloneItem_FromX->Text = xyzArrayObserver[0];
      Snap_CloneItem_FromY->Text = xyzArrayObserver[1];
      Snap_CloneItem_FromZ->Text = xyzArrayObserver[2];
    }
    else
    {
      Snap_CloneItem_FromX->Text = xyzArray1[0];
      Snap_CloneItem_FromY->Text = xyzArray1[1];
      Snap_CloneItem_FromZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Snap_CloneItem_From_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    Snap_CloneItem_FromX->Text = xyzArray2[0];
    Snap_CloneItem_FromY->Text = xyzArray2[1];
    Snap_CloneItem_FromZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Snap_CloneItem_To_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      Snap_CloneItem_ToX->Text = xyzArrayObserver[0];
      Snap_CloneItem_ToY->Text = xyzArrayObserver[1];
      Snap_CloneItem_ToZ->Text = xyzArrayObserver[2];
    }
    else
    {
      Snap_CloneItem_ToX->Text = xyzArray1[0];
      Snap_CloneItem_ToY->Text = xyzArray1[1];
      Snap_CloneItem_ToZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Snap_CloneItem_To_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    Snap_CloneItem_ToX->Text = xyzArray2[0];
    Snap_CloneItem_ToY->Text = xyzArray2[1];
    Snap_CloneItem_ToZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Snap_CloneItem_SnapClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemName[LEN_LAYER_NAME];
    my_strncpy(itemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemName[LEN_LAYER_NAME - 1] = '\0';

    if (*itemName != '\0')
    {
      double fromLocation[3];

      try
      {
        fromLocation[0] = Snap_CloneItem_FromX->Text.ToDouble();
      }
      catch(...)
      {
        fromLocation[0] = 0.0;
      }

      try
      {
        fromLocation[1] = Snap_CloneItem_FromY->Text.ToDouble();
      }
      catch(...)
      {
        fromLocation[1] = 0.0;
      }

      try
      {
        fromLocation[2] = Snap_CloneItem_FromZ->Text.ToDouble();
      }
      catch(...)
      {
        fromLocation[2] = 0.0;
      }

      // --

      double toLocation[3];

      try
      {
        toLocation[0] = Snap_CloneItem_ToX->Text.ToDouble();
      }
      catch(...)
      {
        toLocation[0] = 0.0;
      }

      try
      {
        toLocation[1] = Snap_CloneItem_ToY->Text.ToDouble();
      }
      catch(...)
      {
        toLocation[1] = 0.0;
      }

      try
      {
        toLocation[2] = Snap_CloneItem_ToZ->Text.ToDouble();
      }
      catch(...)
      {
        toLocation[2] = 0.0;
      }

      // --

      if (At3d_Clone_Item_Snap(itemName, &fromLocation[0], &toLocation[0]) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Visible_OnClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Clone_Item_Set_Visible(buffer, 1) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Visible_OffClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Clone_Item_Set_Visible(buffer, 0) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReadClonesFileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (clonesFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(clonesFilename, MAX_PATH, "%s%sClones-%s.txt", drive0, dir0, name0);
  }

  if (clonesFilename[0] != '\0')
  {
    my_splitpath(clonesFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Clones";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  Int32 result = 0;
  Int32 oldMeshCount = At3d_Clone_Mesh_Get_Count();
  Int32 oldItemCount = At3d_Clone_Item_Get_Count();

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(clonesFilename, MAX_PATH, fileName.c_str());


    result = At3d_Clones_Read_File(clonesFilename);
  }

  delete dlg;

  if (result)
  {
    Int32 newMeshCount = At3d_Clone_Mesh_Get_Count();
    Int32 newItemCount = At3d_Clone_Item_Get_Count();

    char name[LEN_LAYER_NAME];

    if (newMeshCount > oldMeshCount)
    {
      for (Int32 i=oldMeshCount; i<newMeshCount; i++)
      {
        if (At3d_Clone_Mesh_Get_Name(i, name) > 0)
        {
          CloneMesh_List->Items->Add(name);
        }
      }
    }

    if (newItemCount > oldItemCount)
    {
      for (Int32 i=oldItemCount; i<newItemCount; i++)
      {
        if (At3d_Clone_Item_Get_Name(i, name) > 0)
        {
          CloneItem_List->Items->Add(name);
        }
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SaveClonesFileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (clonesFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(clonesFilename, MAX_PATH, "%s%sClones-%s.txt", drive0, dir0, name0);
  }

  if (clonesFilename[0] != '\0')
  {
    my_splitpath(clonesFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Clones";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TSaveDialog* dlg;
  try
  {
    dlg = new TSaveDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofNoReadOnlyReturn << ofPathMustExist << ofOverwritePrompt;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(clonesFilename, MAX_PATH, fileName.c_str());


    At3d_Clones_Save_File(clonesFilename);
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayClonesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_DISPLAY_CLONES, (DisplayClones->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Reload_Modified_OGL_TexturesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_Reload_Modified_OGL_Textures();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_SphereMarker_Location_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    Add_SphereMarker_LocationX->Text = xyzArray1[0];
    Add_SphereMarker_LocationY->Text = xyzArray1[1];
    Add_SphereMarker_LocationZ->Text = xyzArray1[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_SphereMarker_Location_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    Add_SphereMarker_LocationX->Text = xyzArray2[0];
    Add_SphereMarker_LocationY->Text = xyzArray2[1];
    Add_SphereMarker_LocationZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_SphereMarker_Color1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_SPHERE_MARKER1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_SphereMarker_Color2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_SPHERE_MARKER2);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_SphereMarker_AddClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, Add_SphereMarker_Name->Text);

  char nameBuffer[LEN_LAYER_NAME];
  my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  nameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(nameBuffer);
  Add_SphereMarker_Name->Text = nameBuffer;

  // --

  double location[3];

  try
  {
    location[0] = Add_SphereMarker_LocationX->Text.ToDouble();
  }
  catch(...)
  {
    location[0] = 0.0;
  }

  try
  {
    location[1] = Add_SphereMarker_LocationY->Text.ToDouble();
  }
  catch(...)
  {
    location[1] = 0.0;
  }

  try
  {
    location[2] = Add_SphereMarker_LocationZ->Text.ToDouble();
  }
  catch(...)
  {
    location[2] = 0.0;
  }

  // --

  double diameter;

  try
  {
    diameter = Add_SphereMarker_Diameter->Text.ToDouble();
  }
  catch(...)
  {
    diameter = 1.0;
  }

  // --

  Int32 interval;

  try
  {
    interval = Add_SphereMarker_Interval->Text.ToInt();
  }
  catch(...)
  {
    interval = 1000;
  }

  // --

  Int32 color2Flag  = (Add_SphereMarker_Use_Color2->Checked) ? 1 : 0;

  Int32 strobeFlag  = (Add_SphereMarker_Strobe->Checked) ? 1 : 0;

  Int32 visibleFlag = (Add_SphereMarker_Visible->Checked) ? 1 : 0;

  // --

  if (*nameBuffer != '\0')
  {
    if (At3d_Sphere_Marker_Add(nameBuffer, diameter, &location[0],
                               &rgbSphereMarkerArray1[0], &rgbSphereMarkerArray2[0],
                               color2Flag, interval, strobeFlag, visibleFlag))
    {
      SphereMarkers_List->Items->Add(nameBuffer);

      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplaySphereMarkersClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_DISPLAY_SPHERE_MARKERS,
                 (DisplaySphereMarkers->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Sphere_Marker_Data(char* namePtr)
{
  if ( (!dllOpen) ||
       (!namePtr) )
  {
    return;
  }

  // --

  double diameter;
  double location[3];
  Int32 color2Flag;
  Int32 interval;
  Int32 strobeFlag;
  Int32 visibleFlag;

  if ( (At3d_Sphere_Marker_Get_Data(namePtr, &diameter, &location[0],
                                    &rgbSphereMarkerArray3[0],
                                    &rgbSphereMarkerArray4[0], &color2Flag,
                                    &interval, &strobeFlag, &visibleFlag)) == 1 )
  {
    Data_SphereMarker_Name->Text = namePtr;

    Data_SphereMarker_Diameter->Text = diameter;

    Data_SphereMarker_LocationX->Text = location[0];
    Data_SphereMarker_LocationY->Text = location[1];
    Data_SphereMarker_LocationZ->Text = location[2];

    Do_RGB_Panel(COLOR_PANEL_SPHERE_MARKER3);
    Do_RGB_Panel(COLOR_PANEL_SPHERE_MARKER4);

    Data_SphereMarker_Use_Color2->Checked = (color2Flag) ? 1 : 0;

    Data_SphereMarker_Interval->Text = interval;

    Data_SphereMarker_Strobe->Checked = (strobeFlag) ? 1 : 0;

    Data_SphereMarker_Visible->Checked = (visibleFlag) ? 1 : 0;

    Change_SphereMarker_Name->Text = namePtr;
  }
  else
  {
    Data_SphereMarker_Name->Text = "";

    Data_SphereMarker_LocationX->Text = 0.0;
    Data_SphereMarker_LocationY->Text = 0.0;
    Data_SphereMarker_LocationZ->Text = 0.0;

    Data_SphereMarker_Diameter->Text = 1.0;

    Data_SphereMarker_Use_Color2->Checked = 0;

    Data_SphereMarker_Interval->Text = 1000;

    Data_SphereMarker_Strobe->Checked = 0;

    Data_SphereMarker_Visible->Checked = 0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Get_DataClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Name->Text);

  char nameBuffer[LEN_LAYER_NAME];
  my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  nameBuffer[LEN_LAYER_NAME - 1] = '\0';

  double diameter;
  double location[3];
  Int32 color2Flag;
  Int32 interval;
  Int32 strobeFlag;
  Int32 visibleFlag;

  if ( (At3d_Sphere_Marker_Get_Data(nameBuffer, &diameter, &location[0],
                                    &rgbSphereMarkerArray5[0],
                                    &rgbSphereMarkerArray6[0], &color2Flag,
                                    &interval, &strobeFlag, &visibleFlag)) == 1 )
  {
    Change_SphereMarker_Diameter->Text = diameter;

    Change_SphereMarker_LocationX->Text = location[0];
    Change_SphereMarker_LocationY->Text = location[1];
    Change_SphereMarker_LocationZ->Text = location[2];

    Do_RGB_Panel(COLOR_PANEL_SPHERE_MARKER5);
    Do_RGB_Panel(COLOR_PANEL_SPHERE_MARKER6);

    Change_SphereMarker_Use_Color2->Checked = (color2Flag) ? 1 : 0;

    Change_SphereMarker_Interval->Text = interval;

    Change_SphereMarker_Strobe->Checked = (strobeFlag) ? 1 : 0;

    Change_SphereMarker_Visible->Checked = (visibleFlag) ? 1 : 0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SphereMarkers_ListClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = SphereMarkers_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, SphereMarkers_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';
  }

  Display_Sphere_Marker_Data(nameBuffer);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Location_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    Change_SphereMarker_LocationX->Text = xyzArray1[0];
    Change_SphereMarker_LocationY->Text = xyzArray1[1];
    Change_SphereMarker_LocationZ->Text = xyzArray1[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::_SphereMarker_Location_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    Change_SphereMarker_LocationX->Text = xyzArray2[0];
    Change_SphereMarker_LocationY->Text = xyzArray2[1];
    Change_SphereMarker_LocationZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Location_DoClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Name->Text);

  char nameBuffer[LEN_LAYER_NAME];
  my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  nameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(nameBuffer);
  Change_SphereMarker_Name->Text = nameBuffer;

  // --

  double location[3];

  try
  {
    location[0] = Change_SphereMarker_LocationX->Text.ToDouble();
  }
  catch(...)
  {
    location[0] = 0.0;
  }

  try
  {
    location[1] = Change_SphereMarker_LocationY->Text.ToDouble();
  }
  catch(...)
  {
    location[1] = 0.0;
  }

  try
  {
    location[2] = Change_SphereMarker_LocationZ->Text.ToDouble();
  }
  catch(...)
  {
    location[2] = 0.0;
  }

  // --

  Int32 relativeFlag = (Change_SphereMarker_Relative->Checked) ? 1 : 0;

  // --

  if ( (At3d_Sphere_Marker_Set_Location(nameBuffer, &location[0], relativeFlag)) == 1 )
  {
    Display_Sphere_Marker_Data(nameBuffer);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Color1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_SPHERE_MARKER5);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Color2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_SPHERE_MARKER6);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Color_OneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Color(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Color_GroupClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Color(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Color_AllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Color(2);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_Change_SphereMarker_Color(Int32 groupFlag)
{
  if (!dllOpen) return;

  // --

  AnsiString tempName;
  if (groupFlag)
  {
    my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Prefix->Text);
  }
  else
  {
    my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Name->Text);
  }

  char nameBuffer[LEN_LAYER_NAME];
  my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  nameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(nameBuffer);
  Change_SphereMarker_Name->Text = nameBuffer;

  // --

  if (groupFlag == 2)
  {
    if ( (At3d_Sphere_Marker_Set_Color_All(&rgbSphereMarkerArray5[0],
                                           &rgbSphereMarkerArray6[0])) == 1 )
    {
      nameBuffer[0] = '\0';
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
  else if (groupFlag == 1)
  {
    if ( (At3d_Sphere_Marker_Set_Color_Group(nameBuffer, &rgbSphereMarkerArray5[0],
                                             &rgbSphereMarkerArray6[0])) == 1 )
    {
      nameBuffer[0] = '\0';
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
  else
  {
    if ( (At3d_Sphere_Marker_Set_Color_One(nameBuffer, &rgbSphereMarkerArray5[0],
                                           &rgbSphereMarkerArray6[0])) == 1 )
    {
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Diameter_OneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Diameter(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Diameter_GroupClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Diameter(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Diameter_AllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Diameter(2);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_Change_SphereMarker_Diameter(Int32 groupFlag)
{
  if (!dllOpen) return;

  // --

  AnsiString tempName;
  if (groupFlag)
  {
    my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Prefix->Text);
  }
  else
  {
    my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Name->Text);
  }

  char nameBuffer[LEN_LAYER_NAME];
  my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  nameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(nameBuffer);
  Change_SphereMarker_Name->Text = nameBuffer;

  // --

  double diameter;

  try
  {
    diameter = Change_SphereMarker_Diameter->Text.ToDouble();
  }
  catch(...)
  {
    diameter = 1.0;
  }

  // --

  if (groupFlag == 2)
  {
    if ( (At3d_Sphere_Marker_Set_Diameter_All(diameter)) == 1 )
    {
      nameBuffer[0] = '\0';
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
  else if (groupFlag == 1)
  {
    if ( (At3d_Sphere_Marker_Set_Diameter_Group(nameBuffer, diameter)) == 1 )
    {
      nameBuffer[0] = '\0';
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
  else
  {
    if ( (At3d_Sphere_Marker_Set_Diameter_One(nameBuffer, diameter)) == 1 )
    {
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Strobe_OneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Strobe(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Strobe_GroupClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Strobe(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Strobe_AllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Strobe(2);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_Change_SphereMarker_Strobe(Int32 groupFlag)
{
  if (!dllOpen) return;

  // --

  AnsiString tempName;
  if (groupFlag)
  {
    my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Prefix->Text);
  }
  else
  {
    my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Name->Text);
  }

  char nameBuffer[LEN_LAYER_NAME];
  my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  nameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(nameBuffer);
  Change_SphereMarker_Name->Text = nameBuffer;

  // --

  Int32 interval;

  try
  {
    interval = Change_SphereMarker_Interval->Text.ToInt();
  }
  catch(...)
  {
    interval = 1000;
  }

  // --

  Int32 strobeFlag = (Change_SphereMarker_Strobe->Checked) ? 1 : 0;

  // --

  if (groupFlag == 2)
  {
    if ( (At3d_Sphere_Marker_Set_Strobe_All(interval, strobeFlag)) == 1 )
    {
      nameBuffer[0] = '\0';
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
  else if (groupFlag == 1)
  {
    if ( (At3d_Sphere_Marker_Set_Strobe_Group(nameBuffer, interval, strobeFlag)) == 1 )
    {
      nameBuffer[0] = '\0';
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
  else
  {
    if ( (At3d_Sphere_Marker_Set_Strobe_One(nameBuffer, interval, strobeFlag)) == 1 )
    {
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Visible_OneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Visible(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Visible_GroupClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Visible(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Visible_AllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Do_Change_SphereMarker_Visible(2);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_Change_SphereMarker_Visible(Int32 groupFlag)
{
  if (!dllOpen) return;

  // --

  AnsiString tempName;
  if (groupFlag)
  {
    my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Prefix->Text);
  }
  else
  {
    my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Name->Text);
  }

  char nameBuffer[LEN_LAYER_NAME];
  my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  nameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(nameBuffer);
  Change_SphereMarker_Name->Text = nameBuffer;

  // --

  Int32 visibleFlag = (Change_SphereMarker_Visible->Checked) ? 1 : 0;

  // --

  if (groupFlag == 2)
  {
    if ( (At3d_Sphere_Marker_Set_Visible_All(visibleFlag)) == 1 )
    {
      nameBuffer[0] = '\0';
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
  else if (groupFlag == 1)
  {
    if ( (At3d_Sphere_Marker_Set_Visible_Group(nameBuffer, visibleFlag)) == 1 )
    {
      nameBuffer[0] = '\0';
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
  else
  {
    if ( (At3d_Sphere_Marker_Set_Visible_One(nameBuffer, visibleFlag)) == 1 )
    {
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Delete_OneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Delete(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Delete_GroupClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Delete(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Delete_AllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Do_Change_SphereMarker_Delete(2);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_Change_SphereMarker_Delete(Int32 groupFlag)
{
  if (!dllOpen) return;

  // --

  AnsiString tempName;
  if (groupFlag)
  {
    my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Prefix->Text);
  }
  else
  {
    my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Name->Text);
  }

  char nameBuffer[LEN_LAYER_NAME];
  my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  nameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(nameBuffer);
  Change_SphereMarker_Name->Text = nameBuffer;

  // --

  if (groupFlag == 2)
  {
    if ( (At3d_Sphere_Marker_Delete_All()) == 1 )
    {
      if ( (At3d_Sphere_Marker_Get_Count()) > 0)
      {
        Check_SphereMarkers_List();
      }
      else
      {
        SphereMarkers_List->Clear();
      }
    }
  }
  else if (groupFlag == 1)
  {
    if ( (At3d_Sphere_Marker_Delete_Group(nameBuffer)) == 1 )
    {
      Check_SphereMarkers_List();
      nameBuffer[0] = '\0';
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
  else
  {
    if ( (At3d_Sphere_Marker_Delete_One(nameBuffer)) == 1 )
    {
      Check_SphereMarkers_List();
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Check_SphereMarkers_List(void)
{
  if ( (!dllOpen) ||
       (!SphereMarkers_List->Count) )
  {
    return;
  }

  // --

  Int32 iStart = SphereMarkers_List->Count - 1;

  for (Int32 i=iStart; i>=0; i--)
  {
    char nameBuffer[LEN_LAYER_NAME];

    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, SphereMarkers_List->Items->Strings[i]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    // function return value
    //  1 = exists
    //  0 = error
    // -1 = does not exist
    //
    if ( (At3d_Sphere_Marker_Exists(nameBuffer)) == -1 )
    {
      SphereMarkers_List->Items->Delete(i);
    }
  }

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_One_All_ParametersClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Name->Text);

  char nameBuffer[LEN_LAYER_NAME];
  my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  nameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(nameBuffer);
  Change_SphereMarker_Name->Text = nameBuffer;

  // --

  double location[3];

  try
  {
    location[0] = Change_SphereMarker_LocationX->Text.ToDouble();
  }
  catch(...)
  {
    location[0] = 0.0;
  }

  try
  {
    location[1] = Change_SphereMarker_LocationY->Text.ToDouble();
  }
  catch(...)
  {
    location[1] = 0.0;
  }

  try
  {
    location[2] = Change_SphereMarker_LocationZ->Text.ToDouble();
  }
  catch(...)
  {
    location[2] = 0.0;
  }

  // --

  double diameter;

  try
  {
    diameter = Change_SphereMarker_Diameter->Text.ToDouble();
  }
  catch(...)
  {
    diameter = 1.0;
  }

  // --

  Int32 interval;

  try
  {
    interval = Change_SphereMarker_Interval->Text.ToInt();
  }
  catch(...)
  {
    interval = 1000;
  }

  // --

  Int32 color2Flag  = (Change_SphereMarker_Use_Color2->Checked) ? 1 : 0;


  Int32 strobeFlag  = (Change_SphereMarker_Strobe->Checked) ? 1 : 0;

  // --

  Int32 visibleFlag = (Change_SphereMarker_Visible->Checked) ? 1 : 0;

  // --

  if ( (At3d_Sphere_Marker_Change(nameBuffer, diameter, &location[0],
                                  &rgbSphereMarkerArray5[0], &rgbSphereMarkerArray6[0],
                                  color2Flag, interval, strobeFlag,
                                  visibleFlag)) == 1 )
  {
    Display_Sphere_Marker_Data(nameBuffer);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::StrobeSphereMarkersClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_STROBE_SPHERE_MARKERS,
                 (StrobeSphereMarkers->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read_SphereMarkers_FileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (sphereMarkersFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(sphereMarkersFilename, MAX_PATH, "%s%sSphere_Markers-%s.txt", drive0, dir0, name0);
  }

  if (sphereMarkersFilename[0] != '\0')
  {
    my_splitpath(sphereMarkersFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Sphere_Markers";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  Int32 result = 0;
  Int32 oldCount = At3d_Sphere_Marker_Get_Count();

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(sphereMarkersFilename, MAX_PATH, fileName.c_str());


    result = At3d_Sphere_Markers_Read_File(sphereMarkersFilename);
  }

  delete dlg;

  if (result)
  {
    Int32 newCount = At3d_Sphere_Marker_Get_Count();

    char name[LEN_LAYER_NAME];

    if (newCount > oldCount)
    {
      for (Int32 i=oldCount; i<newCount; i++)
      {
        if (At3d_Sphere_Marker_Get_Name(i, name) > 0)
        {
          SphereMarkers_List->Items->Add(name);
        }
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Save_SphereMarkers_FileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) ||
       (!At3d_Sphere_Marker_Get_Count()) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (sphereMarkersFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(sphereMarkersFilename, MAX_PATH, "%s%sSphere_Markers-%s.txt", drive0, dir0, name0);
  }

  if (sphereMarkersFilename[0] != '\0')
  {
    my_splitpath(sphereMarkersFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Sphere_Markers";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TSaveDialog* dlg;
  try
  {
    dlg = new TSaveDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofNoReadOnlyReturn << ofPathMustExist << ofOverwritePrompt;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(sphereMarkersFilename, MAX_PATH, fileName.c_str());


    At3d_Sphere_Markers_Save_File(sphereMarkersFilename);
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Use_Color2_OneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Color2Flag(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Use_Color2_GroupClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Color2Flag(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Change_SphereMarker_Use_Color2_AllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_Change_SphereMarker_Color2Flag(2);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_Change_SphereMarker_Color2Flag(Int32 groupFlag)
{
  if (!dllOpen) return;

  // --

  AnsiString tempName;
  if (groupFlag)
  {
    my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Prefix->Text);
  }
  else
  {
    my_copystring_to_ansi_from_vcl(&tempName, Change_SphereMarker_Name->Text);
  }

  char nameBuffer[LEN_LAYER_NAME];
  my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  nameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(nameBuffer);
  Change_SphereMarker_Name->Text = nameBuffer;

  // --

  Int32 color2Flag = (Change_SphereMarker_Use_Color2->Checked) ? 1 : 0;

  // --

  if (groupFlag == 2)
  {
    if ( (At3d_Sphere_Marker_Set_Color2Flag_All(color2Flag)) == 1 )
    {
      nameBuffer[0] = '\0';
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
  else if (groupFlag == 1)
  {
    if ( (At3d_Sphere_Marker_Set_Color2Flag_Group(nameBuffer, color2Flag)) == 1 )
    {
      nameBuffer[0] = '\0';
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
  else
  {
    if ( (At3d_Sphere_Marker_Set_Color2Flag_One(nameBuffer, color2Flag)) == 1 )
    {
      Display_Sphere_Marker_Data(nameBuffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Selected_SphereMarker_InvertClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_INVERT_COLOR_SELECTED_SPHERE_MARKER,
                 (Selected_SphereMarker_Invert->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Selected_SphereMarker_ColorClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_SPHERE_MARKER7);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Selected_SphereMarker_Set_ColorClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_Sphere_Marker_Set_Selected_Color(&rgbSphereMarkerArray7[0]);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Selected_SphereMarker_CancelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_Sphere_Marker_Cancel_Selected();
  Selected_SphereMarker_Name->Text = AnsiString();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Timer_SphereMarker_SetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 interval;

  try
  {
    interval = Timer_SphereMarker_Interval->Text.ToInt();
  }
  catch(...)
  {
    interval = 1000;
  }

  At3d_Sphere_Marker_Set_Timer(interval);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read_Billboards_FileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (billboardsFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(billboardsFilename, MAX_PATH, "%s%sBillboards-%s.txt", drive0, dir0, name0);
  }

  if (billboardsFilename[0] != '\0')
  {
    my_splitpath(billboardsFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Billboards";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  Int32 result = 0;
  Int32 oldCount = At3d_Measure_Billboard_Get_Count();

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(billboardsFilename, MAX_PATH, fileName.c_str());


    result = At3d_Measure_Billboards_Read_File(billboardsFilename);
  }

  delete dlg;

  if (result)
  {
    Int32 newCount = At3d_Measure_Billboard_Get_Count();

    char name[LEN_LAYER_NAME];

    if (newCount > oldCount)
    {
      for (Int32 i=oldCount; i<newCount; i++)
      {
        if (At3d_Measure_Billboard_Get_Name(i, name) > 0)
        {
          Billboard_ID_List->Items->Add(name);
        }
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Save_Billboards_FileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) ||
       (!At3d_Measure_Billboard_Get_Count()) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (billboardsFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(billboardsFilename, MAX_PATH, "%s%sBillboards-%s.txt", drive0, dir0, name0);
  }

  if (billboardsFilename[0] != '\0')
  {
    my_splitpath(billboardsFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Billboards";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TSaveDialog* dlg;
  try
  {
    dlg = new TSaveDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofNoReadOnlyReturn << ofPathMustExist << ofOverwritePrompt;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(billboardsFilename, MAX_PATH, fileName.c_str());


    At3d_Measure_Billboards_Save_File(billboardsFilename);
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read_Measurements_Project_FileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (projectFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(projectFilename, MAX_PATH, "%s%sMeasure_Project-%s.txt", drive0, dir0, name0);
  }

  if (projectFilename[0] != '\0')
  {
    my_splitpath(projectFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Measure_Project";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  Int32 result = 0;
  Int32 oldMeasureCount      = At3d_MeasureList_Get_Count();
  Int32 oldBillboardCount    = At3d_Measure_Billboard_Get_Count();
  Int32 oldCameraCount       = At3d_Roaming_Camera_Count();
  Int32 oldMeshCount         = At3d_Clone_Mesh_Get_Count();
  Int32 oldItemCount         = At3d_Clone_Item_Get_Count();
  Int32 oldSphereMarkerCount = At3d_Sphere_Marker_Get_Count();

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(projectFilename, MAX_PATH, fileName.c_str());

    Int32 skipDefaultCameraFlag = (Project_Skip_Default_Camera->Checked) ? 1 : 0;

    Int32 skipManyViewsFlag = (Project_Skip_Many_Views->Checked) ? 1 : 0;

    result = At3d_Measurements_Project_Read_File(projectFilename,
                                                 skipDefaultCameraFlag,
                                                 skipManyViewsFlag);
  }

  delete dlg;

  if (result)
  {
    Int32 newMeasureCount      = At3d_MeasureList_Get_Count();
    Int32 newBillboardCount    = At3d_Measure_Billboard_Get_Count();
    Int32 newCameraCount       = At3d_Roaming_Camera_Count();
    Int32 newMeshCount         = At3d_Clone_Mesh_Get_Count();
    Int32 newItemCount         = At3d_Clone_Item_Get_Count();
    Int32 newSphereMarkerCount = At3d_Sphere_Marker_Get_Count();

    char name[LEN_LAYER_NAME];

    if (newMeasureCount > oldMeasureCount)
    {
      for (Int32 i=oldMeasureCount; i<newMeasureCount; i++)
      {
        if (At3d_MeasureList_Get_Name(i, name) > 0)
        {
          Measure_ID_List->Items->Add(name);
        }
      }
    }

    if (newBillboardCount > oldBillboardCount)
    {
      for (Int32 i=oldBillboardCount; i<newBillboardCount; i++)
      {
        if (At3d_Measure_Billboard_Get_Name(i, name) > 0)
        {
          Billboard_ID_List->Items->Add(name);
        }
      }
    }

    if (newCameraCount > oldCameraCount)
    {
      Set_Camera_Lists();
    }

    Many_Views_Get_NamesClick(NULL);

    if (newMeshCount > oldMeshCount)
    {
      for (Int32 i=oldMeshCount; i<newMeshCount; i++)
      {
        if (At3d_Clone_Mesh_Get_Name(i, name) > 0)
        {
          CloneMesh_List->Items->Add(name);
        }
      }
    }

    if (newItemCount > oldItemCount)
    {
      for (Int32 i=oldItemCount; i<newItemCount; i++)
      {
        if (At3d_Clone_Item_Get_Name(i, name) > 0)
        {
          CloneItem_List->Items->Add(name);
        }
      }
    }

    if (newSphereMarkerCount > oldSphereMarkerCount)
    {
      for (Int32 i=oldSphereMarkerCount; i<newSphereMarkerCount; i++)
      {
        if (At3d_Sphere_Marker_Get_Name(i, name) > 0)
        {
          SphereMarkers_List->Items->Add(name);
        }
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Save_Measurements_Project_FileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (projectFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(projectFilename, MAX_PATH, "%s%sMeasure_Project-%s.txt", drive0, dir0, name0);
  }

  if (projectFilename[0] != '\0')
  {
    my_splitpath(projectFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Measure_Project";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TSaveDialog* dlg;
  try
  {
    dlg = new TSaveDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofNoReadOnlyReturn << ofPathMustExist << ofOverwritePrompt;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(projectFilename, MAX_PATH, fileName.c_str());

    Int32 measurementsFlag   = (Project_Measurements->Checked) ? 1 : 0;
    Int32 billboardsFlag     = (Project_Billboards->Checked) ? 1 : 0;
    Int32 camerasFlag        = (Project_Cameras->Checked) ? 1 : 0;
    Int32 manyViewsFlag      = (Project_Many_Views->Checked) ? 1 : 0;
    Int32 clonesFlag         = (Project_Clones->Checked) ? 1 : 0;
    Int32 sphereMarkersFlag  = (Project_Sphere_Markers->Checked) ? 1 : 0;

    At3d_Measurements_Project_Save_File(projectFilename, measurementsFlag,
                                        billboardsFlag, camerasFlag,
                                        manyViewsFlag, clonesFlag,
                                        sphereMarkersFlag);
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Many_Views_Setup(void)
{
  // many views type names
  //
  manyViewsTypeNamesPtr[MANY_VIEWS_TYPE_DEFAULT]          = "Default";
  manyViewsTypeNamesPtr[MANY_VIEWS_TYPE_MINI]             = "Mini";
  manyViewsTypeNamesPtr[MANY_VIEWS_TYPE_TWO_UPPER_LOWER]  = "Two-Upper_Lower";
  manyViewsTypeNamesPtr[MANY_VIEWS_TYPE_TWO_SIDE_BY_SIDE] = "Two-Side_By_Side";
  manyViewsTypeNamesPtr[MANY_VIEWS_TYPE_THREE_1_2]        = "Three-1_2";
  manyViewsTypeNamesPtr[MANY_VIEWS_TYPE_FOUR_1_3]         = "Four-1_3";
  manyViewsTypeNamesPtr[MANY_VIEWS_TYPE_FOUR_2_2]         = "Four-2_2";
  manyViewsTypeNamesPtr[MANY_VIEWS_TYPE_SIX_3_3]          = "Six-3_3";
  manyViewsTypeNamesPtr[MANY_VIEWS_TYPE_NINE_3_3_3]       = "Nine-3_3_3";

  // --

  // don't add "Default"
  //
  for (Int32 i=1; i<MANY_VIEWS_TYPE_TOTAL; i++)
  {
    Many_Views_Add_Type->Items->Add(manyViewsTypeNamesPtr[i]);
  }

  Many_Views_Add_Type->ItemIndex = 0;

  Many_Views_ID_List->Items->Add("Default");
  Many_Views_ID_List->ItemIndex = 0;

  PanesPageControl->TabIndex = TAB_PAGE_DEFAULT;
  active_TabPageIndex = TAB_PAGE_DEFAULT;

  Many_Views_Data_Name->Text = "Default";

  // --

  // reference camera comboboxes from an array
  //
  cameraComboboxPtr[INDEX_MINI_TOP_LEFT]     = Mini_Top_Left_CameraName;
  cameraComboboxPtr[INDEX_MINI_TOP_RIGHT]    = Mini_Top_Right_CameraName;
  cameraComboboxPtr[INDEX_MINI_BOTTOM_LEFT]  = Mini_Bottom_Left_CameraName;
  cameraComboboxPtr[INDEX_MINI_BOTTOM_RIGHT] = Mini_Bottom_Right_CameraName;
  //
  cameraComboboxPtr[INDEX_TWO_UL_TOP]    = Two_UL_Top_CameraName;
  cameraComboboxPtr[INDEX_TWO_UL_BOTTOM] = Two_UL_Bottom_CameraName;
  //
  cameraComboboxPtr[INDEX_TWO_LR_LEFT]  = Two_LR_Left_CameraName;
  cameraComboboxPtr[INDEX_TWO_LR_RIGHT] = Two_LR_Right_CameraName;
  //
  cameraComboboxPtr[INDEX_THREE_1_2_TOP]          = Three_1_2_Top_CameraName;
  cameraComboboxPtr[INDEX_THREE_1_2_BOTTOM_LEFT]  = Three_1_2_Bottom_Left_CameraName;
  cameraComboboxPtr[INDEX_THREE_1_2_BOTTOM_RIGHT] = Three_1_2_Bottom_Right_CameraName;
  //
  cameraComboboxPtr[INDEX_FOUR_1_3_TOP]           = Four_1_3_Top_CameraName;
  cameraComboboxPtr[INDEX_FOUR_1_3_BOTTOM_LEFT]   = Four_1_3_Bottom_Left_CameraName;
  cameraComboboxPtr[INDEX_FOUR_1_3_BOTTOM_CENTER] = Four_1_3_Bottom_Center_CameraName;
  cameraComboboxPtr[INDEX_FOUR_1_3_BOTTOM_RIGHT]  = Four_1_3_Bottom_Right_CameraName;
  //
  cameraComboboxPtr[INDEX_FOUR_2_2_TOP_LEFT]     = Four_2_2_Top_Left_CameraName;
  cameraComboboxPtr[INDEX_FOUR_2_2_TOP_RIGHT]    = Four_2_2_Top_Right_CameraName;
  cameraComboboxPtr[INDEX_FOUR_2_2_BOTTOM_LEFT]  = Four_2_2_Bottom_Left_CameraName;
  cameraComboboxPtr[INDEX_FOUR_2_2_BOTTOM_RIGHT] = Four_2_2_Bottom_Right_CameraName;
  //
  cameraComboboxPtr[INDEX_SIX_3_3_TOP_LEFT]      = Six_3_3_Top_Left_CameraName;
  cameraComboboxPtr[INDEX_SIX_3_3_TOP_CENTER]    = Six_3_3_Top_Center_CameraName;
  cameraComboboxPtr[INDEX_SIX_3_3_TOP_RIGHT]     = Six_3_3_Top_Right_CameraName;
  cameraComboboxPtr[INDEX_SIX_3_3_BOTTOM_LEFT]   = Six_3_3_Bottom_Left_CameraName;
  cameraComboboxPtr[INDEX_SIX_3_3_BOTTOM_CENTER] = Six_3_3_Bottom_Center_CameraName;
  cameraComboboxPtr[INDEX_SIX_3_3_BOTTOM_RIGHT]  = Six_3_3_Bottom_Right_CameraName;
  //
  cameraComboboxPtr[INDEX_NINE_3_3_3_TOP_LEFT]      = Nine_3_3_3_Top_Left_CameraName;
  cameraComboboxPtr[INDEX_NINE_3_3_3_TOP_CENTER]    = Nine_3_3_3_Top_Center_CameraName;
  cameraComboboxPtr[INDEX_NINE_3_3_3_TOP_RIGHT]     = Nine_3_3_3_Top_Right_CameraName;
  cameraComboboxPtr[INDEX_NINE_3_3_3_CENTER_LEFT]   = Nine_3_3_3_Center_Left_CameraName;
  cameraComboboxPtr[INDEX_NINE_3_3_3_CENTER_CENTER] = Nine_3_3_3_Center_Center_CameraName;
  cameraComboboxPtr[INDEX_NINE_3_3_3_CENTER_RIGHT]  = Nine_3_3_3_Center_Right_CameraName;
  cameraComboboxPtr[INDEX_NINE_3_3_3_BOTTOM_LEFT]   = Nine_3_3_3_Bottom_Left_CameraName;
  cameraComboboxPtr[INDEX_NINE_3_3_3_BOTTOM_CENTER] = Nine_3_3_3_Bottom_Center_CameraName;
  cameraComboboxPtr[INDEX_NINE_3_3_3_BOTTOM_RIGHT]  = Nine_3_3_3_Bottom_Right_CameraName;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Many_Views_Get_NamesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 viewCount = At3d_Many_Views_Get_Count();

  char name[LEN_LAYER_NAME];

  if (viewCount)
  {
    Many_Views_ID_List->Items->Clear();

    for (Int32 i=0; i<viewCount; i++)
    {
        if (At3d_Many_Views_Get_Name(i, name) > 0)
        {
          Many_Views_ID_List->Items->Add(name);
        }
    }

    Many_Views_ID_List->ItemIndex = 0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Many_Views_ID_ListClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = Many_Views_ID_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Many_Views_ID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    At3d_Many_Views_Make_Active(nameBuffer);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Many_Views_Add_ViewClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, Many_Views_Add_Name->Text);

  my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  nameBuffer[LEN_LAYER_NAME - 1] = '\0';

  Int32 index = Many_Views_Add_Type->ItemIndex;

  if ( (nameBuffer[0] != '\0') &&
       (index > -1) &&
       (index < MANY_VIEWS_TYPE_TOTAL - 1) )
  {
    if (At3d_Many_Views_Add_View(nameBuffer,
                       const_cast<char*>(manyViewsTypeNamesPtr[index + 1])) > 0)
    {
      Many_Views_Get_NamesClick(MY_NULL_PTR);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Many_Views_Free_This_ViewClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = Many_Views_ID_List->ItemIndex;

  char nameBuffer[LEN_LAYER_NAME];
  nameBuffer[0] = '\0';

  if (index > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Many_Views_ID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    if (At3d_Many_Views_Free_One_View(nameBuffer) > 0)
    {
      Many_Views_Get_NamesClick( MY_NULL_PTR );
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Many_Views_Free_All_ViewsClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (At3d_Many_Views_Free_All_Views() > 0)
  {
    Many_Views_Get_NamesClick( MY_NULL_PTR );
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Decals_DisplayClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  At3d_WriteFlag(LIBFLAG_DISPLAY_DECALS, (Decals_Display->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Decals_Read_FileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (decalsFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(decalsFilename, MAX_PATH, "%s%sDecals-%s.txt", drive0, dir0, name0);
  }

  if (decalsFilename[0] != '\0')
  {
    my_splitpath(decalsFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Decals";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  Int32 result = 0;
  Int32 oldCount = At3d_View_Decal_Get_Count();

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(decalsFilename, MAX_PATH, fileName.c_str());


    result = At3d_View_Decals_Read_File(decalsFilename);
  }

  delete dlg;

  if (result)
  {
    Int32 newCount = At3d_View_Decal_Get_Count();

    char name[LEN_LAYER_NAME];

    if (newCount > oldCount)
    {
      for (Int32 i=oldCount; i<newCount; i++)
      {
        if (At3d_View_Decal_Get_Name(i, name) > 0)
        {
          Decals_ID_List->Items->Add(name);
        }
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Decals_Save_FileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) ||
       (!At3d_View_Decal_Get_Count()) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (decalsFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(decalsFilename, MAX_PATH, "%s%sDecals-%s.txt", drive0, dir0, name0);
  }

  if (decalsFilename[0] != '\0')
  {
    my_splitpath(decalsFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Decals";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TSaveDialog* dlg;
  try
  {
    dlg = new TSaveDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofNoReadOnlyReturn << ofPathMustExist << ofOverwritePrompt;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(decalsFilename, MAX_PATH, fileName.c_str());


    At3d_View_Decals_Save_File(decalsFilename);
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Decal_New_Get_Image_FilenameClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  Decal_Get_Image_Filename(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Decal_Get_Image_Filename(Int32 flag)
{
  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  char imageFilename[MAX_PATH];

  if (!flag)
  {
    my_strcpy(imageFilename, MAX_PATH, decalNew_ImageFilename);
  }
  else
  {
    my_strcpy(imageFilename, MAX_PATH, decalSelected_ImageFilename);
  }

  if ( (imageFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(imageFilename, MAX_PATH, "%s%s%s.png", drive0, dir0, name0);
  }

  if (imageFilename[0] != '\0')
  {
    my_splitpath(imageFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".png");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Decal";
  const char* inputExtensionPtr = ".png";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);

    if (!flag)
    {
      my_strcpy(decalNew_ImageFilename, MAX_PATH, fileName.c_str());

      Decal_New_Image_Filename->Text = fileName.c_str();
    }
    else
    {
      my_strcpy(decalSelected_ImageFilename, MAX_PATH, fileName.c_str());

      Decal_Selected_Image_Filename->Text = fileName.c_str();
    }
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Decals_AddClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, Decal_New_Name->Text);

  char newDecalName[LEN_LAYER_NAME];
  my_strncpy(newDecalName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  newDecalName[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(newDecalName);
  Decal_New_Name->Text = newDecalName;

  float newDecalSize;

  try
  {
    newDecalSize = my_double_to_float(Decal_New_Size->Text.ToDouble());
  }
  catch(...)
  {
    newDecalSize = 15.0f;
  }

  if (newDecalSize < 0.001f) newDecalSize = 0.001f;
  if (newDecalSize > 100.0f) newDecalSize = 100.0f;

  // --

  Int32 newDecalUseFrameFlag = (Decal_New_Use_Frame->Checked) ? 1 : 0;

  // --

  float newDecalRotateAngle;

  try
  {
    newDecalRotateAngle = my_double_to_float(Decal_New_Rotate_Angle->Text.ToDouble());
  }
  catch(...)
  {
    newDecalRotateAngle = 0.0f;
  }

  // --

  Int32 newDecalVisibleFlag = (Decal_New_Visible->Checked) ? 1 : 0;

  // --

  Int32 newDecalTypeIndex = Decal_New_Type->ItemIndex;
  if (newDecalTypeIndex < 0) newDecalTypeIndex = 0;

  char newDecalTypeName[LEN_LAYER_NAME];
  my_strncpy(newDecalTypeName, LEN_LAYER_NAME, decalTypeNamesPtr[newDecalTypeIndex], LEN_LAYER_NAME - 1);

  // --

  float newDecalScreenX;

  try
  {
    newDecalScreenX = my_double_to_float(Decal_New_Screen_X->Text.ToDouble());
  }
  catch(...)
  {
    newDecalScreenX = 0.0f;
  }

  if (newDecalScreenX <   0.0f) newDecalScreenX = 0.0f;
  if (newDecalScreenX > 100.0f) newDecalScreenX = 100.0f;

  // --

  float newDecalScreenY;

  try
  {
    newDecalScreenY = my_double_to_float(Decal_New_Screen_Y->Text.ToDouble());
  }
  catch(...)
  {
    newDecalScreenY = 0.0f;
  }

  if (newDecalScreenY <   0.0f) newDecalScreenY = 0.0f;
  if (newDecalScreenY > 100.0f) newDecalScreenY = 100.0f;

  // --

  my_copystring_to_ansi_from_vcl(&tempName, Decal_New_Image_Filename->Text);

  char newDecalImageFilename[MAX_PATH];
  my_strncpy(newDecalImageFilename, MAX_PATH, tempName.c_str(), MAX_PATH - 1);
  newDecalImageFilename[MAX_PATH - 1] = '\0';

  // --

  if (At3d_View_Decal_Add(newDecalName, newDecalImageFilename,
                          newDecalTypeName, newDecalSize,
                          newDecalUseFrameFlag, newDecalRotateAngle,
                          newDecalScreenX, newDecalScreenY, newDecalVisibleFlag) > 0)
  {
    Decals_ID_List->Items->Add(newDecalName);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Decals_ID_ListClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = Decals_ID_List->ItemIndex;

  if (index > -1)
  {
    char nameBuffer[LEN_LAYER_NAME];
    nameBuffer[0] = '\0';

    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Decals_ID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    my_strcpy(decalSelected_Name, LEN_LAYER_NAME, nameBuffer);

    Display_Selected_Decal_Data();
  }
  else
  {
    decalSelected_Name[0] = '\0';
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Selected_Decal_Data(void)
{
  if (!dllOpen) return;

  // --

  char typeName[LEN_LAYER_NAME];
  float size;
  Int32 useFrameFlag;
  float rotateAngle;
  float screenX;
  float screenY;
  Int32 visibleFlag;
  Int32 index = -1;

  if (At3d_View_Decal_Get_Data(decalSelected_Name, decalSelected_ImageFilename,
                                typeName, &size, &useFrameFlag, &rotateAngle,
                                &screenX, &screenY, &visibleFlag) > 0)
  {
    for (Int32 i=0; i<VIEW_DECAL_TYPE_TOTAL; i++)
    {
      if (my_stricmp(decalTypeNamesPtr[i], typeName) == 0)
      {
        index = i;
        break;
      }
    }

    if (index == -1) decalSelected_Name[0] = '\0';
  }
  else
  {
    decalSelected_Name[0] = '\0';
  }

  if (decalSelected_Name[0] == '\0')
  {
    Decal_Selected_Name->Text = "";

    Decal_Selected_Size->Text = 15.0f;

    Decal_Selected_Use_Frame->Checked = 0;

    Decal_Selected_Rotate_Angle->Text = 0.0f;

    Decal_Selected_Visible->Checked = 0;

    Decal_Selected_Type->ItemIndex = -1;

    Decal_Selected_Screen_X->Text = 0.0f;
    Decal_Selected_Screen_Y->Text = 0.0f;

    Decal_Selected_Image_Filename->Text = "";
  }
  else
  {
    Decal_Selected_Name->Text = decalSelected_Name;

    Decal_Selected_Size->Text = size;

    Decal_Selected_Use_Frame->Checked = (useFrameFlag) ? 1 : 0;

    Decal_Selected_Rotate_Angle->Text = rotateAngle;

    Decal_Selected_Visible->Checked = (visibleFlag) ? 1 : 0;

    Decal_Selected_Type->ItemIndex = index;

    Decal_Selected_Screen_X->Text = screenX;
    Decal_Selected_Screen_Y->Text = screenY;

    Decal_Selected_Image_Filename->Text = decalSelected_ImageFilename;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Decals_Free_This_DecalClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Int32 index = Decals_ID_List->ItemIndex;

  if (index > -1)
  {
    char nameBuffer[LEN_LAYER_NAME];
    nameBuffer[0] = '\0';

    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Decals_ID_List->Items->Strings[index]);

    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';

    if (At3d_View_Decal_Free_One(nameBuffer) > 0)
    {
      Decals_ID_List->DeleteSelected();
      if (Decals_ID_List->Count)
      {
        Decals_ID_List->ItemIndex = 0;
      }

      decalSelected_Name[0] = '\0';
      Display_Selected_Decal_Data();
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Decals_Free_AllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  if (At3d_View_Decal_Free_All() > 0)
  {
    Decals_ID_List->Clear();

    decalSelected_Name[0] = '\0';
    Display_Selected_Decal_Data();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Decal_Selected_Get_Image_FilenameClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  Decal_Get_Image_Filename(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Decals_UpdateClick(TObject *Sender)
{
   sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, Decal_Selected_Name->Text);

  char selectedDecalName[LEN_LAYER_NAME];
  my_strncpy(selectedDecalName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  selectedDecalName[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(selectedDecalName);
  Decal_Selected_Name->Text = selectedDecalName;

  float selectedDecalSize;

  try
  {
    selectedDecalSize = my_double_to_float(Decal_Selected_Size->Text.ToDouble());
  }
  catch(...)
  {
    selectedDecalSize = 15.0f;
  }

  if (selectedDecalSize < 0.001f) selectedDecalSize = 0.001f;
  if (selectedDecalSize > 100.0f) selectedDecalSize = 100.0f;

  // --

  Int32 selectedDecalUseFrameFlag = (Decal_Selected_Use_Frame->Checked) ? 1 : 0;

  // --

  float selectedDecalRotateAngle;

  try
  {
    selectedDecalRotateAngle = my_double_to_float(Decal_Selected_Rotate_Angle->Text.ToDouble());
  }
  catch(...)
  {
    selectedDecalRotateAngle = 0.0f;
  }

  // --

  Int32 selectedDecalVisibleFlag = (Decal_Selected_Visible->Checked) ? 1 : 0;

  // --

  Int32 selectedDecalTypeIndex = Decal_Selected_Type->ItemIndex;
  if (selectedDecalTypeIndex < 0) selectedDecalTypeIndex = 0;

  char selectedDecalTypeName[LEN_LAYER_NAME];
  my_strncpy(selectedDecalTypeName, LEN_LAYER_NAME,
             decalTypeNamesPtr[selectedDecalTypeIndex], LEN_LAYER_NAME - 1);

  // --

  float selectedDecalScreenX;

  try
  {
    selectedDecalScreenX = my_double_to_float(Decal_Selected_Screen_X->Text.ToDouble());
  }
  catch(...)
  {
    selectedDecalScreenX = 0.0f;
  }

  if (selectedDecalScreenX <   0.0f) selectedDecalScreenX = 0.0f;
  if (selectedDecalScreenX > 100.0f) selectedDecalScreenX = 100.0f;

  // --

  float selectedDecalScreenY;

  try
  {
    selectedDecalScreenY = my_double_to_float(Decal_Selected_Screen_Y->Text.ToDouble());
  }
  catch(...)
  {
    selectedDecalScreenY = 0.0f;
  }

  if (selectedDecalScreenY <   0.0f) selectedDecalScreenY = 0.0f;
  if (selectedDecalScreenY > 100.0f) selectedDecalScreenY = 100.0f;

  // --

  my_copystring_to_ansi_from_vcl(&tempName, Decal_Selected_Image_Filename->Text);

  char selectedDecalImageFilename[MAX_PATH];
  my_strncpy(selectedDecalImageFilename, MAX_PATH, tempName.c_str(), MAX_PATH - 1);
  selectedDecalImageFilename[MAX_PATH - 1] = '\0';

  // --

  if (At3d_View_Decal_Set_Data(selectedDecalName, selectedDecalImageFilename,
                               selectedDecalTypeName, selectedDecalSize,
                               selectedDecalUseFrameFlag, selectedDecalRotateAngle,
                               selectedDecalScreenX, selectedDecalScreenY,
                               selectedDecalVisibleFlag) > 0)
  {
    //
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Background_Color_BrightClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Set_Background_Color(BACKGROUND_BRIGHT, 184, 212, 255);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Background_Color_DefaultClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Set_Background_Color(BACKGROUND_DEFAULT, 147, 175, 218);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Background_Color_DimClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Set_Background_Color(BACKGROUND_DIM, 4, 32, 75);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Background_Color_DarkClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Set_Background_Color(BACKGROUND_DARK, 0, 0, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Background_Color(Int32 type,
                                             Int32 red, Int32 green, Int32 blue)
{
  if (!dllOpen) return;

  // --

  float rgbArray[3];

  rgbArray[0] = my_change_256_int_to_float(red);
  rgbArray[1] = my_change_256_int_to_float(green);
  rgbArray[2] = my_change_256_int_to_float(blue);

  if (At3d_Light_Background_Set( &rgbArray[0] ) > 0)
  {
    if ( (lightMode > -1) &&
         (lightMode < LIGHT_MODE_COUNT) )
    {
      lightModes[lightMode].backgroundType = type;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Lighting_Mode_ExamineClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Switch_Lighting_Mode(LIGHT_MODE_EXAMINE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Lighting_Mode_Sun_TrackingClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Switch_Lighting_Mode(LIGHT_MODE_SUN_TRACKING);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Lighting_Mode_Night_GogglesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // --

  Switch_Lighting_Mode(LIGHT_MODE_NIGHT_GOGGLES);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Switch_Lighting_Mode(Int32 newMode)
{
  if ( (newMode < 0) ||
       (newMode >= LIGHT_MODE_COUNT) )
  {
    return;
  }

  // --

  lightMode = newMode;

  LIGHT_MODE* lightModePtr = &lightModes[lightMode];

  Global_Ambient_Scroll->Position = static_cast<Int32>(lightModePtr->global_ambientPercent * 2.0f);
  Global_Ambient_Edit->Text = lightModePtr->global_ambientPercent;

  Set_Global_Ambient();

  // --

  Light_Enabled->Checked = (lightModePtr->light0_enabledFlag) ? 1 : 0;

  Light_Rotate_With_Scene->Checked = (lightModePtr->light0_staticFlag) ? 1 : 0;

  Light_Ambient_Scroll->Position = static_cast<Int32>(lightModePtr->light0_ambientPercent * 2.0f);
  Light_Ambient_Edit->Text = lightModePtr->light0_ambientPercent;

  Light_Diffuse_Scroll->Position = static_cast<Int32>(lightModePtr->light0_diffusePercent * 2.0f);
  Light_Diffuse_Edit->Text = lightModePtr->light0_diffusePercent;

  Light_Specular_Scroll->Position = static_cast<Int32>(lightModePtr->light0_specularPercent * 2.0f);
  Light_Specular_Edit->Text = lightModePtr->light0_specularPercent;

  Light_Left_Right_Scroll->Position = static_cast<Int32>(lightModePtr->light0_position[0] * 100.0f);
  Light_Left_Right_Edit->Text = lightModePtr->light0_position[0];

  Light_Down_Up_Scroll->Position = static_cast<Int32>(lightModePtr->light0_position[1] * 100.0f);
  Light_Down_Up_Edit->Text = lightModePtr->light0_position[1];

  Light_In_Out_Scroll->Position = static_cast<Int32>(lightModePtr->light0_position[2] * 100.0f);
  Light_In_Out_Edit->Text = lightModePtr->light0_position[2];

  Set_Light();

  // --                                                       lightModePtr->light0_position[0]

  Int32 type = lightModePtr->backgroundType;

  if (type == BACKGROUND_BRIGHT)
  {
    Background_Color_BrightClick(MY_NULL_PTR);
    Background_Color_Bright->Checked = 1;
  }
  else if (type == BACKGROUND_DIM)
  {
    Background_Color_DimClick(MY_NULL_PTR);
    Background_Color_Dim->Checked = 1;
  }
  else if (type == BACKGROUND_DARK)
  {
    Background_Color_DarkClick(MY_NULL_PTR);
    Background_Color_Dark->Checked = 1;
  }
  else
  {
    Background_Color_DefaultClick(MY_NULL_PTR);
    Background_Color_Default->Checked = 1;
  }

  // --

  At3d_RenderOpenGLScene();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Global_Ambient_ScrollChange(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (lightMode < 0) ||
       (lightMode >= LIGHT_MODE_COUNT) )
  {
    return;
  }

  // --

  LIGHT_MODE* lightModePtr = &lightModes[lightMode];

  lightModePtr->global_ambientPercent = static_cast<float>(Global_Ambient_Scroll->Position) * 0.5f;
  Global_Ambient_Edit->Text = lightModePtr->global_ambientPercent;
  Set_Global_Ambient();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Global_Ambient(void)
{
  if ( (!dllOpen) ||
       (lightMode < 0) ||
       (lightMode >= LIGHT_MODE_COUNT) )
  {
    return;
  }

  // --

  LIGHT_MODE* lightModePtr = &lightModes[lightMode];

  float ambientRGBA[4];

  for (Int32 i=0; i<4; i++)
  {
    if (i == 3)
    {
      ambientRGBA[i] = 1.0f;
    }
    else
    {
      ambientRGBA[i] = lightModePtr->global_ambientRGBA[i]  * lightModePtr->global_ambientPercent  / 100.0f;
    }
  }

  At3d_Light_Global_Set(ambientRGBA);

  At3d_RenderOpenGLScene();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Light(void)
{
  if ( (!dllOpen) ||
       (lightMode < 0) ||
       (lightMode >= LIGHT_MODE_COUNT) )
  {
    return;
  }

  // --

  LIGHT_MODE* lightModePtr = &lightModes[lightMode];

  float ambientRGBA[4];
  float diffuseRGBA[4];
  float specularRGBA[4];

  for (Int32 i=0; i<4; i++)
  {
    if (i == 3)
    {
      ambientRGBA[i]  = 0.0f;
      diffuseRGBA[i]  = 0.0f;
      specularRGBA[i] = 0.0f;
    }
    else
    {
      ambientRGBA[i]  = lightModePtr->light0_ambientRGBA[i]  * lightModePtr->light0_ambientPercent  / 100.0f;
      diffuseRGBA[i]  = lightModePtr->light0_diffuseRGBA[i]  * lightModePtr->light0_diffusePercent  / 100.0f;
      specularRGBA[i] = lightModePtr->light0_specularRGBA[i] * lightModePtr->light0_specularPercent / 100.0f;
    }
  }

  // use light0 for this demo
  //
  At3d_Light_Directional_Enabled_All(0);

  At3d_Light_Directional_Set(0, lightModePtr->light0_enabledFlag,  lightModePtr->light0_staticFlag,
                                ambientRGBA,  diffuseRGBA,
                                specularRGBA, lightModePtr->light0_position);

  At3d_RenderOpenGLScene();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Light_EnabledClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (lightMode < 0) ||
       (lightMode >= LIGHT_MODE_COUNT) )
  {
    return;
  }

  // --

  LIGHT_MODE* lightModePtr = &lightModes[lightMode];

  lightModePtr->light0_enabledFlag = (Light_Enabled->Checked) ? 1 : 0;
  Set_Light();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Light_Rotate_With_SceneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (lightMode < 0) ||
       (lightMode >= LIGHT_MODE_COUNT) )
  {
    return;
  }

  // --

  LIGHT_MODE* lightModePtr = &lightModes[lightMode];

  lightModePtr->light0_staticFlag = (Light_Rotate_With_Scene->Checked) ? 1 : 0;
  Set_Light();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Light_Ambient_ScrollChange(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (lightMode < 0) ||
       (lightMode >= LIGHT_MODE_COUNT) )
  {
    return;
  }

  // --

  LIGHT_MODE* lightModePtr = &lightModes[lightMode];

  lightModePtr->light0_ambientPercent = static_cast<float>(Light_Ambient_Scroll->Position) * 0.5f;
  Light_Ambient_Edit->Text = lightModePtr->light0_ambientPercent;
  Set_Light();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Light_Diffuse_ScrollChange(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (lightMode < 0) ||
       (lightMode >= LIGHT_MODE_COUNT) )
  {
    return;
  }

  // --

  LIGHT_MODE* lightModePtr = &lightModes[lightMode];

  lightModePtr->light0_diffusePercent = static_cast<float>(Light_Diffuse_Scroll->Position) * 0.5f;
  Light_Diffuse_Edit->Text = lightModePtr->light0_diffusePercent;
  Set_Light();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Light_Specular_ScrollChange(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (lightMode < 0) ||
       (lightMode >= LIGHT_MODE_COUNT) )
  {
    return;
  }

  // --

  LIGHT_MODE* lightModePtr = &lightModes[lightMode];

  lightModePtr->light0_specularPercent = static_cast<float>(Light_Specular_Scroll->Position) * 0.5f;
  Light_Specular_Edit->Text = lightModePtr->light0_specularPercent;
  Set_Light();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Light_Left_Right_ScrollChange(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (lightMode < 0) ||
       (lightMode >= LIGHT_MODE_COUNT) )
  {
    return;
  }

  // --

  LIGHT_MODE* lightModePtr = &lightModes[lightMode];

  lightModePtr->light0_position[0] = static_cast<float>(Light_Left_Right_Scroll->Position) * 0.01f;
  Light_Left_Right_Edit->Text = lightModePtr->light0_position[0];
  Set_Light();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Light_Down_Up_ScrollChange(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (lightMode < 0) ||
       (lightMode >= LIGHT_MODE_COUNT) )
  {
    return;
  }

  // --

  LIGHT_MODE* lightModePtr = &lightModes[lightMode];

  lightModePtr->light0_position[1] = static_cast<float>(Light_Down_Up_Scroll->Position) * 0.01f;
  Light_Down_Up_Edit->Text = lightModePtr->light0_position[1];
  Set_Light();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Light_In_Out_ScrollChange(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (lightMode < 0) ||
       (lightMode >= LIGHT_MODE_COUNT) )
  {
    return;
  }

  // --

  LIGHT_MODE* lightModePtr = &lightModes[lightMode];

  lightModePtr->light0_position[2] = static_cast<float>(Light_In_Out_Scroll->Position) * 0.01f;
  Light_In_Out_Edit->Text = lightModePtr->light0_position[2];
  Set_Light();
}
//---------------------------------------------------------------------------

